@@ -1,4 +1,4 @@
-/* $Id: board.cc,v 1.65 2002/04/07 04:49:21 bergo Exp $ */
+/* $Id: board.cc,v 1.74 2002/05/02 00:41:03 bergo Exp $ */
 
 /*
 
@@ -34,145 +34,41 @@
 
 #include "mainwindow.h"
 
-PieceSet * Board::orig=0;
-
-VectorPieces Board::vpieces;
-VectorPieces BareBoard::vpieces;
-
-string Board::BugTell;
-
 Board * Board::PopupOwner = 0;
 
-// --- BareBoard -----------------
-
-BareBoard::BareBoard() : WidgetProxy() {
- myfont=gdk_font_load(global.InfoFont);
- if (!myfont) {
-   cerr << "<BareBoard::BareBoard> can't load font.\n";
-   exit(2);
- }
- strcpy(Names[0],"None");
- strcpy(Names[1],"None");
- 
- widget=gtk_drawing_area_new();
- gtk_widget_set_events(widget,GDK_EXPOSURE_MASK);
- gtk_signal_connect(GTK_OBJECT(widget),"expose_event",
-		    GTK_SIGNAL_FUNC(bareboard_expose),(gpointer)this);
-
- pixbuf=0;
-}
-
-BareBoard::~BareBoard() {
-  gdk_font_unref(myfont);
-  if (pixbuf)
-    gdk_pixmap_unref(pixbuf);
-}
-
-void BareBoard::setPosition(Position *pos) {
-  position = (*pos);
-  repaint();
-}
-
-void BareBoard::setWhite(char *name) {
-  strncpy(Names[0],name,64);
-  repaint();
-}
-
-void BareBoard::setBlack(char *name) {
-  strncpy(Names[1],name,64);
-  repaint();
-}
-  
-void BareBoard::update() {
-  repaint();
-}
-
-void BareBoard::updateClock() {
-  repaint();
-}
-
-gboolean bareboard_expose(GtkWidget *widget,GdkEventExpose *ee,
-			  gpointer data) {
-
-  BareBoard *me;
-  int i,j,ww,wh;
-  int sqside;
-  GdkGC *gc;
-  char z[256],t[64];  
-
-  me=(BareBoard *)data;
-
-  if (! me->pixbuf)
-    me->pixbuf=gdk_pixmap_new(widget->window,SCRAP_WIDTH,SCRAP_WIDTH,-1);
-
-  gdk_window_get_size(widget->window,&ww,&wh);
-  if (ww>SCRAP_WIDTH) ww=SCRAP_WIDTH;
-  if (wh>SCRAP_WIDTH) wh=SCRAP_WIDTH;
-  sqside=ww/8;
-  if (wh<ww) sqside=wh/8;
-
-  gc=gdk_gc_new(me->pixbuf);
-  gdk_rgb_gc_set_foreground(gc,0);
-  gdk_draw_rectangle(me->pixbuf,gc,TRUE,0,0,ww,wh);
-
-  if (sqside > 8) {
-    BareBoard::vpieces.drawSquares(me->pixbuf,gc,sqside);
-    
-    for(i=0;i<8;i++)
-      for(j=0;j<8;j++)
-	BareBoard::vpieces.drawPiece(me->pixbuf,gc,sqside,i*sqside,
-				     j*sqside,
-				     me->position.getPiece(i,7-j));
-
-    gdk_rgb_gc_set_foreground(gc,0xffffff);
-    
-    gdk_draw_string(me->pixbuf,me->myfont,gc,sqside*8+10,20,"Bughouse: Partner Game (not yet working)");
-    me->clockString(me->clock.getValue(0),t);
-    sprintf(z,"White: %s - %s",me->Names[0],t);
-    gdk_draw_string(me->pixbuf,me->myfont,gc,sqside*8+10,40,z);
-    me->clockString(me->clock.getValue(1),t);
-    sprintf(z,"Black: %s - %s",me->Names[1],t);
-    gdk_draw_string(me->pixbuf,me->myfont,gc,sqside*8+10,60,z);
-
-  }
-  gdk_gc_destroy(gc);
-  gc=gdk_gc_new(widget->window);
-  gdk_draw_pixmap(widget->window,gc,me->pixbuf,0,0,0,0,ww,wh);
-  gdk_gc_destroy(gc);
-  return 1;
-}
-
 // --- Board --------------------------
 
-Board::Board(bool withbugpane=false) : WidgetProxy() {
+Board::Board() : WidgetProxy() {
   int i;
-  GtkWidget *tnb=0, *tl1, *tl2;
-  Text *text=0;
+  GtkWidget *tl2;
 
   global.debug("Board","Board");
+  global.create("Board");
+
+  global.addPieceClient(this);
 
   cur=0;
   clock.setHost(this);
 
-  bugboard=0;
   premoveq=0;
   mygame=0;
   clock_ch=1;
   hilite_ch=1;
-  allowMove=1;
+  allowMove=true;
   gotAnimationLoop=0;
-  update_queue=0;
+  update_queue=false;
   LockAnimate=0;
   FreeMove=false;
   dr_active=false;
   FlipInvert=false;
   EditMode=false;
 
-  info[0][0]=info[1][0]=info[2][0]=info[3][0]=info[4][0]=info[5][0]=0;
+  for(i=0;i<6;i++)
+    info[i].erase();
 
   currently_rendered.setPiece(0,0,EMPTY);
 
-  repaint_due=0;
+  repaint_due=false;
   frozen_lvl=0;
 
   f1=global.loadFont(EF_PlayerFont);
@@ -189,53 +85,22 @@ Board::Board(bool withbugpane=false) : WidgetProxy() {
 
   global.BoardList.push_back(this);
 
-  canselect=1;
+  canselect=true;
   sel_color=0xffff00;
   sp=0;
-  flipped=0;
+  flipped=false;
 
-  widget=gtk_vpaned_new();  
-  bugpane=gtk_hbox_new(FALSE,0);
-
-  yidget=gtk_drawing_area_new();
+  widget=yidget=gtk_drawing_area_new();
   gtk_widget_set_events(yidget,GDK_EXPOSURE_MASK|GDK_BUTTON_PRESS_MASK|
 			GDK_BUTTON_RELEASE_MASK|GDK_BUTTON1_MOTION_MASK|
 			GDK_POINTER_MOTION_HINT_MASK);
   buffer=gdk_pixmap_new(MainWindow::RefWindow,SCRAP_WIDTH,SCRAP_WIDTH,-1);
   clkbar=gdk_pixmap_new(MainWindow::RefWindow,250,SCRAP_WIDTH,-1);
+  clkgc=gdk_gc_new(clkbar);
+  wgc=0;
   scrap=(rgbptr)g_malloc(SCRAP_WIDTH*SCRAP_WIDTH*3);
   memset(scrap,0,SCRAP_WIDTH*SCRAP_WIDTH*3);
 
-  if (withbugpane)
-    createBugPane();
-
-  gtk_paned_pack1(GTK_PANED(widget),yidget,TRUE,FALSE);
-
-  if (withbugpane) {
-    tnb=gtk_notebook_new();
-    
-    tl1=gtk_label_new("Console");
-    tl2=gtk_label_new("Bughouse");
-    
-    text=new Text();
-
-    gtk_notebook_append_page(GTK_NOTEBOOK(tnb), text->widget, tl1);
-    gtk_notebook_append_page(GTK_NOTEBOOK(tnb), bugpane, tl2);
-    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tnb), GTK_POS_RIGHT);
-
-    global.outputset->addTarget(text);
-    gtk_paned_pack2(GTK_PANED(widget), tnb, TRUE,TRUE);
-
-  } else {
-
-    gtk_paned_pack2(GTK_PANED(widget),bugpane,TRUE,TRUE);
-
-  }
-
-  gtk_paned_set_position(GTK_PANED(widget),7000);
-  gtk_paned_set_gutter_size(GTK_PANED(widget),withbugpane?8:0);
-  gtk_paned_set_handle_size(GTK_PANED(widget),withbugpane?8:0);
-
   gtk_signal_connect (GTK_OBJECT (yidget), "expose_event",
                       (GtkSignalFunc) board_expose_event, (gpointer) this);
   gtk_signal_connect (GTK_OBJECT (yidget), "configure_event",
@@ -250,16 +115,6 @@ Board::Board(bool withbugpane=false) : WidgetProxy() {
                       (GtkSignalFunc) board_motion_event,
 		      (gpointer) this);
 
-  gtk_widget_show(yidget);
-
-  if (withbugpane) {
-    text->show();
-    gtk_widget_show(tl1);
-    gtk_widget_show(tl2);
-    gtk_widget_show(tnb);
-  }
-
-  gtk_widget_show(bugpane);
   gtk_widget_show(widget);
 }
 
@@ -267,59 +122,11 @@ bool Board::hasGame() {
   return(mygame!=0);
 }
 
-void Board::createBugPane() {
-  static char *stuff[18]={"---","--","-","+","++","+++",
-			  "P","N","B","R","Q","Diag",
-			  "Sit","Go","Fast","Hard","Dead","Safe"};
-
-  GtkWidget *tbl,*v;
-  GtkWidget *qb[18];
-  GtkStyle *style;
-  GdkColor gray[5];
-  int i,c,r;
-  
-  bugboard=new BareBoard();
-  bugboard->show();
-  gtk_box_pack_start(GTK_BOX(bugpane),bugboard->widget,TRUE,TRUE,0);
-
-  v=gtk_vbox_new(FALSE,0);
-  gtk_box_pack_start(GTK_BOX(bugpane),v,FALSE,TRUE,0);
-
-  tbl=gtk_table_new(6,3,FALSE);
-
-  gray[0].red=gray[0].green=gray[0].blue=0xaaaa;
-  gray[1].red=gray[1].green=gray[1].blue=0x8888;
-  gray[2].red=gray[2].green=gray[2].blue=0xcccc;
-  gray[3].red=gray[3].green=gray[3].blue=0x8888;
-  gray[4].red=gray[4].green=gray[4].blue=0xaaaa;
-  style=gtk_style_new();
-  for(i=0;i<5;i++)
-    style->bg[i]=gray[i];
-
-  for(i=0;i<18;i++) {
-
-    qb[i]=gtk_button_new_with_label(stuff[i]);
-    if (i<6)
-      gtk_widget_set_style(qb[i],style);
-
-    c=i/6;
-    r=i%6;
-    gtk_table_attach_defaults(GTK_TABLE(tbl),qb[i],c,c+1,r,r+1);
-    gtk_widget_show(qb[i]);
-    gtk_signal_connect(GTK_OBJECT(qb[i]),"clicked",
-		       GTK_SIGNAL_FUNC(board_ptell),(gpointer)(stuff[i]));
-  }
-
-  gtk_box_pack_start(GTK_BOX(v),tbl,FALSE,FALSE,0);
-  gtk_widget_show(tbl);
-  gtk_widget_show(v);
-}
-
-
 Board::~Board() {
   int i;
   global.debug("Board","~Board");
   global.removeBoard(this);
+  global.removePieceClient(this);
   if (f1)
     gdk_font_unref(f1);
   if (f2)
@@ -331,10 +138,12 @@ Board::~Board() {
   LastMove.clear();
   if (cur)
     delete cur;
+  if (wgc)
+    gdk_gc_destroy(wgc);
+  gdk_gc_destroy(clkgc);
   gdk_pixmap_unref(clkbar);
   gdk_pixmap_unref(buffer);
-  if (bugboard)
-    delete bugboard;
+  global.destroy("Board");
 }
 
 void Board::reloadFonts() {
@@ -354,22 +163,6 @@ void Board::reloadFonts() {
   queueRepaint();
 }
 
-void Board::openBugPane() {
-  int ww,wh;
-  gdk_window_get_size(widget->window,&ww,&wh);
-  ww*=2;
-  ww/=3;
-  gtk_paned_set_position(GTK_PANED(widget),ww);
-}
-
-void Board::closeBugPane() {
-  gtk_paned_set_position(GTK_PANED(widget),7000);
-}
-
-void Board::addBugText(char *text) {
-  
-}
-
 /*
   Resets the board state regarding any previous game played
   here.
@@ -390,7 +183,7 @@ void Board::reset() {
       mygame->setBoard(0);
   }
 
-  update_queue=0;
+  update_queue=false;
 
   while(!UpdateStack.empty())
     UpdateStack.pop();
@@ -398,14 +191,15 @@ void Board::reset() {
   sp=0;
   mygame=0;
   premoveq=0;
-  flipped=0;
+  flipped=false;
   clock_ch=1;
   hilite_ch=1;
-  allowMove=1;
+  allowMove=true;
   dr_active=false;
   LockAnimate=0;
   FlipInvert=false;
-  info[0][0]=info[1][0]=info[2][0]=info[3][0]=info[4][0]=info[5][0]=0;
+  for(i=0;i<6;i++)
+    info[i].erase();
   clock.setMirror(0);
   clock.setClock(0,0,0,1);
   currently_rendered.invalidate();
@@ -414,7 +208,6 @@ void Board::reset() {
     delete(LastMove[i]);
   LastMove.clear();
   cleanTargets();
-  closeBugPane();
   queueRepaint();
 }
 
@@ -430,40 +223,10 @@ bool Board::getFlipInversion() {
   return(FlipInvert);
 }
 
-void Board::setCurrentPieceSet(char *filename, bool chgPieces, bool chgSquares) {
-  list<Board *>::iterator it;
-  char oldp[128],olds[128];
-  PieceSet *oldset=0;
-
-  if (Board::orig) {
-    strcpy(oldp, Board::orig->getName() );
-    strcpy(olds, Board::orig->getSquareName() );
-    oldset=Board::orig;
-  } else {
-    chgPieces=true;
-    chgSquares=true;
-  }
-
-  Board::orig=new PieceSet(chgPieces?filename:oldp,chgSquares?filename:olds);
-  if (oldset)
-    delete oldset;
-
-  for(it=global.BoardList.begin();it!=global.BoardList.end();it++)
-    (*it)->refreshPieceSet();
-}
-
-char * Board::getCurrentPieceSet() {
-  return(Board::orig->getName());
-}
-
-char * Board::getCurrentSquareSet() {
-  return(Board::orig->getSquareName());
-}
-
-void Board::refreshPieceSet() {
-  if (cur)
+void Board::pieceSetChanged() {
+ if (cur)
     delete cur;
-  cur=new PieceSet(orig);
+  cur=new PieceSet(global.pieceset);
   currently_rendered.invalidate();
   if (global.ShowCoordinates) { borx=20; bory=20; } else { borx=bory=0; }
   morey=0;
@@ -488,7 +251,7 @@ void Board::show() {
   // already called on creation
 }
 
-void Board::setSensitive(int state) {
+void Board::setSensitive(bool state) {
   canselect=state;
   if ((!canselect)&&(sp)) {
     sp=0;
@@ -513,8 +276,8 @@ void Board::setSelColor(int color) {
   }
 }
 
-void Board::setFlipped(int flip) {
-  int ov;
+void Board::setFlipped(bool flip) {
+  bool ov;
   ov=flipped;
   flipped=flip;
   if (flipped!=ov) {
@@ -526,9 +289,9 @@ void Board::setFlipped(int flip) {
 
 inline bool Board::effectiveFlip() {
   if (FlipInvert)
-    return(flipped==0);
+    return(!flipped);
   else
-    return(flipped!=0);
+    return(flipped);
 }
 
 void Board::supressNextAnimation() {
@@ -578,7 +341,7 @@ void Board::repaint() {
 }
 
 void Board::renderClock() {
-  GdkGC *gc;
+  GdkGC *gc=clkgc;
   int i,j,k,sq,bh,ch,th;
   char z[64];
   bool ef, lowtime=false;
@@ -592,8 +355,6 @@ void Board::renderClock() {
 
   sq=sqside;
 
-  gc=gdk_gc_new(clkbar);
-
   C.prepare(clkbar,gc,0,0,250,SCRAP_WIDTH);
   C.setColor(0);
   C.drawRect(0,0,C.W,C.H,true);
@@ -661,13 +422,13 @@ void Board::renderClock() {
 
     for(k=0;k<5;k++) {
       if (C.H < 16) break;
-      C.drawString(j, C.Y + 16, 2, info[k]);
+      C.drawString(j, C.Y + 16, 2, info[k] );
       C.consumeTop(16);
     }
     
     // house string is info[5]
 
-    if (strlen(info[5])) {      
+    if (!info[5].empty()) {      
       if ((!global.DrawHouseStock)||(C.H < 96)) {
 	if (C.H >= 16) {
 	  C.drawString(j,C.Y+16,2,info[5]);
@@ -678,13 +439,13 @@ void Board::renderClock() {
 	piece pk[5]={PAWN,ROOK,KNIGHT,BISHOP,QUEEN};
 	int st[5][2];
 	int si,sj,we,be;
-	char *p;
 
 	// build stock info from string
 	for(si=0;si<5;si++) { st[si][0]=st[si][1]=0; }
 	sj=0;
-	for(p=info[5];*p!=0;p++) {
-	  switch(*p) {
+	k=info[5].length();
+	for(i=0;i<j;i++) {
+	  switch(info[5][i]) {
 	  case 'P': st[0][sj]++; break;
 	  case 'R': st[1][sj]++; break;
 	  case 'N': st[2][sj]++; break;
@@ -698,7 +459,7 @@ void Board::renderClock() {
 	if (ef) { we=0; be=48; } else { we=48; be=0; }
 	for(si=0;si<5;si++) {
 	  if (st[si][0]) {
-	    Board::vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+we,pk[si]|WHITE);
+	    global.vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+we,pk[si]|WHITE);
 	    sprintf(z,"%d",st[si][0]);
 	    C.setColor(0xffffff);
 	    C.drawString(40*si+20, C.Y+48+we, 2, z);
@@ -706,7 +467,7 @@ void Board::renderClock() {
 	      addTarget(new DropSource(pk[si]|WHITE,Width()+40*si,C.Y+we,40,48));
 	  }
 	  if (st[si][1]) {
-	    Board::vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+be,pk[si]|BLACK);
+	    global.vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+be,pk[si]|BLACK);
 
 	    sprintf(z,"%d",st[si][1]);
 	    C.setColor(0xffffff);
@@ -718,7 +479,7 @@ void Board::renderClock() {
 
 	C.consumeTop(100);
       } // text | graphic stock
-    } // if strlen info[5]
+    } // if !info[5].empty
 
     // scratch-board pieces and buttons
     if (EditMode) {
@@ -731,8 +492,8 @@ void Board::renderClock() {
       C.drawRect(10,C.Y,i*6,i*2+8,false);
 
       for(k=0;k<6;k++) {
-	Board::vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y,WHITE|(k+1));
-	Board::vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y+i+8,BLACK|(k+1));
+	global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y,WHITE|(k+1));
+	global.vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y+i+8,BLACK|(k+1));
 	addTarget(new DropSource(WHITE|(k+1),Width()+10+i*k,C.Y,i,i));
 	addTarget(new DropSource(BLACK|(k+1),Width()+10+i*k,C.Y+i+8,i,i));
       }
@@ -792,7 +553,6 @@ void Board::renderClock() {
     }
   }
 
-  gdk_gc_destroy(gc);
   clock_ch=0;
 }
 
@@ -812,7 +572,7 @@ void Board::thaw() {
   if (frozen_lvl < 0) frozen_lvl=0;
   if ((!frozen_lvl)&&(repaint_due)) {
     repaint();
-    repaint_due=0;
+    repaint_due=false;
   }
 }
 
@@ -824,21 +584,27 @@ void Board::invalidate() {
 void Board::queueRepaint() {
   global.debug("Board","queueRepaint");
   if (frozen_lvl)
-    repaint_due=1;
+    repaint_due=true;
   else
     repaint();
 }
 
 void Board::setInfo(int idx,char *s) {
   global.debug("Board","setInfo",s);
-  info[idx][63]=0;
-  strncpy(info[idx%6],s,64);
+  info[idx%6]=s;
+  clock_ch=1;
+  queueRepaint();
+}
+
+void Board::setInfo(int idx,string &s) {
+  global.debug("Board","setInfo/2");
+  info[idx%6] = s;
   clock_ch=1;
   queueRepaint();
 }
 
 void Board::update(bool sndflag=false) {
-  int i,j,k;
+  int i,j,k,rate;
   AnimatedPiece *ap;  
   Position rpv;
   bool ef;
@@ -850,13 +616,13 @@ void Board::update(bool sndflag=false) {
     return;
 
   if ((global.AnimateMoves)&&(gotAnimationLoop)) {
-    update_queue=1;
+    update_queue=true;
     UpdateStack.push(sndflag);
     return;
   }  
 
-  position=*(mygame->getCurrentPosition());
-  rpv=*(mygame->getPreviousPosition());
+  position = mygame->getCurrentPosition();
+  rpv = mygame->getPreviousPosition();
   ef=effectiveFlip();
 
   if ((global.HilightLastMove)||(global.AnimateMoves)) {
@@ -869,6 +635,7 @@ void Board::update(bool sndflag=false) {
 	  sok=true; // sound ok
 	  fake=position;
 	  fake.intersection(rpv);
+	  rate = global.SmootherAnimation ? 3 : 2;
 	  for(i=0;i<LastMove.size();i++) {
 	    if (ef)
 	      ap=new FlatAnimatedPiece(LastMove[i]->Piece,
@@ -876,14 +643,14 @@ void Board::update(bool sndflag=false) {
 				       morey+bory+sqside*(LastMove[i]->SY),
 				       borx+sqside*(7-LastMove[i]->DX),
 				       morey+bory+sqside*(LastMove[i]->DY),
-				       2*LastMove[i]->distance(),sndflag);
+				       rate*LastMove[i]->distance(),sndflag);
 	    else
 	      ap=new FlatAnimatedPiece(LastMove[i]->Piece,
 				       borx+sqside*(LastMove[i]->SX),
 				       morey+bory+sqside*(7-LastMove[i]->SY),
 				       borx+sqside*(LastMove[i]->DX),
 				       morey+bory+sqside*(7-LastMove[i]->DY),
-				       2*LastMove[i]->distance(),sndflag);
+				       rate*LastMove[i]->distance(),sndflag);
 	    ap->create(yidget->window,cur,sqside);
 	    animes.push_back(ap);
 	  }
@@ -893,7 +660,10 @@ void Board::update(bool sndflag=false) {
 	    gotAnimationLoop=1;
 	    currently_rendered.invalidate();
 	    board_configure_event(yidget,0,(gpointer)this);
-	    gtk_timeout_add(global.VectorPieces?50:80,board_animate,this);
+	    if (rate==2)
+	      gtk_timeout_add(global.UseVectorPieces?50:80,board_animate,this);
+	    else
+	      gtk_timeout_add(global.UseVectorPieces?30:60,board_animate,this);
 	  }
 	}
   }
@@ -904,7 +674,7 @@ void Board::update(bool sndflag=false) {
   if (dr_active)
     dr_step=0;
 
-  update_queue=0;
+  update_queue=false;
 
   if ((!sok)&&(global.BeepWhenOppMoves)&&(sndflag))
     global.flushSound();
@@ -923,9 +693,9 @@ void Board::sendMove() {
 	global.status->setText(z);
 	goto sendmove_cleanup;	
       }      
-    position.moveCartesian(ax[0],ay[0],ax[1],ay[1]);
+    position.moveCartesian(ax[0],ay[0],ax[1],ay[1],REGULAR,true);
     mygame->sendMove(ax[0],ay[0],ax[1],ay[1]);
-    allowMove=0;
+    allowMove=false;
   } else if (global.Premove) {
     premoveq=1;
     pmx[0]=ax[0];
@@ -940,8 +710,8 @@ void Board::sendMove() {
   repaint();
 }
 
-void Board::setCanMove(int value) {
-  int oldv;
+void Board::setCanMove(bool value) {
+  bool oldv;
   global.debug("Board","setCanMove");
 
   oldv=allowMove;
@@ -954,7 +724,7 @@ void Board::setCanMove(int value) {
     if (pmp==EMPTY) {
       if (position.isMoveLegalCartesian(pmx[0],pmy[0],pmx[1],pmy[1],
 					mygame->MyColor,mygame->Variant)) {
-	position.moveCartesian(pmx[0],pmy[0],pmx[1],pmy[1]);
+	position.moveCartesian(pmx[0],pmy[0],pmx[1],pmy[1],REGULAR,true);
 	mygame->sendMove(pmx[0],pmy[0],pmx[1],pmy[1]);
       }
     } else {
@@ -990,7 +760,7 @@ void Board::walkBack1() {
     return;
   freeze();
   mygame->goBack1();
-  position=*(mygame->getCurrentPosition());
+  position= mygame->getCurrentPosition();
   setInfo(2,position.getLastMove());
   setInfo(4,position.getMaterialString(mygame->Variant));
   setInfo(5,position.getHouseString());
@@ -1003,7 +773,7 @@ void Board::walkBackAll() {
     return;
   freeze();
   mygame->goBackAll();
-  position=*(mygame->getCurrentPosition());
+  position= mygame->getCurrentPosition();
   setInfo(2,position.getLastMove());
   setInfo(4,position.getMaterialString(mygame->Variant));
   setInfo(5,position.getHouseString());
@@ -1016,7 +786,7 @@ void Board::walkForward1() {
     return;
   freeze();
   mygame->goForward1();
-  position=*(mygame->getCurrentPosition());
+  position= mygame->getCurrentPosition();
   setInfo(2,position.getLastMove());
   setInfo(4,position.getMaterialString(mygame->Variant));
   setInfo(5,position.getHouseString());
@@ -1029,7 +799,7 @@ void Board::walkForwardAll() {
     return;
   freeze();
   mygame->goForwardAll();
-  position=*(mygame->getCurrentPosition());
+  position=mygame->getCurrentPosition();
   setInfo(2,position.getLastMove());
   setInfo(4,position.getMaterialString(mygame->Variant));
   setInfo(5,position.getHouseString());
@@ -1041,7 +811,7 @@ void Board::outlineRectangle(GdkGC *gc, int x,int y,int color,int pen) {
   int i,j,k,X,Y,S;
   gdk_rgb_gc_set_foreground(gc,color);
   j=x; k=y; if (effectiveFlip()) j=7-j; else k=7-k;
-  if ((global.VectorPieces)||(!cur->extruded))
+  if ((global.UseVectorPieces)||(!cur->extruded))
     for(i=0;i<pen;i++)
       gdk_draw_rectangle(yidget->window,gc,FALSE,borx+j*sqside+i,
 			 morey+bory+k*sqside+i,
@@ -1106,7 +876,7 @@ void Board::drop(piece p) {
 void Board::popupDropMenu(int col,int row,int sx,int sy,guint32 etime) {
   GtkWidget *popmenu;
   GtkWidget *mi[7];
-  Position *pos;
+  Position pos;
   int i,j;
   char z[64];
 
@@ -1128,23 +898,23 @@ void Board::popupDropMenu(int col,int row,int sx,int sy,guint32 etime) {
   gtk_widget_set_sensitive(mi[0],FALSE);
   mi[1]=gtk_menu_item_new();
 
-  sprintf(z,"Pawn    %d",i=pos->getStockCount(PAWN|mygame->MyColor));
+  sprintf(z,"Pawn    %d",i=pos.getStockCount(PAWN|mygame->MyColor));
   mi[2]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[2],FALSE);
 
-  sprintf(z,"Rook    %d",i=pos->getStockCount(ROOK|mygame->MyColor));
+  sprintf(z,"Rook    %d",i=pos.getStockCount(ROOK|mygame->MyColor));
   mi[3]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[3],FALSE);
 
-  sprintf(z,"Knight  %d",i=pos->getStockCount(KNIGHT|mygame->MyColor));
+  sprintf(z,"Knight  %d",i=pos.getStockCount(KNIGHT|mygame->MyColor));
   mi[4]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[4],FALSE);
 
-  sprintf(z,"Bishop  %d",i=pos->getStockCount(BISHOP|mygame->MyColor));
+  sprintf(z,"Bishop  %d",i=pos.getStockCount(BISHOP|mygame->MyColor));
   mi[5]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[5],FALSE);
 
-  sprintf(z,"Queen   %d",i=pos->getStockCount(QUEEN|mygame->MyColor));
+  sprintf(z,"Queen   %d",i=pos.getStockCount(QUEEN|mygame->MyColor));
   mi[6]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[6],FALSE);
 
@@ -1343,18 +1113,18 @@ void Board::drawCoordinates(GdkPixmap *dest,GdkGC *gc,int side) {
 void Board::composeVecBoard(GdkGC *gc) {
   Position *joker;
   int i,j;
-  vpieces.drawSquares(buffer,gc,sqside, borx, bory);
+  global.vpieces.drawSquares(buffer,gc,sqside, borx, bory);
 
   if (gotAnimationLoop) joker=&fake; else joker=&position;
   if (effectiveFlip())
     for(i=0;i<8;i++)
       for(j=0;j<8;j++)
-	Board::vpieces.drawPiece(buffer,gc,sqside,borx+i*sqside,bory+j*sqside,
+	global.vpieces.drawPiece(buffer,gc,sqside,borx+i*sqside,bory+j*sqside,
 				 joker->getPiece(7-i,j));
   else
     for(i=0;i<8;i++)
       for(j=0;j<8;j++)
-	Board::vpieces.drawPiece(buffer,gc,sqside,borx+i*sqside,bory+j*sqside,
+	global.vpieces.drawPiece(buffer,gc,sqside,borx+i*sqside,bory+j*sqside,
 				 joker->getPiece(i,7-j));
 
   if (global.ShowCoordinates)
@@ -1408,7 +1178,7 @@ static GdkEventMotion LastMotionEvent;
 gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
                             gpointer data) {
   Board *me;
-  GdkGC *gc=0;
+  GdkGC *gc;
   int i,j,k,w,h,ww,wh,sq,pw,fw,fh;
 
   me=(Board *)data;
@@ -1419,6 +1189,10 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
 
   sq=me->sqside;
 
+  if (!me->wgc)
+    me->wgc=gdk_gc_new(widget->window);
+  gc=me->wgc;
+
   if (fw> ( me->Width() + 250) )
     gdk_draw_rectangle(widget->window,widget->style->black_gc,TRUE,
 		       me->Width()+250,0,fw-me->Width()-250,fh);
@@ -1437,10 +1211,8 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
 
   // draw board+pieces
 
-  if (global.VectorPieces) {
-    gc=gdk_gc_new(widget->window);
+  if (global.UseVectorPieces)
     me->composeVecBoard(gc);
-  }
 
   gdk_draw_pixmap(widget->window,
 		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
@@ -1461,8 +1233,6 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
   // draw highlighted squares
 
   if ((me->sp)||(!me->LastMove.empty())||(me->premoveq)) {
-    if (!gc)
-      gc=gdk_gc_new(widget->window);
     
     if ((global.HilightLastMove)&&(!me->LastMove.empty()))
       for(i=0;i<me->LastMove.size();i++) {
@@ -1486,9 +1256,6 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
     }
   }
 
-  if (gc)
-    gdk_gc_destroy(gc);
-
   /* if dragging... */
   if (me->dr_active) {
     me->dr_step=0;
@@ -1539,7 +1306,7 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   ww-=200;
 
   if ( (ww-2*me->borx) > (wh-2*me->bory) ) {
-    if ( (global.VectorPieces) || (! me->orig->extruded) ) {
+    if ( (global.UseVectorPieces) || (! global.pieceset->extruded) ) {
       optimal_square=(wh - 2 * me->bory)/8;
       me->morey=0;
     } else {
@@ -1548,7 +1315,7 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
     }
   } else {
     optimal_square=(ww - 2 * me->borx)/8;
-    if ( (global.VectorPieces) || (! me->orig->extruded) )
+    if ( (global.UseVectorPieces) || (! global.pieceset->extruded) )
       me->morey = 0;
     else
       me->morey = optimal_square / 2;
@@ -1557,11 +1324,11 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   if (optimal_square < 10) return FALSE;
 
   if (me->cur == 0)
-    me->cur=new PieceSet(me->orig);
+    me->cur=new PieceSet(global.pieceset);
   if (me->cur->side != optimal_square) {
-    if (me->cur->side != me->orig->side) {
+    if (me->cur->side != global.pieceset->side) {
       delete me->cur;
-      me->cur=new PieceSet(me->orig);
+      me->cur=new PieceSet(global.pieceset);
     }      
     me->cur->scale(optimal_square);
     me->currently_rendered.invalidate();
@@ -1573,7 +1340,7 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   }
   me->sqside=sq;
 
-  if (global.VectorPieces)
+  if (global.UseVectorPieces)
     return 1;
 
   if ((me->currently_rendered == me->position)&&(!me->update_queue))
@@ -1614,10 +1381,9 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
 		     GDK_RGB_DITHER_NORMAL,me->scrap,SCRAP_WIDTH*3);
 
   if (global.ShowCoordinates) {
-    GdkGC *gc;
-    gc=gdk_gc_new(me->yidget->window);
-    me->drawCoordinates(me->buffer,gc,sq);
-    gdk_gc_destroy(gc);
+    if (!me->wgc)
+      me->wgc=gdk_gc_new(widget->window);
+    me->drawCoordinates(me->buffer,me->wgc,sq);
   }
 
   me->currently_rendered=*(joker);
@@ -1712,12 +1478,15 @@ gboolean board_motion_event(GtkWidget *widget,
     return FALSE;
 
   sq=he=me->sqside;
-  if ((me->cur->extruded)&&(!global.VectorPieces)) he=me->cur->height;
+  if ((me->cur->extruded)&&(!global.UseVectorPieces)) he=me->cur->height;
 
   // STEP 0: prepare double buffer for drawing
 
   gdk_window_get_size(widget->window,&ww,&wh);
-  gcpgc=gdk_gc_new(widget->window);
+
+  if (!me->wgc)
+    me->wgc=gdk_gc_new(widget->window);
+  gcpgc=me->wgc;
   gdk_rgb_gc_set_foreground(gcpgc,0);
 
   sw=ww; sh=wh;
@@ -1747,7 +1516,7 @@ gboolean board_motion_event(GtkWidget *widget,
     if (rh > he) { rh = he; clock_too=true; }
  
     if ((rw>0)&&(rh>0))
-      if (global.VectorPieces) {
+      if (global.UseVectorPieces) {
 	gdk_draw_pixmap(GCPbuffer,
 			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
 			me->buffer,
@@ -1777,7 +1546,7 @@ gboolean board_motion_event(GtkWidget *widget,
   // STEP 2: erase source square (if not dragging from zhouse stock)
 
   if (! me->dr_fromstock) {
-    if (global.VectorPieces) {
+    if (global.UseVectorPieces) {
 
       gc=gdk_gc_new(GCPbuffer);
       gdk_rgb_gc_set_foreground(gc,((me->dr_c+me->dr_r)&0x01)?global.DarkSqColor:global.LightSqColor);
@@ -1854,10 +1623,10 @@ gboolean board_motion_event(GtkWidget *widget,
   if (dx < 0) { x-=dx; dx=0; }
   if (dy < 0) { y-=dy; dy=0; }
 
-  if (global.VectorPieces) {
+  if (global.UseVectorPieces) {
     if (!gc)
       gc=gdk_gc_new(GCPbuffer);
-    Board::vpieces.drawPiece((GdkPixmap *)(GCPbuffer),
+    global.vpieces.drawPiece((GdkPixmap *)(GCPbuffer),
 			     gc,sq,dx,dy,me->dr_p);    
     xr=dy;
   } else {
@@ -1870,6 +1639,7 @@ gboolean board_motion_event(GtkWidget *widget,
     if (me->cur->extruded) {
       xr=dy-he+sq; xc=he-sq;
       if (xr < 0) { xc+=xr; xr=0; }
+      if (xc > me->morey) { --xc; ++xr; }
     } else { xc=0; xr=dy; }
 
     got_ghost = rects_overlap(dx,xr,sq,sq+xc,me->borx+me->dr_c*sq,
@@ -1877,6 +1647,7 @@ gboolean board_motion_event(GtkWidget *widget,
 
     if (got_ghost) { // fake empty square
       tsq=(rgbptr)g_malloc(sq*he*4);
+
       me->cur->bitblt(me->scrap,tsq, 
 		      me->borx+me->dr_c*sq, 
 		      me->morey+me->bory+me->dr_r*sq-xc,SCRAP_WIDTH,
@@ -1980,13 +1751,12 @@ gboolean board_motion_event(GtkWidget *widget,
   }
 
   // STEP 3 1/2: draw coordinates if not in vector mode
-  if ((!global.VectorPieces)&&(global.ShowCoordinates))
+  if ((!global.UseVectorPieces)&&(global.ShowCoordinates))
     me->drawCoordinates(GCPbuffer,gcpgc,sq);
 
   // STEP 4: commit double buffer to window
 
   gdk_draw_pixmap(widget->window,gcpgc,GCPbuffer,0,0,0,0,ww,wh);
-  gdk_gc_destroy(gcpgc);
 
   me->dr_dirty[0]=dx;
   me->dr_dirty[1]=xr;
@@ -2195,7 +1965,7 @@ gboolean vec_board_animate(gpointer data) {
   for(li=me->animes.begin();li!=me->animes.end();li++)
     if (nonover) {
       (*li)->step();
-      Board::vpieces.drawPiece(me->buffer,gc,me->sqside,
+      global.vpieces.drawPiece(me->buffer,gc,me->sqside,
 			       (*li)->getX(),(*li)->getY(),(*li)->getPiece());
     }
   me->pasteVecBoard();
@@ -2244,7 +2014,7 @@ gboolean board_animate(gpointer data) {
   int sh0=0,sh1=0;
 
   global.debug("Board","board_animate");
-  if (global.VectorPieces)
+  if (global.UseVectorPieces)
     return(vec_board_animate(data));
 
   int not_over=0;
@@ -2370,35 +2140,9 @@ DropSource * TargetManager::hitTarget(int x,int y) {
   return 0;
 }
 
-void board_ptell(GtkWidget *b,gpointer data) {
-  char z[256],x[256],c;
-  
-  strcpy(z,(char *)data);
-
-  if ((z[0]=='-')||(z[0]=='+')) {
-    Board::BugTell=z;
-    return;
-  } else {
-    if (Board::BugTell.empty())
-      Board::BugTell=z;
-    else {
-      strcpy(x,Board::BugTell.c_str());
-      c=x[strlen(x)-1];
-      if ((c=='+')||(c=='-'))
-	Board::BugTell+=z;
-      else
-	Board::BugTell=z;
-    }
-  }
-
-  if (global.network) {
-    sprintf(z,"ptell %s",Board::BugTell.c_str());
-    global.network->writeLine(z);
-  }
-}
-
-EditBoard::EditBoard(ChessGame *cg) : Board(false) {
+EditBoard::EditBoard(ChessGame *cg) : Board() {
   EditMode = true;
   FreeMove = true;
   setGame(cg);
 }
+
