@@ -1,10 +1,10 @@
-/* $Id: board.h,v 1.34 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: board.h,v 1.40 2002/04/23 07:55:09 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -66,58 +66,30 @@ class RootBoard {
   void clockString(int val,char *z);
 };
 
-class BareBoard : public WidgetProxy,
-		  public RootBoard,
-                  public ClockHost {
- public:
-  BareBoard();
-  ~BareBoard();
-
-  void setPosition(Position *pos);
-  void setWhite(char *name);
-  void setBlack(char *name);
-  virtual void updateClock();
-  
-  ChessClock clock;
-
- private:
-  void update();
-
-  static VectorPieces vpieces;
-  char Names[2][64];
-  Position position;
-  GdkPixmap *pixbuf;
-  GdkFont *myfont;
-
-  friend gboolean bareboard_expose(GtkWidget *widget,GdkEventExpose *ee,
-				   gpointer data);
-};
-
 class Board : public WidgetProxy,
 	      public ClockHost,
 	      public RootBoard,
 	      public NotebookInsider,
-              public TargetManager
+              public TargetManager,
+	      public PieceChangeListener
 {
  public:
-  Board(bool withbugpane=false);
+  Board();
   ~Board();
 
-  static void  setCurrentPieceSet(char *filename, bool chgPieces, bool chgSquares);
-  static char *getCurrentPieceSet();
-  static char *getCurrentSquareSet();
-
   void repaint();
 
-  void  setSensitive(int state);
+  void  setSensitive(bool state);
+  void  setPartnerGame();
   void  setSelColor(int color);
   void  clearSel();
-  void  setFlipped(int flip);
+  void  setFlipped(bool flip);
   void  setFlipInversion(bool v);
   bool  getFlipInversion();
   void  setClock(int wsecs,int bsecs,int actv,int cdown);
   void  setInfo(int idx,char *s);
-  void  setCanMove(int value);
+  void  setInfo(int idx,string &s);
+  void  setCanMove(bool value);
   void  stopClock();
   void  reset();
   
@@ -135,6 +107,8 @@ class Board : public WidgetProxy,
   void update(bool sndflag=false);
   virtual void updateClock();
 
+  virtual void pieceSetChanged();
+
   void walkBack1();
   void walkBackAll();
   void walkForward1();
@@ -142,46 +116,41 @@ class Board : public WidgetProxy,
   void openMovelist();
   bool hasGame();
 
+  bool effectiveFlip();
+
   void dump();
 
-  void refreshPieceSet();
   void supressNextAnimation();
 
-  void openBugPane();
-  void closeBugPane();
-  void addBugText(char *text);
-
   bool FreeMove; // editing positions, examining games
   bool EditMode; // explicit edit mode (with pieces to drop and additional commands)
  
-  BareBoard *bugboard;
-
-  static string BugTell;
   static Board * PopupOwner;
 
  private:
-  static PieceSet *orig;
-  static VectorPieces vpieces;
-
-  GtkWidget *yidget, *bugpane;
+  GtkWidget *yidget;
 
   PieceSet *cur;
   GdkPixmap *buffer,*clkbar;
+  GdkGC *clkgc, *wgc;
   GdkFont *f1,*f2,*f3;
   rgbptr scrap;
-  int canselect;
-  int sel_color;
-  int flipped;
-  int frozen_lvl;
-  int repaint_due;
-  int allowMove;
-  int LockAnimate;
+
+  bool canselect;
+  bool allowMove;
   bool FlipInvert;
+  bool flipped;
+  bool repaint_due;
+  bool update_queue;
+
+  int sel_color;
+  int frozen_lvl;
+  int LockAnimate;
+
   vector<SMove *> LastMove;
 
   list<AnimatedPiece *> animes;
   int gotAnimationLoop;
-  int update_queue;
 
   Position position;
   Position currently_rendered;
@@ -211,7 +180,7 @@ class Board : public WidgetProxy,
   bool  dr_fto;
 
   // info lines
-  char info[6][64];
+  string info[6];
 
   int dropsq[2];
 
@@ -237,10 +206,6 @@ class Board : public WidgetProxy,
   void composeVecBoard(GdkGC *gc);
   void pasteVecBoard();
 
-  void createBugPane();
-
-  bool effectiveFlip();
-
   friend gboolean board_animate(gpointer data);
   friend gboolean vec_board_animate(gpointer data);
 
@@ -269,7 +234,6 @@ class Board : public WidgetProxy,
   friend void menu_blackp(GtkMenuItem *item, gpointer data);
   friend void menu_gamep(GtkMenuItem *item, gpointer data);
 
-  friend void board_ptell(GtkWidget *b,gpointer data);
 };
 
 class EditBoard : public Board {
