@@ -1,4 +1,4 @@
-/* $Id: chess.cc,v 1.51 2002/03/29 14:27:25 bergo Exp $ */
+/* $Id: chess.cc,v 1.60 2002/05/02 00:41:04 bergo Exp $ */
 
 /*
 
@@ -38,6 +38,17 @@
 #include "util.h"
 #include "global.h"
 
+// --- stream operators
+
+ostream &operator<<(ostream &s, PGNheader h) {
+  int i;
+  for(i=0;i<h.header.size();i++)
+    s << '[' << h.header[i]->name << " \"" << h.header[i]->value << "\"]\n";
+  return(s);
+}
+
+// --- classes
+
 ChessGame::ChessGame() {
   int i;
 
@@ -69,6 +80,8 @@ ChessGame::ChessGame() {
   source=GS_Other;
   source_data="n/a";
   AmPlaying=false;
+
+  global.create("ChessGame");
 }
 
 ChessGame::ChessGame(int _number,int _tyme,int _inc, int _rated,
@@ -98,11 +111,12 @@ ChessGame::ChessGame(int _number,int _tyme,int _inc, int _rated,
   source=GS_Other;
   source_data="n/a";
   AmPlaying=false;
+
+  global.create("ChessGame");
 }
 
 ChessGame::ChessGame(ChessGame *src) {
-  list<Position *>::iterator i;
-  Position *p;
+  list<Position>::iterator i;
   bool cset=false;
 
   GameNumber = src->GameNumber;
@@ -127,29 +141,24 @@ ChessGame::ChessGame(ChessGame *src) {
 
   // clone move list
   for(i=src->moves.begin();i!=src->moves.end();i++) {
-    p=new Position();
-    (*p) = (*(*i));
-    moves.push_back(p);
+    moves.push_back(Position(*i));
     if (src->cursor == i) { cursor = moves.end(); cursor--; cset = true; }
   }
 
   if (moves.empty())
-    moves.push_back(new Position());
+    moves.push_back(Position());
   if (!cset) { cursor = moves.end();  cursor--; }
 
   source=src->source;
   source_data=src->source_data;
   AmPlaying=src->AmPlaying;
+
+  global.create("ChessGame");
 }
 
 ChessGame::~ChessGame() {
-  list<Position *>::iterator mi;
-
-  for(mi=moves.begin();mi!=moves.end();mi++)
-    delete(*mi);
-
   moves.clear();
-
+  global.destroy("ChessGame");
 }
 
 int ChessGame::operator==(int gnum) {
@@ -215,7 +224,11 @@ void ChessGame::updateStock() {
   if (!myboard)
     return;
   if (!moves.empty())
-    myboard->setInfo(5,moves.back()->getHouseString());
+    myboard->setInfo(5,moves.back().getHouseString());
+
+  // bughouse partner game
+  if (protodata[2])
+    global.bugpane->setPosition(moves.back());
 }
 
 bool ChessGame::isFresh() {
@@ -225,37 +238,26 @@ bool ChessGame::isFresh() {
 // usually called to remove an illegal move from the movelist
 void ChessGame::retreat(int nmoves) {
   for(;nmoves;nmoves--)
-    if (!moves.empty()) {
-      delete(moves.back());
+    if (!moves.empty())
       moves.pop_back();
-    }
   cursor=moves.end();
   cursor--;
 }
 
 void ChessGame::fixExamineZigZag(Position &suspect) {
-  list<Position *>::iterator i;
+  list<Position>::iterator i;
 
   for(i=moves.begin();i!=moves.end();i++)
-    if ( (*(*i)) == suspect ) 
-      if (!strcmp(suspect.getLastMove(), (*i)->getLastMove() )) {
-	junkMoves(i,moves.end());
+    if ( (*i) == suspect ) 
+      if (! suspect.getLastMove().compare( (*i).getLastMove() ) ) {
+	moves.erase(i,moves.end());
 	return;
       }
 }
 
-void ChessGame::junkMoves(list<Position *>::iterator from,
-			  list<Position *>::iterator to) {
-  list<Position *>::iterator i;
-  for(i=from;i!=to;i++)
-    delete(*i);
-  moves.erase(from,to);
-}
-
 void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
 			       int wclock,int bclock,char *infoline, 
 			       bool sndflag=false) {
-  Position *np;
   char z[32];
 
   global.debug("ChessGame","updatePosition");
@@ -263,19 +265,32 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
   if (over)
     return;
 
+  // bughouse partner's game
+  if (protodata[2]) {
+    global.bugpane->freeze();
+    global.bugpane->setPosition(p);
+
+    if (StopClock)
+      global.bugpane->setClock(wclock,bclock,0,clock_regressive);
+    else
+      global.bugpane->setClock(wclock,bclock,blacktomove?1:-1,clock_regressive);
+
+    global.bugpane->thaw();
+  }
+
   if (last_half_move>=0)
-    if (! (moves.empty()) )
-      if (p == *(moves.back())) {
+    if (! moves.empty() )
+      if (p == moves.back() ) {
 	myboard->update(sndflag);
 	return;
       }
 
-  np=new Position();
-  *np=p;
-  np->setLastMove(infoline);
+  Position np;
+  np=p;
+  np.setLastMove(infoline);
   //  cerr << "LastMove = [" << infoline << "]\n";
 
-  fixExamineZigZag(*np);
+  fixExamineZigZag(np);
 
   moves.push_back(np);
   cursor=moves.end();
@@ -292,8 +307,8 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
     
     if (infoline)
       myboard->setInfo(2,infoline);
-    myboard->setInfo(4,np->getMaterialString(Variant));
-    myboard->setInfo(5,np->getHouseString());
+    myboard->setInfo(4,np.getMaterialString(Variant));
+    myboard->setInfo(5,np.getHouseString());
     myboard->update(sndflag);
     myboard->thaw();
     myboard->contentUpdated();
@@ -317,25 +332,25 @@ Board * ChessGame::getBoard() {
   return(myboard);
 }
 
-Position * ChessGame::getLastPosition() {
+Position & ChessGame::getLastPosition() {
   if (moves.empty())
-    return(&startpos);
+    return(startpos);
   return(moves.back());
 }
 
-Position * ChessGame::getCurrentPosition() {
+Position & ChessGame::getCurrentPosition() {
   if (moves.empty())
-    return(&startpos);
+    return(startpos);
   if (cursor!=moves.end())
     return(*cursor);
   else
     return(moves.back());
 }
 
-Position * ChessGame::getPreviousPosition() {
-  list<Position *>::iterator pv;
+Position & ChessGame::getPreviousPosition() {
+  list<Position>::iterator pv;
   if (moves.empty())
-    return(&startpos);
+    return(startpos);
   if (cursor!=moves.end()) {
     pv=cursor;
     if (pv!=moves.begin())
@@ -400,7 +415,7 @@ void ChessGame::moveListClosed() {
 }
 
 void ChessGame::editEmpty() {
-  Position *p;
+  Position p;
   int i,j;
 
   if (!LocalEdit) return;
@@ -408,46 +423,43 @@ void ChessGame::editEmpty() {
   moves.clear();
   last_half_move=1;
 
-  p = new Position();
-  for(i=0;i<8;i++) for(j=0;j<8;j++) p->setPiece(i,j,EMPTY);
+  for(i=0;i<8;i++) for(j=0;j<8;j++) p.setPiece(i,j,EMPTY);
   moves.push_back(p);
   cursor=moves.end();
   cursor--;
-  if (myboard) { myboard->setInfo(2,(*cursor)->getLastMove()); myboard->update(); }
+  if (myboard) { myboard->setInfo(2,(*cursor).getLastMove()); myboard->update(); }
 }
 
 void ChessGame::editStartPos() {
-  Position *p;
+  Position p;
 
   if (!LocalEdit) return;
 
   moves.clear();
   last_half_move=1;
 
-  p = new Position();
   moves.push_back(p);
   cursor=moves.end();
   cursor--;
-  if (myboard) { myboard->setInfo(2,(*cursor)->getLastMove()); myboard->update(); }
+  if (myboard) { myboard->setInfo(2,(*cursor).getLastMove()); myboard->update(); }
 }
 
 void ChessGame::sendDrop(piece p, int x, int y) {
-  Position *o;
-  list<Position *>::iterator z;
+  list<Position>::iterator z;
 
   if (LocalEdit) {
-    o = new Position();
-    (*o) = *(*cursor);
-    o->setPiece(x,y,p);
+    Position o;
+    o = *cursor;
+    o.setPiece(x,y,p);
 
     z=cursor; z++; 
-    junkMoves(z,moves.end());
+    moves.erase(z,moves.end());
 
     moves.push_back(o);
     last_half_move++;
     cursor=moves.end();
     cursor--;
-    if (myboard) { myboard->setInfo(2,(*cursor)->getLastMove()); myboard->update(); }
+    if (myboard) { myboard->setInfo(2,(*cursor).getLastMove()); myboard->update(); }
   } else {
     if (global.protocol) global.protocol->sendDrop(p&PIECE_MASK,x,y);
   }
@@ -455,38 +467,44 @@ void ChessGame::sendDrop(piece p, int x, int y) {
 }
 
 void ChessGame::sendMove(int x1,int y1,int x2,int y2) {
-  Position *p;
   int promote=0;
   piece q;
-  list<Position *>::iterator z;
+  list<Position>::iterator z;
   char v[16];
 
   if (!moves.empty()) {
+    Position p;
     p=moves.back();
-    if ((p->getPiece(x1,y1)==(PAWN|WHITE))&&
+
+    if ((p.getPiece(x1,y1)==(PAWN|WHITE))&&
 	(y1==6)&&(y2==7))
       promote=1;
-    if ((p->getPiece(x1,y1)==(PAWN|BLACK))&&
+    if ((p.getPiece(x1,y1)==(PAWN|BLACK))&&
 	(y1==1)&&(y2==0))
       promote=1;
   }
 
+  // ====================== BEGIN SCRATCH BOARD ONLY ===============================
   if (LocalEdit) {
-    p = new Position();
-    (*p) = *(*cursor);
+    Position p;
+    p = *cursor;
 
-    q=p->getPiece(x1,y1)&COLOR_MASK;
-    if (p->isMoveLegalCartesian(x1,y1,x2,y2,q,REGULAR)) {
-      p->stdNotationForMove(x1,y1,x2,y2,
-			    promote ?  q | global.promotion->getPiece() : EMPTY,
-			    v, REGULAR);
-      p->setLastMove(v);
+    q=p.getPiece(x1,y1)&COLOR_MASK;
+
+    if (p.isMoveLegalCartesian(x1,y1,x2,y2,q,REGULAR)) {
+      p.stdNotationForMove(x1,y1,x2,y2,
+			   promote ?  q | global.promotion->getPiece() : EMPTY,
+			   v, REGULAR);
+      p.setLastMove(v);
 					   
-      p->moveCartesian(x1,y1,x2,y2,REGULAR);
-      if (promote) p->setPiece(x2,y2, q | global.promotion->getPiece() );
+      p.moveCartesian(x1,y1,x2,y2,REGULAR);
+      if (promote) p.setPiece(x2,y2, q | global.promotion->getPiece() );
+
     } else {
-      p->setPiece(x2,y2,p->getPiece(x1,y1));
-      p->setPiece(x1,y1,EMPTY);
+
+      p.setPiece(x2,y2,p.getPiece(x1,y1));
+      p.setPiece(x1,y1,EMPTY);
+
     }
     
     z=cursor; z++; if (z!=moves.end()) moves.erase(z,moves.end());
@@ -495,8 +513,11 @@ void ChessGame::sendMove(int x1,int y1,int x2,int y2) {
     last_half_move++;
     cursor=moves.end();
     cursor--;
-    if (myboard) { myboard->setInfo(2,(*cursor)->getLastMove()); myboard->update(); }
-  } else {
+    if (myboard) { myboard->setInfo(2,(*cursor).getLastMove()); myboard->update(); }
+  }
+  // ====================== END SCRATCH BOARD ONLY ===============================
+
+  else {
     if (global.protocol) global.protocol->sendMove(x1,y1,x2,y2,promote);
   }
 
@@ -504,10 +525,10 @@ void ChessGame::sendMove(int x1,int y1,int x2,int y2) {
 
 void ChessGame::flipHint(int flip) {
   if (myboard)
-    myboard->setFlipped(flip);
+    myboard->setFlipped(flip!=0);
 }
 
-void ChessGame::enableMoving(int flag) {
+void ChessGame::enableMoving(bool flag) {
   if (!myboard)
     return;
   myboard->setCanMove(flag);
@@ -518,6 +539,10 @@ void ChessGame::endGame(char *reason,GameResult _result) {
     return; // can't end twice
   result=_result;
 
+  // bughouse partner game
+  if (protodata[2])
+    global.bugpane->stopClock();
+
   if (myboard)
     myboard->stopClock();
 
@@ -527,7 +552,7 @@ void ChessGame::endGame(char *reason,GameResult _result) {
 
   showResult();
   if (myboard)
-    myboard->setCanMove(0);
+    myboard->setCanMove(false);
 
   if (pgn.empty())
     guessPGNFromInfo();
@@ -540,26 +565,23 @@ int ChessGame::isOver() {
   return(over);
 }
 
-void ChessGame::updateGame(list<Position *> &gamedata) {
-  list<Position *>::iterator li;
-  Position *np, startpos;
+void ChessGame::updateGame(list<Position> &gamedata) {
+  list<Position>::iterator li;
+  Position startpos;
   int hmn=-1;
 
-  for(li=moves.begin();li!=moves.end();li++)
-    delete(*li);
   moves.clear();
 
   for(li=gamedata.begin();li!=gamedata.end();li++) {
-    np=new Position();
-    (*np)=(*(*li));
-    moves.push_back(np);
+    Position p;
+    p = *li;
+    moves.push_back(p);
     ++hmn;
   }
 
   // should fix the adding of FEN fields on followed games
   if (moves.empty() && Variant!=WILD) {
-    np=new Position();
-    moves.push_back(np);
+    moves.push_back(Position());
     hmn=0;
   }
 
@@ -604,8 +626,8 @@ char * ChessGame::getPlayerString(int index) {
 // ---- PGN
 
 void ChessGame::guessInfoFromPGN() {
+  static char *tcsep = "+ -/\t";
   const char *cp;
-  TString t(128);
 
   if (cp=pgn.get("White"))
     strcpy(PlayerName[0],cp);
@@ -622,9 +644,10 @@ void ChessGame::guessInfoFromPGN() {
   }
 
   if (cp=pgn.get("TimeControl")) {
+    tstring t;
     t.set((char *) cp);
-    STime = atoi(t.token("+ -/\t"));
-    Inc = atoi(t.token("+ -/\t"));
+    STime = t.tokenvalue(tcsep);
+    Inc   = t.tokenvalue(tcsep);
   }
 
   if (cp=pgn.get("Variant")) {
@@ -692,17 +715,17 @@ void ChessGame::guessPGNFromInfo() {
     }
   }
 
-  if ( startpos != *moves.front() )
-    pgn.set("FEN",(char *)(moves.front()->getFEN()));
+  if ( startpos != moves.front() )
+    pgn.set("FEN", moves.front().getFEN() );
 }
 
 bool ChessGame::savePGN(char *filename, bool append=false) {
-  FILE *f;
-  char z[512],zz[512],*p;
+  char z[512];
   const char *cp;
-  list<Position *>::iterator li;
+  list<Position>::iterator li;
   int xp,lm,mn;
-  TString t(64);
+  tstring t;
+  string sp, path, *p;
 
   if (moves.size() < 4) {
     global.status->setText("savePGN failed: Won't save game with less than 2 moves");
@@ -711,11 +734,14 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
 
   // FIXME ~username won't work, only ~/something
   if (filename[0]=='~') {
-    sprintf(zz,"%s/%s",getenv("HOME"),&filename[2]);
-    filename=zz;
-  }
+    path=global.env.Home;
+    path+='/';
+    path+=(&filename[2]);
+  } else
+    path=filename;
+
+  ofstream f(path.c_str(),append? ios::app : ios::out);
 
-  f=fopen(filename,append?"a":"w");
   if (!f) {
     sprintf(z,"savePGN failed: %s",filename);
     global.status->setText(z);
@@ -724,54 +750,52 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
 
   if (pgn.empty())
     guessPGNFromInfo();
-  pgn.write(f);
 
-  /* TO BE GONE SOON
-  if (Variant!=REGULAR)
-    fprintf(f,"%%eboard:variant:%s\n",variantName(Variant));
-  if (STime > 0)
-    fprintf(f,"%%eboard:clock/r:%d:%d:%d\n",STime,Inc,Rated);
-  if (strlen(ereason)>5)
-    fprintf(f,"%%eboard:clue:%s\n",ereason);
-  */
-
-  fprintf(f,"\n");
+  f << pgn << endl;
 
   lm=0; xp=0;
   for(li=moves.begin();li!=moves.end();li++) {
-    p=(*li)->getLastMove();
-    if (!strlen(p))
+
+    sp=(*li).getLastMove();
+
+    if (sp.empty())
       continue;
-    t.set(p);
-    mn=atoi(t.token("."));
+
+    t.set(sp);
+
+    mn=t.tokenvalue(".");
     if (!mn)
       continue;
     if (mn>lm) {
-      fprintf(f,"%d. ",mn);
+      f << mn << ". ";
       lm=mn;
-      xp+=(lm>9)?4:3;
+      xp+=(lm>9)?4:3; // MINOR BUG FOR GAMES WITH 100+ MOVES
     }
     p=t.token("(). \t");
-    fprintf(f,"%s ",p);
-    xp+=1+strlen(p);
+    f << (*p) << ' ';
+    xp+=1+p->length();
 
     if (xp>70) {
       xp=0;
-      fprintf(f,"\n");
+      f << endl;
     }
   }
 
   if (strlen(ereason) > 5) {
-    if (xp) fprintf(f,"\n");
-    fprintf(f,"{%s} ",ereason);
+    if (xp) f << endl;
+    f << '{' << ereason << "} ";
   }
 
   cp=pgn.get("Result");
+
   if (cp)
-    fprintf(f,"%s\n\n",cp);
+    f << cp;
   else
-    fprintf(f,"*\n\n");
-  fclose(f);
+    f << '*';
+
+  f << endl << endl;
+
+  f.close();
 
   sprintf(z,"--- %s game to PGN file %s",
 	  append?"Appended":"Wrote",filename);
@@ -780,7 +804,7 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
 }
 
 // indexonly: do not load the move text now
-bool ChessGame::ParsePgnGame(FILE *f, 
+bool ChessGame::ParsePgnGame(istream &f, 
 			     char * filename,
 			     bool indexonly, 
 			     int gameid,
@@ -806,9 +830,8 @@ bool ChessGame::ParsePgnGame(FILE *f,
   int   halfmove = 0;
   piece color    = WHITE;
 
-  list<Position *> gl;
-  list<Position *>::iterator gli;
-  Position * g;
+  list<Position> gl;
+  list<Position>::iterator gli;
   ChessGame *game=0;
 
   ExtPatternMatcher Legacy0, Legacy1, Legacy2, Result;
@@ -833,7 +856,7 @@ bool ChessGame::ParsePgnGame(FILE *f,
   Result.set("*%N-%N*");
   Binder2.add(&Result,0);
 
-  pstart=ftell(f);
+  pstart=f.tellg();
   if (!updatee) {
     game = new ChessGame();
     game->Variant = v;
@@ -856,7 +879,8 @@ bool ChessGame::ParsePgnGame(FILE *f,
   cerr << "entering new game\n";
 #endif
 
-  while( fgets(line, 512, f) ) {
+  while( memset(line,0,512), f.getline(line,510,'\n') ) {
+    line[strlen(line)]='\n';
 
 #ifdef PGN_DEBUG
     cerr << "state = " << state << ", current line is: " << line;
@@ -928,14 +952,19 @@ bool ChessGame::ParsePgnGame(FILE *f,
 
       case 4:
 	if (*p == ']') { 
-	  if (game)
+	  if (game) {
 	    game->pgn.set(alpha,beta);
+	    if (!strcmp(alpha,"Variant")) {
+	      v=variantFromName(beta);
+	      game->Variant=v;
+	    }
+	  }
 
 	  if (!indexonly)
 	    if (!strcmp(alpha,"FEN")) {
-	      g=new Position();
-	      g->setFEN(beta);
-	      if (!gl.empty()) gl.clear(); // LEAK
+	      Position g;
+	      g.setFEN(beta);
+	      gl.clear();
 	      gl.push_back(g);	    
 	    }
 
@@ -992,17 +1021,20 @@ bool ChessGame::ParsePgnGame(FILE *f,
 	  if (indexonly) { state=8; break; }
 
 	  // move tokens
-	  if (gl.empty()) gl.push_back(new Position());
+	  if (gl.empty()) gl.push_back(Position());
 	  
-	  g = new Position();
-	  (*g) = *(gl.back());
-	  g->clearAnnotation();
+	  {
+	    Position g;
+	    g = gl.back();
+	    g.clearAnnotation();
+
+	    g.moveAnyNotation(alpha,color,game ? game->Variant : v);
+	    sprintf(beta,"%d%s %s",1+halfmove/2,
+		    (color==WHITE)?".":". ...",alpha);
+	    g.setLastMove(beta);
+	    gl.push_back(g);
+	  }
 
-	  g->moveAnyNotation(alpha,color,game ? game->Variant : v);
-	  sprintf(beta,"%d%s %s",1+halfmove/2,
-		  (color==WHITE)?".":". ...",alpha);
-	  g->setLastMove(beta);
-	  gl.push_back(g);
 	  color^=COLOR_MASK;  
 	  halfmove++;
 	  state=8;
@@ -1020,8 +1052,9 @@ bool ChessGame::ParsePgnGame(FILE *f,
 
 	  if (game)
 	    if ( (*p == '@') && (game->Variant == REGULAR) ) {
-	      fseek(f,pstart,SEEK_SET);
-	      gl.clear(); // LEAK
+	      f.seekg(pstart);
+	      gl.clear();
+
 	      delete game;
 	      return(ParsePgnGame(f,filename,indexonly,gameid,CRAZYHOUSE));
 	    }
@@ -1039,7 +1072,7 @@ bool ChessGame::ParsePgnGame(FILE *f,
 
 	if (! gl.empty() ) {
 	  if (! global.annotator.isOpen() )
-	    gl.back()->addAnnotation( global.annotator.open() );	    
+	    gl.back().addAnnotation( global.annotator.open() );	    
 	  global.annotator.append(*p >= ' ' ? *p : ' ');
 	}
 
@@ -1047,8 +1080,6 @@ bool ChessGame::ParsePgnGame(FILE *f,
 
       case 9: // game over, waiting end of line
 	if (*p == '\n') {
-	  for(gli=gl.begin();gli!=gl.end();gli++)
-	    delete(*gli);
 	  gl.clear();
 	  return true;
 	}
@@ -1057,48 +1088,47 @@ bool ChessGame::ParsePgnGame(FILE *f,
 
     } // for p
 
-  } // while fgets
+  } // while getline
+
+  if (!gl.empty())
+    gl.clear();
 
-  if (!gl.empty()) gl.clear(); // LEAK
   if (game) delete game;  
   return false;
 } // method
 
 bool ChessGame::loadMoves() {
-  FILE *f;
   char z[512];
   if (Loaded) return true;
-  f=fopen(PGNSource,"r");
+
+  ifstream f(PGNSource);
 
   if (!f) {
-    sprintf(z,"can't load PGN move text from %s (error opening file, errno=%d)",
-	    PGNSource, errno);
+    sprintf(z,"can't load PGN move text from %s (error opening file)",PGNSource);
     global.output->append(z,0xc0ff00);
     return false;
   }
 
-  if (fseek(f,SourceOffset,SEEK_SET)) {
-    sprintf(z,"can't seek to offset %lu of %s (errno=%d)",
-	    SourceOffset,PGNSource,errno);
+  if (!f.seekg(SourceOffset)) {
+    sprintf(z,"can't seek to offset %lu of %s",SourceOffset,PGNSource);
     global.output->append(z,0xc0ff00);
-    fclose(f);
+    f.close();
     return false;
   }
   
   if (!ParsePgnGame(f,PGNSource,false,-1,Variant,this)) {
     strcpy(z,"error parsing PGN data");
     global.output->append(z,0xc0ff00);
-    fclose(f);
+    f.close();
     return false;
   }
-  fclose(f);
+  f.close();
 
   Loaded = true;
   return true;
 }
 
 void ChessGame::LoadPGN(char *filename) {
-  FILE *f;
   UnboundedProgressWindow *pw;
   int nextid,count=0;
 
@@ -1107,7 +1137,7 @@ void ChessGame::LoadPGN(char *filename) {
   list<ChessGame *>::iterator gi;
   bool id_space_empty=false;
 
-  f=fopen(filename,"r");
+  ifstream f(filename);
   if (!f)
     return;
 
@@ -1129,7 +1159,7 @@ void ChessGame::LoadPGN(char *filename) {
   if ( ii == g_ids.end() )
     id_space_empty = true;
 
-  while(!feof(f)) {
+  while(!f.eof()) {
 
     if (ParsePgnGame(f,filename,true,nextid)) {
 
@@ -1148,7 +1178,7 @@ void ChessGame::LoadPGN(char *filename) {
     if (! (count%5) ) pw->setProgress(count);
   }
 
-  fclose(f);
+  f.close();
   g_ids.clear();
   delete pw;
 }
@@ -1204,16 +1234,13 @@ PGNpair::PGNpair(const char *n, char *v) {
   name=n;
   value=v;
 }
-  
-void PGNpair::write(FILE *f) {
-  fprintf(f,"[%s \"%s\"]\n",name.c_str(),value.c_str());
-}
 
+PGNpair::PGNpair(const char *n, string &v) {
+  name=n;
+  value=v;
+}
+  
 PGNheader::~PGNheader() {
-  int i,j;
-  j=size();
-  for(i=0;i<j;i++)
-    delete header[i];
   header.clear();
 }
 
@@ -1227,6 +1254,16 @@ void PGNheader::set(const char *n,char *v) {
   header.push_back(new PGNpair(n,v));
 }
 
+void PGNheader::set(const char *n,string &v) {
+  vector<PGNpair *>::iterator li;
+  for(li=header.begin();li!=header.end();li++)
+    if ( (*li)->name == n ) {
+      (*li)->value = v;
+      return;
+    }
+  header.push_back(new PGNpair(n,v));
+}
+
 void PGNheader::setIfAbsent(const char *n,char *v) {
   vector<PGNpair *>::iterator li;
   for(li=header.begin();li!=header.end();li++)
@@ -1245,12 +1282,6 @@ void PGNheader::remove(const char *n) {
     }
 }
 
-void PGNheader::write(FILE *f) {
-  vector<PGNpair *>::iterator li;
-  for(li=header.begin();li!=header.end();li++)
-    (*li)->write(f);
-}
-
 int PGNheader::empty() {
   return(header.empty());
 }
@@ -1265,9 +1296,11 @@ PGNpair * PGNheader::get(int index) {
 
 const char * PGNheader::get(const char *n) {
   vector<PGNpair *>::iterator li;
-  for(li=header.begin();li!=header.end();li++)
+
+  for(li=header.begin();li!=header.end();li++) {
     if ( (*li)->name == n )
       return((*li)->value.c_str());
+  }
   return 0;
 }
 
