@@ -1,10 +1,10 @@
-/* $Id: chess.h,v 1.34 2002/03/29 14:27:25 bergo Exp $ */
+/* $Id: chess.h,v 1.37 2002/04/28 21:04:34 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@ class PGNpair {
  public:
   PGNpair();
   PGNpair(const char *n, char *v);
-  void write(FILE *f);
+  PGNpair(const char *n, string &v);
 
   string name;
   string value;
@@ -53,9 +53,9 @@ class PGNheader {
  public:
   ~PGNheader();
   void set(const char *n,char *v);
+  void set(const char *n,string &v);
   void setIfAbsent(const char *n,char *v);
   void remove(const char *n);
-  void write(FILE *f);
   int  empty();
   int  size();
   PGNpair *get(int index);
@@ -63,6 +63,8 @@ class PGNheader {
 
  private:
   vector<PGNpair *> header;
+
+  friend ostream &operator<<(ostream &s, PGNheader h);
 };
 
 class ChessGame : public MoveListListener {
@@ -78,7 +80,7 @@ class ChessGame : public MoveListListener {
   void updatePosition(Position &p,int movenum,int blacktomove,
 		      int wclock,int bclock,char *infoline,bool sndflag=false);
   void updateStock();
-  void updateGame(list<Position *> &gamedata);
+  void updateGame(list<Position> &gamedata);
   void flipHint(int flip);
   void setBoard(Board *b);
   Board * getBoard();
@@ -87,7 +89,7 @@ class ChessGame : public MoveListListener {
   void acknowledgeInfo();
   void endGame(char *reason,GameResult _result);
   void fireWhiteClock(int wval,int bval);
-  void enableMoving(int flag);
+  void enableMoving(bool flag);
   int  isOver();
 
   char *getPlayerString(int index);
@@ -96,9 +98,9 @@ class ChessGame : public MoveListListener {
 
   void dump();
 
-  Position * getLastPosition();
-  Position * getCurrentPosition();
-  Position * getPreviousPosition();
+  Position & getLastPosition();
+  Position & getCurrentPosition();
+  Position & getPreviousPosition();
 
   void goBack1();
   void goBackAll();
@@ -113,7 +115,7 @@ class ChessGame : public MoveListListener {
 
   void retreat(int nmoves);
   static void LoadPGN(char *filename);
-  static bool ParsePgnGame(FILE *f,
+  static bool ParsePgnGame(istream &f,
 			   char * filename,
 			   bool indexonly, 
 			   int gameid,
@@ -164,11 +166,9 @@ class ChessGame : public MoveListListener {
  private:
   void showResult();
   void fixExamineZigZag(Position &suspect);
-  void junkMoves(list<Position *>::iterator from,
-		 list<Position *>::iterator to);
 
-  list<Position *> moves;
-  list<Position *>::iterator cursor;
+  list<Position> moves;
+  list<Position>::iterator cursor;
 
   char PrivateString[96];
 
