@@ -1,4 +1,4 @@
-/* $Id: dlg_connect.cc,v 1.9 2002/03/12 02:03:23 bergo Exp $ */
+/* $Id: dlg_connect.cc,v 1.10 2002/04/24 01:16:33 bergo Exp $ */
 
 /*
 
@@ -170,7 +170,7 @@ void dlg_connect_ok(GtkWidget *w,gpointer data) {
   ConnectDialog *cd;
   ProtocolParser *pp;
   HostBookmark *hbm;
-  TString t(256);
+  tstring t;
 
   cd=(ConnectDialog *)data;
   strncpy(cd->Host,gtk_entry_get_text(GTK_ENTRY(cd->server)),256);
@@ -179,9 +179,8 @@ void dlg_connect_ok(GtkWidget *w,gpointer data) {
 	  gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cd->protocol)->entry)),64);  
 
   t.set(cd->Host);
-  if (t.token(" \n\t\r") == 0) {
+  if (! t.token(" \n\t\r"))
     return;
-  }
 
   hbm=new HostBookmark();
   strcpy(hbm->host,cd->Host);
