@@ -1,4 +1,4 @@
-/* $Id: dlg_prefs.cc,v 1.20 2002/04/08 00:44:50 bergo Exp $ */
+/* $Id: dlg_prefs.cc,v 1.24 2002/04/27 14:43:42 bergo Exp $ */
 
 /*
 
@@ -40,7 +40,7 @@ PreferencesDialog::PreferencesDialog() : ModalDialog("Preferences") {
   GtkWidget *v,*hs,*ok,*apply,*cancel,*bhb,*nb;
 
   // gui
-  GtkWidget *uil,*uiv,*uih,*tfr,*tr[4],*tv,*uih2,*uih3,*uih4;
+  GtkWidget *uil,*uiv,*uih,*tfr,*tr[4],*tv,*uih3,*uih4,*uih5;
   GSList *tl;
 
   // -- fonts
@@ -115,11 +115,12 @@ PreferencesDialog::PreferencesDialog() : ModalDialog("Preferences") {
 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(plainb), global.PlainSquares?1:0);
 
-  uih2=gtk_hbox_new(FALSE,4);
   uih3=gtk_hbox_new(FALSE,4);
   uih4=gtk_hbox_new(FALSE,4);
-  aapieceb=gtk_check_button_new_with_label("Antialias piece images");
-  gtk_box_pack_start(GTK_BOX(uih2),aapieceb,FALSE,FALSE,0);
+  uih5=gtk_hbox_new(FALSE,4);
+
+  smoothb=gtk_check_button_new_with_label("Smoother animation (eats more CPU)");
+  gtk_box_pack_start(GTK_BOX(uih5),smoothb,FALSE,FALSE,0);
 
   dhsb=gtk_check_button_new_with_label("Graphic representation of crazy/bughouse stock");
   gtk_box_pack_start(GTK_BOX(uih3),dhsb,FALSE,FALSE,0);
@@ -127,18 +128,18 @@ PreferencesDialog::PreferencesDialog() : ModalDialog("Preferences") {
   aqbar=gtk_check_button_new_with_label("Show shortcut buttons below board");
   gtk_box_pack_start(GTK_BOX(uih4),aqbar,FALSE,FALSE,0);
 
-  gtk_box_pack_start(GTK_BOX(uiv),uih2,FALSE,FALSE,4);
+  gtk_box_pack_start(GTK_BOX(uiv),uih5,FALSE,FALSE,4);
   gtk_box_pack_start(GTK_BOX(uiv),uih3,FALSE,FALSE,4);
   gtk_box_pack_start(GTK_BOX(uiv),uih4,FALSE,FALSE,4);
 
-  gtk_widget_show(aapieceb);
+  gtk_widget_show(smoothb);
   gtk_widget_show(dhsb);
   gtk_widget_show(aqbar);
-  gtk_widget_show(uih2);
   gtk_widget_show(uih3);
   gtk_widget_show(uih4);
+  gtk_widget_show(uih5);
 
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aapieceb), global.AntialiasPieces?1:0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(smoothb), global.SmootherAnimation?1:0);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dhsb), global.DrawHouseStock?1:0);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aqbar), global.ShowQuickbar?1:0);
 
@@ -155,7 +156,7 @@ PreferencesDialog::PreferencesDialog() : ModalDialog("Preferences") {
 
   gtk_container_set_border_width(GTK_CONTAINER(icv),6);
 
-  ifr1=gtk_frame_new("Seek Graph");
+  ifr1=gtk_frame_new("Seek Table");
   gtk_frame_set_shadow_type(GTK_FRAME(ifr1),GTK_SHADOW_ETCHED_IN);
   ifr2=gtk_frame_new("Channel Tells");
   gtk_frame_set_shadow_type(GTK_FRAME(ifr2),GTK_SHADOW_ETCHED_IN);
@@ -173,10 +174,10 @@ PreferencesDialog::PreferencesDialog() : ModalDialog("Preferences") {
 
   gtk_box_pack_start(GTK_BOX(icv),ifr1,FALSE,FALSE,4);
 
-  sgb=gtk_check_button_new_with_label("Dynamic Seek Graph");
+  sgb=gtk_check_button_new_with_label("Dynamic Seek Table");
   gtk_box_pack_start(GTK_BOX(ifv1),sgb,FALSE,FALSE,4);
 
-  hsb=gtk_check_button_new_with_label("Inhibit seek lines on console when Seek Graph is active");
+  hsb=gtk_check_button_new_with_label("Inhibit seek lines on console when Seek Table is active");
   gtk_box_pack_start(GTK_BOX(ifv1),hsb,FALSE,FALSE,4);
 
   gtk_box_pack_start(GTK_BOX(icv),ifr2,FALSE,FALSE,4);
@@ -570,7 +571,7 @@ void PreferencesDialog::Apply() {
   ApplyEntry(efont[3],global.ConsoleFont,96,&changed,&fchg);
 
   ApplyCheckBox(plainb, &global.PlainSquares, &changed, &psetchg);
-  ApplyCheckBox(aapieceb, &global.AntialiasPieces, &changed, &psetchg);
+  ApplyCheckBox(smoothb, &global.SmootherAnimation, &changed, 0);
   ApplyCheckBox(dhsb, &global.DrawHouseStock, &changed, 0);
   ApplyCheckBox(aqbar, &global.ShowQuickbar, &changed, 0);
   ApplyCheckBox(showratb, &global.ShowRating, &changed, 0);
@@ -634,7 +635,7 @@ void PreferencesDialog::Apply() {
 	global.BLi++) {
       (*global.BLi)->reloadFonts();
       if (psetchg)
-	(*global.BLi)->refreshPieceSet();
+	global.respawnPieceSet();
       else
 	(*global.BLi)->invalidate();
       (*global.BLi)->queueRepaint();
@@ -688,7 +689,6 @@ void prefs_ok(GtkWidget *w,gpointer data) {
 void prefs_cfont1(GtkWidget *w,gpointer data) {
   PreferencesDialog *me;
   GtkWidget *fd;
-  FILE *f;
   char cmd[256];
   sigfunc pipeopt;
 
@@ -764,7 +764,6 @@ void prefs_fok4(GtkWidget *w,gpointer data) {
 void prefs_cfont2(GtkWidget *w,gpointer data) {
   PreferencesDialog *me;
   GtkWidget *fd;
-  FILE *f;
   char cmd[256];
   sigfunc pipeopt;
 
@@ -796,7 +795,6 @@ void prefs_cfont2(GtkWidget *w,gpointer data) {
 void prefs_cfont3(GtkWidget *w,gpointer data) {
   PreferencesDialog *me;
   GtkWidget *fd;
-  FILE *f;
   char cmd[256];
   sigfunc pipeopt;
 
@@ -828,9 +826,7 @@ void prefs_cfont3(GtkWidget *w,gpointer data) {
 void prefs_cfont4(GtkWidget *w,gpointer data) {
   PreferencesDialog *me;
   GtkWidget *fd;
-  FILE *f;
   char cmd[256];
-  sigfunc pipeopt;
 
   me=(PreferencesDialog *)data;  
 
