@@ -152,7 +152,7 @@ extern "C" {
 
   gint gtk_ftext_save_buffer(GtkFText * ftext, char *filename);
   gint gtk_ftext_get_last_line(GtkFText * ftext);
-  gint gtk_ftext_find_text_upwards(GtkFText * ftext, char *needle,
+  gint gtk_ftext_find_text_upwards(GtkFText * ftext, const char *needle,
 				   int topline, int bottomline, 
 				   gboolean hilight);
   void gtk_ftext_goto(GtkFText * ftext, int pseudoline);
