@@ -1,4 +1,4 @@
-/* $Id: global.cc,v 1.53 2002/04/08 00:44:50 bergo Exp $ */
+/* $Id: global.cc,v 1.63 2002/05/02 00:41:04 bergo Exp $ */
 
 /*
 
@@ -46,13 +46,55 @@
 #include "notebook.h"
 #include "board.h"
 #include "quickbar.h"
+#include "pieces.h"
 
 Global global;
 
+// stream ops
+
+ostream & operator<<(ostream &s, WindowGeometry w) {
+  s << '(' << w.X << ',' << w.Y << ',';
+  s << w.W << ',' << w.H << ')';
+  return(s);
+}
+
+ostream & operator<<(ostream &s, Desktop d) {
+  s << d.wMain << ',' << d.wGames << ',' << d.wLocal << ',';
+  s << d.wAds;
+  return(s);
+}
+
+ostream & operator<<(ostream &s, EngineBookmark b) {
+  s << b.caption << '^';
+  if (b.directory.empty()) s << "NULL"; else s << b.directory;
+  s << '^';
+  if (b.cmdline.empty()) s << "NULL"; else s << b.cmdline;
+  s << '^' << b.humanwhite << '^' << b.secspermove << '^';
+  s << b.maxply  << '^' << b.think << '^';
+  s << b.proto << '^' << ((int)b.mode);
+  return(s);
+}
+
+ostream & operator<<(ostream &s, TerminalColor t) {
+  s.setf(ios::hex,ios::basefield);
+  s << t.TextDefault    << ',';
+  s << t.TextBright     << ',';
+  s << t.PrivateTell    << ',';
+  s << t.NewsNotify     << ',';
+  s << t.Mamer          << ',';
+  s << t.KibitzWhisper  << ',';
+  s << t.Shouts         << ',';
+  s << t.Seeks          << ',';
+  s << t.ChannelTell    << ',';
+  s << t.Engine         << ',';
+  s << t.Background;
+  s.setf(ios::dec,ios::basefield);
+  return(s);
+}
+
 Global::Global() {
   input=0;
   output=0;
-  outputset=0;
   network=0;
   status=0;
   protocol=0;
@@ -64,6 +106,11 @@ Global::Global() {
   bmlistener=0;
   qbcontainer=0;
   quickbar=0;
+  killbox=0;
+  lowernotebook=0;
+  mainpaned=0;
+  bugpane=0;
+  pieceset=0;
 
   LastScratch = 0;
 
@@ -98,10 +145,9 @@ Global::Global() {
   LightSqColor=0xe7cf93;
   DarkSqColor=0x9e8661;
 
-  AntialiasPieces=0;
   ShowRating=1;
 
-  VectorPieces=0;
+  UseVectorPieces=0;
   CheckLegality=0;
   DrawHouseStock=1;
 
@@ -115,6 +161,8 @@ Global::Global() {
   SplitChannels=0;
   ChannelsToConsoleToo=0;
 
+  SmootherAnimation=0;
+
   ShowQuickbar=1;
   LowTimeWarningLimit=5;
   RetrieveChannelNames=1;
@@ -149,7 +197,7 @@ Global::Global() {
   RCKeys.push_back("LightSqColor");
   RCKeys.push_back("DarkSqColor");     //10
   RCKeys.push_back("Host");
-  RCKeys.push_back("Antialias");       //12
+  RCKeys.push_back("Antialias");       //12 [deprecated]
   RCKeys.push_back("ShowRating");
   RCKeys.push_back("ScrollBack");      //14
   RCKeys.push_back("FicsAutoLogin");
@@ -178,6 +226,8 @@ Global::Global() {
   RCKeys.push_back("QuickbarButton");  //38
   RCKeys.push_back("LowTimeWarningLimit");
   RCKeys.push_back("RetrieveChannelNames"); //40
+  RCKeys.push_back("SmootherAnimation");
+  RCKeys.push_back("EngineBookmark");  // 42
 
   PopupHelp = false;
 }
@@ -247,15 +297,14 @@ void Global::statOS() {
 }
 
 void Global::ensureDirectories() {
-  char *home;
   char z[256];
   DIR *tdir;
-  home=getenv("HOME");
-  if (!home) {
+
+  if (env.Home.empty()) {
     cerr << "[eboard] ** no $HOME" << endl;
     return;
   }
-  sprintf(z,"%s/.eboard",home);
+  sprintf(z,"%s/.eboard",env.Home.c_str());
 
   tdir=opendir(z);
   if (tdir==NULL)
@@ -264,11 +313,11 @@ void Global::ensureDirectories() {
     closedir(tdir);
     
   if (createDir(z)) return;
-  sprintf(z,"%s/.eboard/craftylog",home);
+  sprintf(z,"%s/.eboard/craftylog",env.Home.c_str());
   createDir(z);
-  sprintf(z,"%s/.eboard/eng-out",home);
+  sprintf(z,"%s/.eboard/eng-out",env.Home.c_str());
   createDir(z);
-  sprintf(z,"%s/.eboard/scripts",home);
+  sprintf(z,"%s/.eboard/scripts",env.Home.c_str());
   createDir(z);
 }
 
@@ -286,180 +335,204 @@ int Global::createDir(char *z) {
 }
 
 void Global::readRC() {
-  TString t(512);
-  static char *sep=" \n\t,:\r";
+  tstring t;
+  string  *p;
+  char    line[512],rev[128];
+
+  static char *sep=" \n\t,:\r^";
   static char *sep2="\n\t,:\r";
-  char z[512],*p;
-  FILE *f;
+  static char *sep3=" \n\t:\r";
+  static char *sep4=",\n\r";
+  static char *sep5="\n\t:\r";
+
   HostBookmark *hbm;
-  int i;
+  EngineBookmark *ebm;
+  int i,j;
   QButton *qb;
 
-  sprintf(z,"%s/.eboard/eboard.conf",getenv("HOME"));
-
-  f=fopen(z,"r");
-  if (!f)
+  if (env.Config.empty())
     return;
 
-  while(fgets(z,511,f)) {
-    t.set(z);
+  ifstream rc(env.Config.c_str());
+
+  if (!rc)
+    return;
+
+  memset(rev,0,128);
+  rev['R']=0;
+  rev['L']=1;
+  rev['T']=2;
+  rev['B']=3;
+
+  t.setChomp(true);
+
+  memset(line,0,512);
+  while(rc.getline(line,511,'\n')) {
+    t.set(line);
+    memset(line,0,512);
+
     p=t.token(sep);
     if (!p) continue;
-    if (p[0]=='#') continue;
-    if (!strcmp(p,RCKeys[0])) {HilightLastMove=atoi(t.token(sep)); continue;}
-    if (!strcmp(p,RCKeys[1])) {AnimateMoves=atoi(t.token(sep)); continue;}
-    if (!strcmp(p,RCKeys[2])) {Premove=atoi(t.token(sep)); continue;}
-    if (!strcmp(p,RCKeys[3])) {
-      Board::setCurrentPieceSet(t.token("\n\r\t: "),true,true); 
-      continue;
-    }
-    if (!strcmp(p,RCKeys[4])) {
-      if (p=t.token(sep))
-	switch(p[0]) {
-	case 'R': TabPos=0; break;
-	case 'L': TabPos=1; break;
-	case 'T': TabPos=2; break;
-	case 'B': TabPos=3; break;
-	}
-      continue;
-    }
-    if (!strcmp(p,RCKeys[5]))  { strncpy(ClockFont,t.token(sep2),96); continue; }
-    if (!strcmp(p,RCKeys[6]))  { strncpy(PlayerFont,t.token(sep2),96); continue; }
-    if (!strcmp(p,RCKeys[7]))  { strncpy(InfoFont,t.token(sep2),96); continue; }
-    if (!strcmp(p,RCKeys[8]))  { PlainSquares=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[9]))  { LightSqColor=strtol(t.token(sep),0,16); continue; }
-    if (!strcmp(p,RCKeys[10])) { DarkSqColor=strtol(t.token(sep),0,16); continue; }
+    if (p->at(0)=='#') continue;
 
-    if (!strcmp(p,RCKeys[11])) {
-      hbm=new HostBookmark();
-      strcpy(hbm->host,t.token(",\n\r"));
-      hbm->port=atoi(t.token(",\n\r"));
-      strcpy(hbm->protocol,t.token(",\n\r"));
-      HostHistory.push_back(hbm);
-    }
+    for(j=0;j<=42;j++) {
+      if (! p->compare(RCKeys[j]) ) {
+	switch(j) {
+	case  0: HilightLastMove =t.tokenvalue(sep); break;
+	case  1: AnimateMoves    =t.tokenvalue(sep); break;
+	case  2: Premove         =t.tokenvalue(sep); break;
+	case  3: setPieceSet(*(t.token(sep3)),true,true); break;
+	case  4: p=t.token(sep); TabPos=rev[p->at(0)]; break;
+	case  5: p=t.token(sep2);        memset(ClockFont,0,96);
+	         p->copy(ClockFont,95);  break;
+	case  6: p=t.token(sep2);        memset(PlayerFont,0,96);
+	         p->copy(PlayerFont,95); break;
+	case  7: p=t.token(sep2);        memset(InfoFont,0,96);
+	         p->copy(InfoFont,95);   break;
+	case  8: PlainSquares    =t.tokenvalue(sep); break;
+	case  9: LightSqColor    =t.tokenvalue(sep,16); break;
+	case 10: DarkSqColor     =t.tokenvalue(sep,16); break;
+	case 11: hbm=new HostBookmark(); 
+	         p=t.token(sep4); p->copy(hbm->host,128);
+	         hbm->port=t.tokenvalue(sep4);
+	         p=t.token(sep4); p->copy(hbm->protocol,64);
+	         HostHistory.push_back(hbm);
+	         break;
+	case 12: break; // deprecated (antialias)
+	case 13: ShowRating      =t.tokenvalue(sep); break;
+	case 14: ScrollBack      =t.tokenvalue(sep); break;
+	case 15: FicsAutoLogin   =t.tokenvalue(sep); break;
+	case 16: BeepWhenOppMoves=t.tokenvalue(sep); break;
+	case 17: i=t.tokenvalue(sep); 
+	         if (i < N_SOUND_EVENTS) sndevents[i].read(t);
+	         break;
+	case 18: EnableSounds    =t.tokenvalue(sep); break;
+	case 19: UseVectorPieces =t.tokenvalue(sep); break;
+	case 20: CheckLegality   =t.tokenvalue(sep); break;
+	case 21: AppendPlayed    =t.tokenvalue(sep); break;
+	case 22: AppendObserved  =t.tokenvalue(sep); break;
+	case 23: p=t.token(sep);          memset(AppendFile,0,128);
+ 	         p->copy(AppendFile,127); break;
+	case 24: p=t.token(sep2);         memset(ConsoleFont,0,96);
+	         p->copy(ConsoleFont,95); break;
+	case 25: IcsSeekGraph        =t.tokenvalue(sep); break;
+	case 26: HideSeeks           =t.tokenvalue(sep); break;
+	case 27: SplitChannels       =t.tokenvalue(sep); break;
+	case 28: ChannelsToConsoleToo=t.tokenvalue(sep); break;
+	case 29: DrawHouseStock      =t.tokenvalue(sep); break;
+	case 30: p=t.token(sep3);
+	         if (p->compare(pieceset->getSquareName()))
+		   setPieceSet(*p,false,true);
+		 break;
+	case 31: PopupSecondaryGames =t.tokenvalue(sep); break;
+	case 32: SmartDiscard        =t.tokenvalue(sep); break;
+	case 33: ShowCoordinates     =t.tokenvalue(sep); break;
+	case 34: Colors.read(t);      break;
+	case 35: Desk.read(t);        break;
+	case 36: Desk.readConsole(t); break;
+	case 37: ShowQuickbar        =t.tokenvalue(sep); break;
+	case 38: qb=new QButton(); qb->icon=t.tokenvalue(sep5);
+       	         qb->caption=*(t.token(sep5)); qb->command=*(t.token(sep5));
+		 QuickbarButtons.push_back(qb);
+		 break;
+	case 39: LowTimeWarningLimit =t.tokenvalue(sep); break;
+	case 40: RetrieveChannelNames=t.tokenvalue(sep); break;
+	case 41: SmootherAnimation   =t.tokenvalue(sep); break;
+	case 42: ebm=new EngineBookmark(); ebm->read(t);
+	         EnginePresets.push_back(ebm); break;
 
-    if (!strcmp(p,RCKeys[12])) { AntialiasPieces=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[13])) { ShowRating=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[14])) { ScrollBack=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[15])) { FicsAutoLogin=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[16])) { BeepWhenOppMoves=atoi(t.token(sep)); continue; }
+		 //	default: cerr << "ignored [" << (*p) << "]\n";
 
-    // sound events
-    if (!strcmp(p,RCKeys[17])) {
-      i=atoi(t.token(sep));
-      if (i < N_SOUND_EVENTS )
-	sndevents[i].read(t);
-      continue;
-    }
+	} // switch
+      } // compare
+    } // for j 0..41
+  } // while getline
 
-    if (!strcmp(p,RCKeys[18])) { EnableSounds=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[19])) { VectorPieces=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[20])) { CheckLegality=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[21])) { AppendPlayed=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[22])) { AppendObserved=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[23])) { strncpy(AppendFile,t.token(sep),128); continue; }
-    if (!strcmp(p,RCKeys[24])) { strncpy(ConsoleFont,t.token(sep2),96); continue; }
-    if (!strcmp(p,RCKeys[25])) { IcsSeekGraph=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[26])) { HideSeeks=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[27])) { SplitChannels=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[28])) { ChannelsToConsoleToo=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[29])) { DrawHouseStock=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[30])) {
-      strcpy(z,t.token("\n\r\t: "));
-      if (strcmp(Board::getCurrentSquareSet(),z))
-	Board::setCurrentPieceSet(z,false,true);
-      continue;
-    }
-    if (!strcmp(p,RCKeys[31])) { PopupSecondaryGames=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[32])) { SmartDiscard=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[33])) { ShowCoordinates=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[34])) { Colors.read(t); continue; }
-    if (!strcmp(p,RCKeys[35])) { Desk.read(t); continue; }
-    if (!strcmp(p,RCKeys[36])) { Desk.readConsole(t); continue; }
-    if (!strcmp(p,RCKeys[37])) { ShowQuickbar=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[38])) { 
-      qb=new QButton();
-      qb->icon=atoi(t.token("\n\r\t:"));
-      qb->caption=t.token("\n\r\t:");
-      qb->command=t.token("\n\r\t:");
-      QuickbarButtons.push_back(qb);
-    }
-    if (!strcmp(p,RCKeys[39])) { LowTimeWarningLimit=atoi(t.token(sep)); continue; }
-    if (!strcmp(p,RCKeys[40])) { RetrieveChannelNames=atoi(t.token(sep)); continue; }
-
-  }
-  fclose(f);  
+  rc.close();
 }
 
 void Global::writeRC() {
-  char z[512];
-  FILE *f;
+
+  string div;
+  
   list<HostBookmark *>::iterator bi;
+  list<EngineBookmark *>::iterator ei;
   static char *tabpos="RLTB";
   int i;
 
-  sprintf(z,"%s/.eboard/eboard.conf",getenv("HOME"));
-
-  f=fopen(z,"w");
-  if (!f)
+  if (env.Config.empty())
     return;
 
-  fprintf(f,"%s::%d\n",RCKeys[0],HilightLastMove);
-  fprintf(f,"%s::%d\n",RCKeys[1],AnimateMoves);
-  fprintf(f,"%s::%d\n",RCKeys[2],Premove);
-  fprintf(f,"%s::%s\n",RCKeys[3],Board::getCurrentPieceSet());
-  fprintf(f,"%s::%c\n",RCKeys[4],tabpos[TabPos%4]);
-  fprintf(f,"%s::%s\n",RCKeys[5],ClockFont);
-  fprintf(f,"%s::%s\n",RCKeys[6],PlayerFont);
-  fprintf(f,"%s::%s\n",RCKeys[7],InfoFont);
+  div="::";
 
-  fprintf(f,"%s::%d\n",RCKeys[8],PlainSquares);
-  fprintf(f,"%s::%x\n",RCKeys[9],LightSqColor);
-  fprintf(f,"%s::%x\n",RCKeys[10],DarkSqColor);
+  ofstream rc(env.Config.c_str());
+  if (!rc)
+    return;
+
+  rc << RCKeys[0] << div << HilightLastMove << endl;
+  rc << RCKeys[1] << div << AnimateMoves << endl;
+  rc << RCKeys[2] << div << Premove << endl;
+  rc << RCKeys[3] << div << pieceset->getName() << endl;
+  rc << RCKeys[4] << div << tabpos[TabPos%4] << endl;
+  rc << RCKeys[5] << div << ClockFont << endl;
+  rc << RCKeys[6] << div << PlayerFont << endl;
+  rc << RCKeys[7] << div << InfoFont << endl;
+  rc << RCKeys[8] << div << PlainSquares << endl;
+
+  rc.setf(ios::hex, ios::basefield);
+  rc << RCKeys[9]  << div << LightSqColor << endl;
+  rc << RCKeys[10] << div << DarkSqColor << endl;
+  rc.setf(ios::dec, ios::basefield);
 
   for(bi=HostHistory.begin();bi!=HostHistory.end();bi++)
-    fprintf(f,"%s,%s,%d,%s\n",RCKeys[11],
-	    (*bi)->host,(*bi)->port,(*bi)->protocol);
+    rc << RCKeys[11] << ',' << (*bi)->host << ',' <<
+      (*bi)->port << ',' << (*bi)->protocol << endl;
 
-  fprintf(f,"%s::%d\n",RCKeys[12],AntialiasPieces);
-  fprintf(f,"%s::%d\n",RCKeys[13],ShowRating);
-  fprintf(f,"%s::%d\n",RCKeys[14],ScrollBack);
-  fprintf(f,"%s::%d\n",RCKeys[15],FicsAutoLogin);
-  fprintf(f,"%s::%d\n",RCKeys[16],BeepWhenOppMoves);
+  // 12: antialias deprecated
+  rc << RCKeys[13] << div << ShowRating << endl;
+  rc << RCKeys[14] << div << ScrollBack << endl;
+  rc << RCKeys[15] << div << FicsAutoLogin<< endl;
+  rc << RCKeys[16] << div << BeepWhenOppMoves << endl;
 
   for(i=0;i< N_SOUND_EVENTS;i++)
-    sndevents[i].write(f,RCKeys[17],i);
+    rc << RCKeys[17] << div << i << ',' << sndevents[i] << endl;
 
-  fprintf(f,"%s::%d\n",RCKeys[18],EnableSounds);
-  fprintf(f,"%s::%d\n",RCKeys[19],VectorPieces);
-  fprintf(f,"%s::%d\n",RCKeys[20],CheckLegality);
-  fprintf(f,"%s::%d\n",RCKeys[21],AppendPlayed);
-  fprintf(f,"%s::%d\n",RCKeys[22],AppendObserved);
-  fprintf(f,"%s::%s\n",RCKeys[23],AppendFile);
+  rc << RCKeys[18] << div << EnableSounds << endl;
+  rc << RCKeys[19] << div << UseVectorPieces << endl;
+  rc << RCKeys[20] << div << CheckLegality << endl;
+  rc << RCKeys[21] << div << AppendPlayed << endl;
+  rc << RCKeys[22] << div << AppendObserved << endl;
+  rc << RCKeys[23] << div << AppendFile << endl;
+
+  rc << RCKeys[24] << div << ConsoleFont << endl;
+  rc << RCKeys[25] << div << IcsSeekGraph << endl;
+  rc << RCKeys[26] << div << HideSeeks << endl;
+  rc << RCKeys[27] << div << SplitChannels << endl;
+  rc << RCKeys[28] << div << ChannelsToConsoleToo << endl;
+  rc << RCKeys[29] << div << DrawHouseStock << endl;
+  rc << RCKeys[30] << div << pieceset->getSquareName() << endl;
+  rc << RCKeys[31] << div << PopupSecondaryGames << endl;
+  rc << RCKeys[32] << div << SmartDiscard << endl;
+  rc << RCKeys[33] << div << ShowCoordinates << endl;
+
+  rc << RCKeys[34] << div << Colors << endl;
+  rc << RCKeys[35] << div << Desk << endl;
+
+  Desk.writeConsoles(rc,RCKeys[36]);
+
+  rc << RCKeys[37] << div << ShowQuickbar << endl;
 
-  fprintf(f,"%s::%s\n",RCKeys[24],ConsoleFont);
-  fprintf(f,"%s::%d\n",RCKeys[25],IcsSeekGraph);
-  fprintf(f,"%s::%d\n",RCKeys[26],HideSeeks);
-  fprintf(f,"%s::%d\n",RCKeys[27],SplitChannels);
-  fprintf(f,"%s::%d\n",RCKeys[28],ChannelsToConsoleToo);
-  fprintf(f,"%s::%d\n",RCKeys[29],DrawHouseStock);
-  fprintf(f,"%s::%s\n",RCKeys[30],Board::getCurrentSquareSet());
-  fprintf(f,"%s::%d\n",RCKeys[31],PopupSecondaryGames);
-  fprintf(f,"%s::%d\n",RCKeys[32],SmartDiscard);
-  fprintf(f,"%s::%d\n",RCKeys[33],ShowCoordinates);
-  fprintf(f,"%s::",RCKeys[34]); Colors.write(f);
-  fprintf(f,"%s::",RCKeys[35]); Desk.write(f);
-  Desk.writeConsoles(f,RCKeys[36]);
-  fprintf(f,"%s::%d\n",RCKeys[37],ShowQuickbar);
   for(i=0;i<QuickbarButtons.size();i++)
-    fprintf(f,"%s::%d:%s:%s\n",RCKeys[38], 
-	    QuickbarButtons[i]->icon,
-	    QuickbarButtons[i]->caption.c_str(),
-	    QuickbarButtons[i]->command.c_str());
-  fprintf(f,"%s::%d\n",RCKeys[39],LowTimeWarningLimit);
-  fprintf(f,"%s::%d\n",RCKeys[40],RetrieveChannelNames);
+    rc << RCKeys[38] << div << (*QuickbarButtons[i]) << endl;
 
-  fclose(f);
+  rc << RCKeys[39] << div << LowTimeWarningLimit << endl;
+  rc << RCKeys[40] << div << RetrieveChannelNames << endl;
+  rc << RCKeys[41] << div << SmootherAnimation << endl;
+
+  for(ei=EnginePresets.begin();ei!=EnginePresets.end();ei++)    
+    rc << RCKeys[42] << '^' << (*(*ei)) << endl;
+
+  rc.close();
 }
 
 ChessGame * Global::getGame(int num) {
@@ -572,14 +645,14 @@ void Global::timeRunningOut() {
 }
 
 void Global::repaintAllBoards() {
-  for(BLi=BoardList.begin();BLi!=BoardList.end();BLi++)
-    (*BLi)->refreshPieceSet();
+  respawnPieceSet();
 }
 
-bool Global::hasSoundFile(char *p) {
-  vector<string>::iterator vi;
-  for(vi=SoundFiles.begin();vi!=SoundFiles.end();vi++)
-    if ( (*vi) == p )
+bool Global::hasSoundFile(string &p) {
+  int i,j;
+  j=SoundFiles.size();
+  for(i=0;i<j;i++)
+    if ( ! SoundFiles[i].compare(p) )
       return true;
   return false;
 }
@@ -592,62 +665,68 @@ void Global::setPasswordMode(int pm) {
 }
 
 void Global::debug(char *klass,char *method,char *data=0) {
-  FILE *f;
-  char z[256],rm[512];
+  char z[256];
   time_t now;
+  string rm;
 
   if (!DebugLog)
     return;
 
-  sprintf(z,"%s/DEBUG.eboard",getenv("HOME"));
-  f=fopen(z,"a");
+  if (env.Home.empty())
+    return;
+
+  sprintf(z,"%s/DEBUG.eboard",env.Home.c_str());
+
+  ofstream f(z,ios::app);
   if (!f) return;
 
-  sprintf(rm,"+ %s::%s",klass,method);
-  if (data) {
-    strcat(rm," [");
-    strcat(rm,data);
-    strcat(rm,"]");
-  }
+  rm="+ ";
+  rm+=klass;
+  rm+="::";
+  rm+=method;
+  if (data) { rm+=" ["; rm+=data; rm+=']'; }
 
   now=time(0);
   strftime(z,255,"%Y-%b-%d %H:%M:%S",localtime(&now));
-  fprintf(f,"%s [%d] %s\n",z,getpid(),rm);
-  fclose(f);
+
+  f << z << " [" << ((int) getpid()) << "] " << rm << endl;
+  f.close();
 }
 
 void Global::LogAppend(char *msg) {
-  FILE *f;
-  char z[256],rm[512],*p,*q;
+  char z[256],*p;
   static char hexa[17]="0123456789abcdef";
   time_t now;
+  string s;
+
+  if (env.Home.empty())
+    return;
 
   if (PauseLog)
     msg="(message obfuscated -- password mode ?)";
 
   if (CommLog) {
-    sprintf(z,"%s/LOG.eboard",getenv("HOME"));
-    f=fopen(z,"a");
+    sprintf(z,"%s/LOG.eboard",env.Home.c_str());
+
+    ofstream f(z,ios::app);
     if (!f) return;
-    p=msg; q=rm;
-    memset(rm,0,512);
-    for(;*p;p++) {
-      if ( *p == '\n' ) { *q++='\\'; *q++='n'; continue; }
-      if ( *p == '\r' ) { *q++='\\'; *q++='r'; continue; }
-      if ( *p < 32 ) { 
-	*q++='('; 
-	*q++='0';
-	*q++='x';
-	*q++=hexa[(*p)>>4];
-	*q++=hexa[(*p)&0x0f];
-	*q++=')';
-      } else
-	*q++=*p;
-    }
+
+    for(p=msg;*p;p++)
+      switch(*p) {
+      case '\n': s+="\\n"; break;
+      case '\r': s+="\\r"; break;
+      default:
+	if (*p < 32) {
+	  s+="(0x"; s+=hexa[(*p)>>4]; s+=hexa[(*p)&0xf]; s+=')';
+	} else
+	  s+=*p;
+      }
+
     now=time(0);
     strftime(z,255,"%Y-%b-%d %H:%M:%S",localtime(&now));
-    fprintf(f,"%s [%d] %s\n",z,getpid(),rm);
-    fclose(f);
+
+    f << z << "[ " << ((int) getpid()) << "] " << s << endl;
+    f.close();
   }
 }
 
@@ -695,6 +774,25 @@ void Global::addHostBookmark(HostBookmark *hbm) {
   if (bmlistener != 0) bmlistener->updateBookmarks();
 }
 
+void Global::addEngineBookmark(EngineBookmark *ebm) {
+  list<EngineBookmark *>::iterator ei;
+
+  for(ei=EnginePresets.begin();ei!=EnginePresets.end();ei++)
+    if ( (*(*ei)) == ebm ) {
+      delete ebm;
+      return;
+    }
+
+  EnginePresets.push_front(ebm);
+  if (EnginePresets.size() > 16) {
+    delete(EnginePresets.back());
+    EnginePresets.pop_back();
+  }
+    
+  writeRC();
+  if (bmlistener != 0) bmlistener->updateBookmarks();
+}
+
 void Global::updateScrollBacks() {
   output->updateScrollBack();
   updateChannelScrollBacks();
@@ -752,8 +850,61 @@ GdkFont * Global::loadFont(EboardFont i) {
 
 }
 
+void Global::setPieceSet(string &filename,bool chgPieces,bool chgSquares) {
+  string oldp,olds;
+  PieceSet *oldset=0;
+
+  if (pieceset) {
+    oldp=pieceset->getName();
+    olds=pieceset->getSquareName();
+    oldset=pieceset;
+  } else {
+    chgPieces=true;
+    chgSquares=true;
+  }
+
+  pieceset=new PieceSet(chgPieces?filename:oldp,chgSquares?filename:olds);
+  if (oldset)
+    delete oldset;
+
+  respawnPieceSet();
+}
+
+void Global::respawnPieceSet() {
+  list<PieceChangeListener *>::iterator i;
+
+  // notify all objects that use the pieceset
+  for(i=PieceClients.begin();i!=PieceClients.end();i++)
+    (*i)->pieceSetChanged();
+}
+
+void Global::addPieceClient(PieceChangeListener *pcl) {
+  global.debug("Global","addPieceClient");
+  PieceClients.push_back(pcl); 
+}
+
+void Global::removePieceClient(PieceChangeListener *pcl) {
+  list<PieceChangeListener *>::iterator i;
+
+  global.debug("Global","removePieceClient");
+
+  for(i=PieceClients.begin();
+      i!=PieceClients.end();
+      i++)
+    if ( (*i) == pcl ) {
+      PieceClients.erase(i);
+      return;
+    }
+}
+
 // ----------
 
+HostBookmark::HostBookmark() {
+  memset(host,0,128);
+  memset(protocol,0,64);
+  port=0;
+}
+
 int HostBookmark::operator==(HostBookmark *hbm) {
   if (strcmp(host,hbm->host)) return 0;
   if (port!=hbm->port) return 0;
@@ -761,13 +912,43 @@ int HostBookmark::operator==(HostBookmark *hbm) {
   return 1;
 }
 
+int EngineBookmark::operator==(EngineBookmark *ebm) {
+  if (humanwhite != ebm->humanwhite) return 0;
+  if (secspermove != ebm->secspermove) return 0;
+  if (maxply != ebm->maxply) return 0;
+  if (think != ebm->think) return 0;
+  if (proto != ebm->proto) return 0;
+  if (mode != ebm->mode) return 0;
+  if (directory.compare(ebm->directory)) return 0;
+  if (cmdline.compare(ebm->cmdline))     return 0;
+  return 1;
+}
+
+void EngineBookmark::read(tstring &t) {
+  static char *sep="^\n\r";
+
+  caption     = *(t.token(sep));
+  directory   = *(t.token(sep));
+  cmdline     = *(t.token(sep));
+  humanwhite  = t.tokenvalue(sep);
+  secspermove = t.tokenvalue(sep);
+  maxply      = t.tokenvalue(sep);
+  think       = t.tokenvalue(sep);
+  proto       = t.tokenvalue(sep);
+  mode        = (variant) t.tokenvalue(sep);
+
+  if (!directory.compare("NULL")) directory.erase();
+  if (!cmdline.compare("NULL")) cmdline.erase();
+}
+
 // -------------------------------- channel splitting
 
 IcsChannel::IcsChannel(char *s) {
-  TString t(512);
+  static char *sep="\t\r\n";
+  tstring t;
   t.set(s);
-  number = atoi( t.token("\t\r\n") );
-  name = t.token("\t\r\n");
+  number = t.tokenvalue(sep);
+  name   = * (t.token(sep));
 }
 
 void ChannelSplitter::getChannels(char *ipaddr) {
@@ -799,7 +980,7 @@ void ChannelSplitter::getChannels(char *ipaddr) {
   if (kid==0) {
 
     execlp("wget","wget","-q","-O",destname,url,0);
-    exit(0);
+    _exit(0);
 
   } else {
 
@@ -817,22 +998,21 @@ void ChannelSplitter::ZombieNotification(int pid) {
 }
 
 void ChannelSplitter::parseChannelList() {
-  FILE *f;
   char s[512];
 
   global.debug("ChannelSplitter","parseChannelList");
   channels.clear();
 
-  f=fopen( chlist.c_str(), "r" );
+  ifstream f(chlist.c_str());
   if (!f) {
     global.debug("ChannelSplitter","parseChannelList","can't read file");
     return;
   }
 
-  if (fgets(s,511,f) != 0) {
+  if (memset(s,0,512), f.getline(s,511,'\n')) {
 
     if (strstr(s,"text/ics-channel-list")) {    
-      while( fgets(s,511,f) != 0) {
+      while( memset(s,0,512), f.getline(s,511,'\n') ) {
 	if (!isdigit(s[0])) break;
 	channels.push_back( IcsChannel(s) );
       }
@@ -840,7 +1020,7 @@ void ChannelSplitter::parseChannelList() {
 
   }
 
-  fclose(f);
+  f.close();
 }
 
 const char * ChannelSplitter::getChannelTitle(int n) {
@@ -935,27 +1115,20 @@ TerminalColor::TerminalColor() {
   Background    = 0;
 }
 
-void TerminalColor::read(TString &t) {
+void TerminalColor::read(tstring &t) {
   static char *comma=",:\n\r \t";  
 
-  TextDefault   = strtol(t.token(comma), 0, 16);
-  TextBright    = strtol(t.token(comma), 0, 16);
-  PrivateTell   = strtol(t.token(comma), 0, 16);
-  NewsNotify    = strtol(t.token(comma), 0, 16);
-  Mamer         = strtol(t.token(comma), 0, 16);
-  KibitzWhisper = strtol(t.token(comma), 0, 16);
-  Shouts        = strtol(t.token(comma), 0, 16);
-  Seeks         = strtol(t.token(comma), 0, 16);
-  ChannelTell   = strtol(t.token(comma), 0, 16);
-  Engine        = strtol(t.token(comma), 0, 16);
-  Background    = strtol(t.token(comma), 0, 16);
-}
-
-void TerminalColor::write(FILE *f) {
-  fprintf(f,"%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x\n",
-	  TextDefault,TextBright,PrivateTell,NewsNotify,
-	  Mamer,KibitzWhisper,Shouts,Seeks,ChannelTell,
-	  Engine,Background);
+  TextDefault   = t.tokenvalue(comma,16);
+  TextBright    = t.tokenvalue(comma,16);
+  PrivateTell   = t.tokenvalue(comma,16);
+  NewsNotify    = t.tokenvalue(comma,16);
+  Mamer         = t.tokenvalue(comma,16);
+  KibitzWhisper = t.tokenvalue(comma,16);
+  Shouts        = t.tokenvalue(comma,16);
+  Seeks         = t.tokenvalue(comma,16);
+  ChannelTell   = t.tokenvalue(comma,16);
+  Engine        = t.tokenvalue(comma,16);
+  Background    = t.tokenvalue(comma,16);
 }
 
 // ---- desktop saving
@@ -986,16 +1159,12 @@ void WindowGeometry::setNull() {
   X=Y=W=H=0;
 }
 
-void WindowGeometry::write(FILE *f) {
-  fprintf(f,"(%d,%d,%d,%d)",X,Y,W,H);
-}
-
-void WindowGeometry::read(TString &t) {
+void WindowGeometry::read(tstring &t) {
   static char *sep=":,()\n\t\r ";
-  X=atoi(t.token(sep));
-  Y=atoi(t.token(sep));
-  W=atoi(t.token(sep));
-  H=atoi(t.token(sep));
+  X=t.tokenvalue(sep);
+  Y=t.tokenvalue(sep);
+  W=t.tokenvalue(sep);
+  H=t.tokenvalue(sep);
 }
 
 // --------
@@ -1017,45 +1186,36 @@ void Desktop::clear() {
 
   consoles.clear();
   cfilters.clear();
+
 }
 
-void Desktop::write(FILE *f) {
-  wMain.write(f);    fprintf(f,",");
-  wGames.write(f);   fprintf(f,",");
-  wLocal.write(f);   fprintf(f,",");
-  wAds.write(f);     fprintf(f,"\n");
-}
-
-void Desktop::read(TString &t) {
+void Desktop::read(tstring &t) {
   wMain.read(t);
   wGames.read(t);
   wLocal.read(t);
   wAds.read(t);
 }
 
-void Desktop::writeConsoles(FILE *f, const char *key) {
+void Desktop::writeConsoles(ostream &s, const char *key) {
   int i,j;
   j=consoles.size();
-  
   for(i=0;i<j;i++) {
-    fprintf(f,"%s::",key);
-    consoles[i]->write(f);
-    fprintf(f,"%s\n",cfilters[i]->c_str());
+    s << key << "::" << (*consoles[i]);
+    s << (*(cfilters[i])) << endl;
   }
 }
 
-void Desktop::readConsole(TString &t) {
+void Desktop::readConsole(tstring &t) {
   WindowGeometry *wg;
-  string *s;
+  string *p,*s;
   static char *sep="\n\r";
-  char *p;
-
+  
   wg=new WindowGeometry();
   wg->read(t);
 
-  s=new string();
   p=t.token(sep);
-  if (p) (*s)=p;
+  s=new string();
+  if (p) (*s)=(*p);
 
   consoles.push_back(wg);
   cfilters.push_back(s);
@@ -1121,3 +1281,79 @@ void zh_sigchild_handler(int sig) {
   if (sig == SIGCHLD)
     global.zombies.handleSigChild();
 }
+
+// ---------
+
+void ObjectCounter::create(const char * k) {
+  int i;
+  i=getIndex(k);
+  if (i>=0) {
+    N[i]++;
+    T[i]++;
+  } else {
+    names.push_back(string(k));
+    N.push_back(1);
+    T.push_back(1);
+  }
+  // cout << "+" << k << endl;
+}
+
+void ObjectCounter::destroy(const char * k) {
+  int i;
+  i=getIndex(k);
+  if (i>=0)
+    N[i]--;
+  else {
+    names.push_back(string(k));
+    N.push_back(-1);
+    T.push_back(0);
+  }
+  //  cout << "-" << k << endl;
+}
+
+void ObjectCounter::report() {
+  int i,j;
+  j=names.size();
+
+  cout << "object report:\n";
+  cout << "--------------------\n";
+  for(i=0;i<j;i++)
+    cout << names[i] << "\ttotal " << T[i] << ", rest " << N[i] << endl;
+  cout << "--------------------\n";
+}
+
+int ObjectCounter::getIndex(const char *k) {
+  int i,j;
+  j=names.size();
+  for(i=0;i<j;i++)
+    if (!strcmp(k,names[i].c_str()))
+      return i;
+  return -1;
+}
+
+Environment::Environment() {
+  char *p;
+
+  p=getenv("HOME");
+  if (p) {
+    Home=p;    
+  } else {
+    Home.erase();
+    cerr << "** eboard ** warning: HOME environment variable not set\n";
+  }
+
+  p=getenv("USER");
+  if (p) {
+    User=p;
+  } else {
+    User="Human";
+  }
+
+  if (!Home.empty()) {
+    Config=Home;
+    Config+="/.eboard/eboard.conf";
+  } else {
+    Config.erase();
+  }
+}
+
