@@ -1,4 +1,4 @@
-/* $Id: global.h,v 1.49 2002/04/08 00:44:50 bergo Exp $ */
+/* $Id: global.h,v 1.59 2002/04/27 14:43:43 bergo Exp $ */
 
 /*
 
@@ -36,6 +36,7 @@
 #include "sound.h"
 #include "seekgraph.h"
 #include "history.h"
+#include "bugpane.h"
 #include "stl.h"
 
 class Board;
@@ -51,15 +52,36 @@ class QButton;
 class SeekGraph;
 class SoundSlave;
 class StringCollection;
+class BugPane;
+class PieceSet;
+class VectorPieces;
 
 class HostBookmark {
  public:
+  HostBookmark();
   int operator==(HostBookmark *hbm);
   char host[128];
   int  port;
   char protocol[64];
 };
 
+class EngineBookmark {
+ public:
+  int operator==(EngineBookmark *ebm);
+  void read(tstring &t);
+
+  string  caption;
+  string  directory;
+  string  cmdline;
+  int     humanwhite;
+  int     secspermove;
+  int     maxply;
+  int     think;  
+  int     proto;
+  variant mode;
+
+};
+
 class ZombieHunter {
  public:
   ZombieHunter();
@@ -111,8 +133,7 @@ class ChannelSplitter : public SigChildHandler
 class TerminalColor {
  public:
   TerminalColor();
-  void read(TString &t);
-  void write(FILE *f);
+  void read(tstring &t);
 
   int TextDefault;
   int TextBright;
@@ -132,8 +153,7 @@ class WindowGeometry {
   WindowGeometry(int a,int b,int c,int d);
   WindowGeometry();
 
-  void write(FILE *f);
-  void read(TString &t);
+  void read(tstring &t);
 
   void retrieve(GtkWidget *w);
   bool isNull();
@@ -145,15 +165,15 @@ class Desktop {
  public:
 
   void clear();
-  void write(FILE *f);
-  void read(TString &t);
 
-  void writeConsoles(FILE *f, const char *key);
-  void readConsole(TString &t);
+  void read(tstring &t);
+  void readConsole(tstring &t);
+  void writeConsoles(ostream &s, const char *key);
 
   void addConsole(DetachedConsole *dc);
   void spawnConsoles(TextSet *ts);
 
+
   WindowGeometry wMain, wGames, wLocal, wAds;
 
  private:
@@ -161,7 +181,29 @@ class Desktop {
   vector<string *> cfilters;
 };
 
-class Global : public ChannelSplitter {
+class ObjectCounter {
+ public:
+  void create(const char * k);
+  void destroy(const char * k);
+  void report();
+ private:
+  int getIndex(const char *k);  
+  vector<string> names;
+  vector<int>    N, T;
+};
+
+class Environment {
+ public:
+  Environment();
+
+  string Home; 
+  string User;
+  string Config;
+};
+
+class Global : public ChannelSplitter,
+	       public ObjectCounter
+{
  public:
   Global();
   
@@ -180,6 +222,7 @@ class Global : public ChannelSplitter {
   void writeRC();
 
   void addHostBookmark(HostBookmark *hbm);
+  void addEngineBookmark(EngineBookmark *ebm);
 
   void WrappedMainIteration();
   void WrappedMainQuit();
@@ -210,12 +253,17 @@ class Global : public ChannelSplitter {
   void updateScrollBacks();
   void dropQuickbarButtons();
 
-  bool hasSoundFile(char *p);
+  bool hasSoundFile(string &p);
 
   Notebook * getNotebook();
 
   GdkFont *loadFont(EboardFont i);
 
+  void setPieceSet(string &filename,bool chgPieces,bool chgSquares);
+  void respawnPieceSet();
+  void addPieceClient(PieceChangeListener *pcl);
+  void removePieceClient(PieceChangeListener *pcl);
+
   // free with free() as usual
   static void * safeMalloc(int nbytes);
 
@@ -227,10 +275,13 @@ class Global : public ChannelSplitter {
 
   list<NetConnection *> Agents;
   list<DetachedConsole *> Consoles;
+  list<PieceChangeListener *> PieceClients;
+
+  VectorPieces       vpieces;
+  PieceSet           *pieceset;
 
   InputHandler       *input;
   OutputPane         *output;
-  TextSet            *outputset;
   NetConnection      *network;
   Status             *status;
   ProtocolParser     *protocol;
@@ -242,6 +293,10 @@ class Global : public ChannelSplitter {
   BookmarkListener   *bmlistener;
   UpdateInterface    *qbcontainer;
   QuickBar           *quickbar;
+  GtkWidget          *killbox;
+  GtkWidget          *lowernotebook;
+  GtkWidget          *mainpaned;
+  BugPane            *bugpane;
 
   StringCollection   annotator;
   SoundSlave         sndslave;
@@ -266,7 +321,6 @@ class Global : public ChannelSplitter {
   int PlainSquares;
   int LightSqColor;
   int DarkSqColor;
-  int AntialiasPieces;
 
   int ShowRating;
   int ScrollBack;
@@ -275,7 +329,7 @@ class Global : public ChannelSplitter {
   int HideSeeks;
   int BeepWhenOppMoves;
   int EnableSounds;
-  int VectorPieces;
+  int UseVectorPieces;
   int CheckLegality;
 
   int SplitChannels;
@@ -294,6 +348,7 @@ class Global : public ChannelSplitter {
   int RetrieveChannelNames;
   bool PopupHelp;
   int LowTimeWarningLimit;
+  int SmootherAnimation;
 
   TerminalColor Colors;
   Desktop Desk;
@@ -307,7 +362,8 @@ class Global : public ChannelSplitter {
 
   int Quitting;
 
-  list<HostBookmark *> HostHistory;
+  list<HostBookmark *>   HostHistory;
+  list<EngineBookmark *> EnginePresets;
 
   vector<string> SoundFiles;
   vector<QButton *> QuickbarButtons;
@@ -315,6 +371,9 @@ class Global : public ChannelSplitter {
   char * Version;
   char   SystemType[64];
   char   argv0[512];
+
+  Environment env;
+
  private:
   int createDir(char *z);
   void renumberGame(ChessGame *cg,int id);
