@@ -1,4 +1,4 @@
-/* $Id: mainwindow.cc,v 1.83 2002/04/07 04:49:21 bergo Exp $ */
+/* $Id: mainwindow.cc,v 1.91 2002/04/24 05:25:09 bergo Exp $ */
 
 /*
 
@@ -34,6 +34,7 @@
 #include <gdk/gdkkeysyms.h>
 #include "mainwindow.h"
 #include "chess.h"
+#include "bugpane.h"
 #include "global.h"
 
 #include "config.h"
@@ -66,6 +67,7 @@ static GtkItemFactoryEntry mainwindow_mainmenu[] = {
   { "/Peer/Play against Engine/Cr_afty...",NULL,GTK_SIGNAL_FUNC(peer_connect_crafty), 0, NULL },
   { "/Peer/Play against Engine/sep1",NULL,NULL,0,"<Separator>" },
   { "/Peer/Play against Engine/_Generic Engine...",NULL,GTK_SIGNAL_FUNC(peer_connect_xboard), 0,NULL },
+  { "/Peer/Engine B_ookmarks", NULL, NULL, 0,"<Branch>" },
   { "/Peer/sep3",            NULL,           NULL, 0, "<Separator>" },
   { "/Peer/_Empty Scratch Board",     NULL,  GTK_SIGNAL_FUNC(peer_scratch_empty), 0, NULL },
   { "/Peer/_Scratch Board with Initial Position",NULL,  GTK_SIGNAL_FUNC(peer_scratch_initial), 0, NULL },
@@ -130,6 +132,10 @@ MainWindow::MainWindow() {
   GtkAccelGroup *mag;
   GtkWidget *tw[10];
   GtkWidget *bhb,*whb,*dm;
+
+  GtkWidget *P,*SN, *tl1, *tl2;
+  Text *text;
+
   Board *tmp;
   int i;
   int nitems=sizeof(mainwindow_mainmenu)/sizeof(mainwindow_mainmenu[0]);
@@ -169,7 +175,7 @@ MainWindow::MainWindow() {
   searchThemes();
   global.readRC();
 
-  /* build bookmark submenu under Peer */
+  /* build ics bookmark submenu under Peer */
   updateBookmarks();
 
   /* promotion menu */
@@ -186,19 +192,62 @@ MainWindow::MainWindow() {
 
   gtk_widget_show(whb);
 
-  /* notebook */
+  /* paned widget */
+  P=gtk_vpaned_new();
+  global.mainpaned=P;
+  gtk_box_pack_start(GTK_BOX(v), P, TRUE, TRUE, 0);
+
+  /* main notebook */
   notebook=new Notebook();
   {
     GtkPositionType rqda[4]={GTK_POS_RIGHT,GTK_POS_LEFT,GTK_POS_TOP,GTK_POS_BOTTOM};
     notebook->setTabPosition(rqda[global.TabPos%4]);
   }
   notebook->show();
-  gtk_box_pack_start (GTK_BOX (v), notebook->widget, TRUE, TRUE, 0);
+
+  gtk_paned_pack1 (GTK_PANED (P), notebook->widget, TRUE, FALSE);
+
+  SN=gtk_notebook_new();
+  gtk_paned_pack2 (GTK_PANED (P), SN, TRUE, TRUE);
+
+  gtk_paned_set_position(GTK_PANED(P),7000);
+  gtk_paned_set_gutter_size(GTK_PANED(P),8);
+  gtk_paned_set_handle_size(GTK_PANED(P),8);
+
+  global.lowernotebook = SN;
+
+  /* the console set */
 
   icsout=new TextSet();
-  global.outputset=icsout;
 
-  tmp=new Board(true);
+  /* console snippet */
+
+  tl1=gtk_label_new("Console");
+  text=new Text();
+  
+  gtk_notebook_append_page(GTK_NOTEBOOK(SN), text->widget, tl1);
+  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(SN), GTK_POS_RIGHT);
+
+  text->show();
+  gtk_widget_show(tl1);
+  gtk_widget_show(SN);
+  gtk_widget_show(P);
+  
+  icsout->addTarget(text);
+
+  /* bugpane */
+
+  global.bugpane = new BugPane();
+  global.bugpane->show();
+  tl2=gtk_label_new("Bughouse");
+  gtk_notebook_append_page(GTK_NOTEBOOK(SN), global.bugpane->widget, tl2);
+  gtk_widget_show(tl2);
+
+  gtk_notebook_set_page(GTK_NOTEBOOK(SN),0);
+
+  /* main board */
+
+  tmp=new Board();
   notebook->addPage(tmp->widget,"Main Board",-1);
   tmp->setNotebook(notebook,-1);
 
@@ -289,7 +338,7 @@ MainWindow::MainWindow() {
   gtk_signal_connect(GTK_OBJECT(dm),"toggled",GTK_SIGNAL_FUNC(sett_osound),0);
 
   dm=gtk_item_factory_get_widget(gif,"/Settings/Vectorized Pieces (Faster Rendering)");
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(dm),global.VectorPieces);
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(dm),global.UseVectorPieces);
   gtk_signal_connect(GTK_OBJECT(dm),"toggled",GTK_SIGNAL_FUNC(sett_vector),0);
   vector_checkbox=dm;
 
@@ -322,6 +371,7 @@ MainWindow::MainWindow() {
   global.bmlistener=(BookmarkListener *)this;
   global.qbcontainer=(UpdateInterface *)this;
   global.quickbar=quickbar;
+  global.killbox=tw[1];
   mainw=this;
 
   notebook->setListener(this);
@@ -494,35 +544,40 @@ void MainWindow::restoreDesk() {
 }
 
 void MainWindow::parseThemeFile(char *name) {
-  FILE *f;
-  char s[256], aux[16], *p;
+  static char * comma = ",";
+  char s[256], aux[16];
   ThemeEntry *te;
   list<ThemeEntry *>::iterator it;
+  tstring t;
+  string *p;
   int ndc=0;
 
   if (name==0) return;
 
-  f=fopen(name,"r");
+  ifstream f(name);
   if (!f) return;
 
   global.debug("MainWindow","parseThemeFile",name);
 
-  while(fgets(s,256,f)) {
+  while( memset(s,0,256), f.getline(s,255,'\n') ) {
     if (s[0]=='#') continue;
     
     // sound file
     if (s[0]=='+') {
-      p=strtok(s+1,",\n");
-      if ((p)&&(!global.hasSoundFile(p)))
-	global.SoundFiles.push_back( * (new string(p)) );
+      t.set(&s[1]);
+      p=t.token(comma);
+      if ( p && !global.hasSoundFile(*p) )
+	global.SoundFiles.push_back( * (new string(*p)) );
       continue;
     }
-    p=strtok(s,",\n");
+
+    t.set(s);
+    p=t.token(comma);
     if (!p) continue;
     te=new ThemeEntry();
-    strcpy(te->Filename,p);
-    p=strtok(0,",\n");
-    strcpy(te->Text,p);
+    te->Filename=*p;
+    p=t.token(comma);
+    if (p) te->Text=*p;
     
     // avoid dupes      
     for(it=Themes.begin();it!=Themes.end();it++) {
@@ -534,21 +589,26 @@ void MainWindow::parseThemeFile(char *name) {
       if ( (*it)->isNameDupe(te) ) {
 	++ndc;
 	sprintf(aux," (%d)",ndc);
-	strcat(te->Text,aux);
+	te->Text+=aux;
       }
     }
     
     if (te)
       Themes.push_back(te);
   }
-  fclose(f);
+  f.close();
 }
 
 void MainWindow::updateBookmarks() {
   GtkWidget *pmenu, *item;
   GList *r,*s;
   list<HostBookmark *>::iterator hi;
+  list<EngineBookmark *>::iterator ei;
   char z[256];
+  int i;
+  string x;
+
+  // ics bookmarks
 
   pmenu=gtk_item_factory_get_widget(gif,"/Peer/ICS Bookmarks");
 
@@ -557,8 +617,9 @@ void MainWindow::updateBookmarks() {
     gtk_container_remove( GTK_CONTAINER(pmenu), GTK_WIDGET(s->data) );
   g_list_free(r);
 
-  for(hi=global.HostHistory.begin();hi!=global.HostHistory.end();hi++) {
-    snprintf(z,256,"Connect to %s:%d (%s)",(*hi)->host,(*hi)->port,(*hi)->protocol);
+  i=1;
+  for(hi=global.HostHistory.begin();hi!=global.HostHistory.end();hi++,i++) {
+    snprintf(z,256,"%d. Connect to %s:%d (%s)",i,(*hi)->host,(*hi)->port,(*hi)->protocol);
     z[255]=0;
     item=gtk_menu_item_new_with_label( z );
 
@@ -576,6 +637,38 @@ void MainWindow::updateBookmarks() {
     gtk_menu_append(GTK_MENU(pmenu),item);
     gtk_widget_show(item);    
   }
+
+  // engine bookmarks
+
+  pmenu=gtk_item_factory_get_widget(gif,"/Peer/Engine Bookmarks");
+
+  r=gtk_container_children(GTK_CONTAINER(pmenu));
+  for(s=r;s!=0;s=g_list_next(s))
+    gtk_container_remove( GTK_CONTAINER(pmenu), GTK_WIDGET(s->data) );
+  g_list_free(r);
+
+  i=1;
+  for(ei=global.EnginePresets.begin();ei!=global.EnginePresets.end();ei++, i++) {
+    sprintf(z,"%d. ",i);
+    x=z;
+    x+=(*ei)->caption;
+    item=gtk_menu_item_new_with_label( x.c_str() );
+
+    gtk_signal_connect(GTK_OBJECT(item),"activate",
+		       GtkSignalFunc(mainwindow_connect_bookmark2),
+		       (gpointer)(*ei));
+
+    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_widget_show(item);    
+  }
+
+  if (global.EnginePresets.empty()) {
+    item=gtk_menu_item_new_with_label("(no bookmarks)");
+    gtk_widget_set_sensitive(item,FALSE);
+    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_widget_show(item);    
+  }
+
 }
 
 void MainWindow::searchThemes() {
@@ -589,6 +682,7 @@ void MainWindow::searchThemes() {
   DIR *dh;
   struct dirent *ds;
   ExtPatternMatcher ExtraConf;
+  string s;
 
   tmenu[0]=gtk_item_factory_get_widget(gif,"/Settings/Bitmapped Piece Sets/Load Theme");
   tmenu[1]=gtk_item_factory_get_widget(gif,"/Settings/Bitmapped Piece Sets/Load Pieces Only");
@@ -597,7 +691,7 @@ void MainWindow::searchThemes() {
   j=eff.getPathCount();
 
   for(i=0;i<j;i++) {
-    strcpy(tmp,eff.getPath(i));
+    strcpy(tmp,eff.getPath(i).c_str());
     strcat(tmp,"/");
     strcat(tmp,"eboard_themes.conf");
     parseThemeFile(tmp);
@@ -622,7 +716,7 @@ void MainWindow::searchThemes() {
 
       // load all
 
-      menuitem=gtk_menu_item_new_with_label( (*it)->Text );
+      menuitem=gtk_menu_item_new_with_label( (*it)->Text.c_str() );
       gtk_signal_connect(GTK_OBJECT(menuitem),"activate",
 			 GtkSignalFunc(mainwindow_themeitem),
 			 (gpointer)(*it));
@@ -631,7 +725,7 @@ void MainWindow::searchThemes() {
 
       // load pieces
 
-      menuitem=gtk_menu_item_new_with_label( (*it)->Text );
+      menuitem=gtk_menu_item_new_with_label( (*it)->Text.c_str() );
       gtk_signal_connect(GTK_OBJECT(menuitem),"activate",
 			 GtkSignalFunc(mainwindow_themeitem2),
 			 (gpointer)(*it));
@@ -640,7 +734,7 @@ void MainWindow::searchThemes() {
 
       // load squares
 
-      menuitem=gtk_menu_item_new_with_label( (*it)->Text );
+      menuitem=gtk_menu_item_new_with_label( (*it)->Text.c_str() );
       gtk_signal_connect(GTK_OBJECT(menuitem),"activate",
 			 GtkSignalFunc(mainwindow_themeitem3),
 			 (gpointer)(*it));
@@ -648,9 +742,11 @@ void MainWindow::searchThemes() {
       gtk_widget_show(menuitem);
 
     }
-    Board::setCurrentPieceSet(Themes.front()->Filename,true,true);
-  } else
-    Board::setCurrentPieceSet("basic_w.png",true,true);
+    global.setPieceSet(Themes.front()->Filename,true,true);
+  } else {
+    s="basic_w.png";
+    global.setPieceSet(s,true,true);
+  }
 }
 
 void MainWindow::greet() {
@@ -662,6 +758,7 @@ void MainWindow::greet() {
   icsout->append("http://www.gnu.org/copyleft/gpl.html",0xffc000,IM_IGNORE);
   icsout->append("Source code available at http://eboard.sourceforge.net",0xc0ff00,IM_IGNORE);
   icsout->append("---",0xc0ff00,IM_IGNORE);
+
 }
 
 void MainWindow::setPasswordMode(int pm) {
@@ -824,6 +921,27 @@ void MainWindow::openSjeng() {
   openEngine(spp);  
 }
 
+void MainWindow::openEngineBookmark(EngineBookmark *bm) {
+  XBoardProtocolParser *xpp;
+
+  switch(bm->proto) {
+  case 0: xpp=new XBoardProtocolParser(); break;
+  case 1: xpp=new CraftyProtocolParser(); break;
+  case 2: xpp=new SjengProtocolParser(); break;
+  case 3: xpp=new GnuChess4ProtocolParser(); break;
+  default: 
+    cerr << "** [eboard] bad engine protocol # in bookmark: " << bm->proto << endl;
+    return;
+  }
+
+  disconnect();
+  if (xpp->run(bm)) {
+    // success
+    global.protocol=xpp;
+    incrementHook();
+  }
+}
+
 void MainWindow::openEngine(XBoardProtocolParser *xpp) {
   global.debug("MainWindow","openEngine");
   disconnect();
@@ -1218,8 +1336,8 @@ mainwindow_themeitem (GtkMenuItem *menuitem, gpointer data) {
   ThemeEntry *te;
   te=(ThemeEntry *)data;
   gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->vector_checkbox),0);
-  global.VectorPieces=0;
-  Board::setCurrentPieceSet(te->Filename,true,true);
+  global.UseVectorPieces=0;
+  global.setPieceSet(te->Filename,true,true);
   global.writeRC();
 }
 
@@ -1228,8 +1346,8 @@ mainwindow_themeitem2 (GtkMenuItem *menuitem, gpointer data) {
   ThemeEntry *te;
   te=(ThemeEntry *)data;
   gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->vector_checkbox),0);
-  global.VectorPieces=0;
-  Board::setCurrentPieceSet(te->Filename,true,false);
+  global.UseVectorPieces=0;
+  global.setPieceSet(te->Filename,true,false);
   global.writeRC();
 }
 
@@ -1238,8 +1356,8 @@ mainwindow_themeitem3 (GtkMenuItem *menuitem, gpointer data) {
   ThemeEntry *te;
   te=(ThemeEntry *)data;
   gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mainw->vector_checkbox),0);
-  global.VectorPieces=0;
-  Board::setCurrentPieceSet(te->Filename,false,true);
+  global.UseVectorPieces=0;
+  global.setPieceSet(te->Filename,false,true);
   global.writeRC();
 }
 
@@ -1517,7 +1635,7 @@ sett_osound(GtkWidget *w,gpointer data) {
 
 void
 sett_vector(GtkWidget *w,gpointer data) {
-  global.VectorPieces=GTK_CHECK_MENU_ITEM(w)->active;
+  global.UseVectorPieces=GTK_CHECK_MENU_ITEM(w)->active;
   global.writeRC();
   global.repaintAllBoards();
 }
@@ -1597,16 +1715,24 @@ mainwindow_connect_bookmark(GtkWidget *w, gpointer data) {
 			      strcmp(bm->protocol,"FICS") ? 0 : p);
 }
 
+void
+mainwindow_connect_bookmark2(GtkWidget *w, gpointer data) {
+  EngineBookmark *bm;
+  bm=(EngineBookmark *)data;
+
+  mainw->openEngineBookmark(bm);
+}
+
 // ThemeEntry
 
 bool ThemeEntry::isDupe(ThemeEntry *te) {
   if (!te) return false;
-  if (!strcmp(Filename,te->Filename)) return true;
+  if (!Filename.compare(te->Filename)) return true;
   return false;
 }
 
 bool ThemeEntry::isNameDupe(ThemeEntry *te) {
   if (!te) return false;
-  if (!strcmp(Text,te->Text)) return true;
+  if (!Text.compare(te->Text)) return true;
   return false;
 }
