@@ -1,4 +1,4 @@
-/* $Id: mainwindow.h,v 1.51 2002/03/17 04:53:36 bergo Exp $ */
+/* $Id: mainwindow.h,v 1.53 2002/04/24 05:25:09 bergo Exp $ */
 
 /*
 
@@ -46,13 +46,14 @@
 #include "quickbar.h"
 #include "script.h"
 #include "stl.h"
+#include "global.h"
 
 class ThemeEntry {
  public:
   bool isDupe(ThemeEntry *te);
   bool isNameDupe(ThemeEntry *te);
-  char Filename[128];
-  char Text[128];
+  string Filename;
+  string Text;
 };
 
 // ask about our multiple inheritance contest
@@ -143,6 +144,7 @@ class MainWindow : public WidgetProxy,
   void disconnect();
 
   void openEngine(XBoardProtocolParser *xpp);
+  void openEngineBookmark(EngineBookmark *bm);
   void openGnuChess4();
   void openCrafty();
   void openSjeng();
@@ -194,6 +196,7 @@ class MainWindow : public WidgetProxy,
   friend void     mainwindow_icsout_changed(GtkEditable *gtke, gpointer data);
 
   friend void     mainwindow_connect_bookmark(GtkWidget *w, gpointer data);
+  friend void     mainwindow_connect_bookmark2(GtkWidget *w, gpointer data);
 
   friend void     navbar_back_all(GtkWidget *w,gpointer data);
   friend void     navbar_back_1(GtkWidget *w,gpointer data);
