@@ -1,4 +1,4 @@
-/* $Id: movelist.h,v 1.5 2001/10/24 11:10:22 bergo Exp $ */
+/* $Id: movelist.h,v 1.6 2002/04/28 21:04:34 bergo Exp $ */
 
 /*
 
@@ -34,11 +34,11 @@
 
 class MoveListWindow : public WidgetProxy {
  public:
-  MoveListWindow(char *p1,char *p2,int gid,list<Position *> &moves,
+  MoveListWindow(char *p1,char *p2,int gid,list<Position> &moves,
 		 int over,GameResult result, char *reason);
   ~MoveListWindow();
   void setListener(MoveListListener *mll);
-  void updateList(list<Position *> &moves,
+  void updateList(list<Position> &moves,
 		  int over=0,GameResult result=UNDEF, char *reason=0);
   void close();
 
@@ -47,7 +47,7 @@ class MoveListWindow : public WidgetProxy {
   MoveListListener *listener;
   Status *textbar;
 
-  void populate_clist(list<Position *> &moves, int over,
+  void populate_clist(list<Position> &moves, int over,
 		      GameResult result,char *reason);
 
   friend void movelist_destroy (GtkWidget *w, gpointer data);
