@@ -1,4 +1,4 @@
-/* $Id: notebook.cc,v 1.17 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: notebook.cc,v 1.18 2002/04/07 17:24:23 bergo Exp $ */
 
 /*
 
@@ -29,6 +29,13 @@
 #include "notebook.h"
 
 #include "dead.xpm"
+#include "trash2.xpm"
+
+void page_trash(GtkWidget *w, gpointer data) {
+  Page *p;
+  p=(Page *) data; 
+  global.removeRemovablePage(p->number);
+}
 
 Page::Page() {
   global.debug("Page","Page","1");
@@ -41,16 +48,50 @@ Page::Page() {
   gtk_widget_show(label);
 }
 
-Page::Page(int _number,GtkWidget *_content, char *_title) {
+Page::Page(int _number,GtkWidget *_content, char *_title, bool _removable=false) {
+  
+  GtkWidget *b,*p;
+
   global.debug("Page","Page","2");
   number=_number;
   content=_content;
   title[255]=0;
   level=IM_ZERO;
   naughty=false;
+  removable=_removable;
+
   strncpy(title,_title,255);
-  label=gtk_label_new(title);
-  gtk_widget_show(label);
+
+  if (removable) {
+
+    labelframe=gtk_hbox_new(FALSE,1);
+
+    Notebook::ensurePixmaps();
+    b=gtk_button_new();
+    gtk_button_set_relief(GTK_BUTTON(b), GTK_RELIEF_HALF);
+    p=gtk_pixmap_new(Notebook::trashface, Notebook::trashmask);
+    gtk_container_add(GTK_CONTAINER(b), p);
+    gtk_box_pack_start(GTK_BOX(labelframe), b, FALSE, FALSE, 0);
+
+    label=gtk_label_new(title);
+    gtk_box_pack_start(GTK_BOX(labelframe), label, FALSE, TRUE, 1);
+
+    gtk_widget_show(p);
+    gtk_widget_show(b);
+    gtk_widget_show(label);
+    gtk_widget_show(labelframe);
+
+    gtk_signal_connect(GTK_OBJECT(b),"clicked",
+		       GTK_SIGNAL_FUNC(page_trash), (gpointer) this);
+
+  } else {
+
+    label=gtk_label_new(title);
+    gtk_widget_show(label);
+    labelframe=label;
+
+  }
+
 }
 
 int Page::operator==(int n) {
@@ -97,22 +138,40 @@ void Page::dump() {
   cerr << "level=" << level << endl;
 }
 
+GdkPixmap * Notebook::trashface=0;
+GdkBitmap * Notebook::trashmask=0;
+GtkWidget * Notebook::swidget=0;
+
+void Notebook::ensurePixmaps() {
+  GtkStyle *style;
+
+  if (swidget==0) return;
+
+  if (trashface==0) {
+    style=gtk_widget_get_style(swidget);
+    trashface = gdk_pixmap_create_from_xpm_d (swidget->window, &trashmask,
+					      &style->bg[GTK_STATE_NORMAL],
+					      (gchar **) trash2_xpm);
+  }
+}
+
 Notebook::Notebook() {
   global.debug("Notebook","Notebook");
   pcl=0;
   widget=gtk_notebook_new();
+  swidget=widget;
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(widget),GTK_POS_RIGHT);
   gtk_signal_connect(GTK_OBJECT(widget),"switch_page",
 		     GTK_SIGNAL_FUNC(notebook_switch_page),
 		     (gpointer)this);
 }
 
-void Notebook::addPage(GtkWidget *what,char *title,int id) {
+void Notebook::addPage(GtkWidget *what,char *title,int id,bool removable=false) {
   Page *pg;
   global.debug("Notebook","addPage",title);
-  pg=new Page(id,what,title);
+  pg=new Page(id,what,title,removable);
   pages.push_back(pg);  
-  gtk_notebook_append_page(GTK_NOTEBOOK(widget),what,pg->label);
+  gtk_notebook_append_page(GTK_NOTEBOOK(widget),what,pg->labelframe);
 
   /* to test graphics only, adds dead.xpm to console tab
   if (pages.size() == 2)
