@@ -1,4 +1,4 @@
-/* $Id: pieces.cc,v 1.19 2002/03/17 04:53:36 bergo Exp $ */
+/* $Id: pieces.cc,v 1.23 2002/04/27 14:43:44 bergo Exp $ */
 
 /*
 
@@ -44,34 +44,36 @@ int PReq::operator> (PReq * a) { return(y > a->y); }
 // http://www.mail-archive.com/dia-list@lysator.liu.se/msg02435.html
 //
 
-PieceSet::PieceSet(char *name, char *sqname) {
+PieceSet::PieceSet(string &name, string &sqname) {
   EboardFileFinder eff;
-  char wherever[512];
+  string wherever;
+  char cpath[512];
   int i;
 
-  global.debug("PieceSet","PieceSet",name);
+  // global.debug("PieceSet","PieceSet",name);
+  global.create("PieceSet");
 
-  if (!eff.find(name,wherever)) {
+  Name=name;
+  SqName=sqname;
+
+  if (!eff.find(Name,wherever)) {
     cerr << "<PieceSet::PieceSet> ** file not found: " << name << endl;
     exit(2);
   }
 
-  isrc=gdk_imlib_load_image(wherever);  
+  memset(cpath,0,512);
+  wherever.copy(cpath,512);
+
+  isrc=gdk_imlib_load_image(cpath);
   if (isrc==0) {
     cerr << "<PieceSet::PieceSet> ** unable to load " << wherever << endl;
     exit(2);
   }
   gdk_imlib_changed_image(isrc);
 
-  memset(Name,0,128);
-  strncpy(Name,name,127);
-
-  memset(SqName,0,128);
-  strncpy(SqName,sqname,127);
-
   // use squares from other file ?
-  if (strcmp(SqName,Name)) {
-    if (!eff.find(SqName,wherever))
+  if (Name.compare(SqName)) {
+    if (!eff.find(SqName, wherever))
       cerr << "<PieceSet::PieceSet> ** file not found: " << SqName << endl;
     else
       loadSquares(wherever);
@@ -102,8 +104,8 @@ PieceSet::PieceSet(char *name, char *sqname) {
 }
 
 PieceSet::PieceSet(PieceSet *src) {
-  strcpy(Name,src->Name);
-  strcpy(SqName,src->SqName);
+  Name=src->Name;
+  SqName=src->SqName;
   initCache();
   isrc=gdk_imlib_clone_image(src->isrc);
   gdk_imlib_changed_image(isrc);
@@ -112,8 +114,8 @@ PieceSet::PieceSet(PieceSet *src) {
   extruded=src->extruded;
   queuef=src->queuef;
   bgcolor=src->bgcolor;
-  alphamask=(rgbptr)g_malloc(side*side*14);
-  memcpy(alphamask,src->alphamask,side*side*14);
+  alphamask=(rgbptr)g_malloc(side*height*14);
+  memcpy(alphamask,src->alphamask,side*height*14);
   bottom_only=false;
   top_only=false;
   init_cheap();
@@ -132,25 +134,30 @@ PieceSet::~PieceSet() {
       delete(*qi);
     myq.clear();
   }
-
+  global.destroy("PieceSet");
 }
 
-char * PieceSet::getName() {
+string & PieceSet::getName() {
   return(Name);
 }
 
-char * PieceSet::getSquareName() {
+string & PieceSet::getSquareName() {
   return(SqName);
 }
 
-void PieceSet::loadSquares(char *path) {
+void PieceSet::loadSquares(string &path) {
   GdkImlibImage *tmp,*aux;
+  char cpath[512];
+
   int w1,w2,h1,h2;
   int x,w;
 
-  global.debug("PieceSet","loadSquares",path);
+  memset(cpath,0,512);
+  path.copy(cpath,512);
 
-  tmp=gdk_imlib_load_image(path);
+  global.debug("PieceSet","loadSquares",cpath);
+
+  tmp=gdk_imlib_load_image(cpath);
   if (tmp==0) {
     cerr << "<PieceSet::loadSquares> ** unable to load " << path << endl;
     return;
@@ -283,36 +290,370 @@ void PieceSet::scale(int newside) {
   else
     newheight = newside;
 
-  if (factor <= 0.2500) {
-    prescale(1,4);
-    goto no_more_prescale;
-  }
-  if (factor <= 0.3333) {
-    prescale(1,3);
-    goto no_more_prescale;
-  }
-  if (factor <= 0.5000) {
-    prescale(1,2);
-    goto no_more_prescale;
-  }
+  ni=gimp_scale_preserve(isrc,alphamask,newside*7,newheight*2);
 
-  no_more_prescale:
-
-  ni=gdk_imlib_clone_scaled_image(isrc,newside*7,newheight*2);
-  gdk_imlib_changed_image(ni);
   side=newside;
   height=newheight;
+
   gdk_imlib_destroy_image(isrc);
+
   isrc=ni;
   calcAlphaMask();
 
-  if (global.AntialiasPieces)
-    if (side > 35)
-      antialias_a();
-
   buildCache();
 }
 
+#define EPSILON 0.0001
+
+GdkImlibImage *
+PieceSet::gimp_scale_preserve(GdkImlibImage *src, rgbptr alpha, int nw,int nh) {
+
+  GdkImlibImage *img, *a, *b;
+  rgbptr s,d;
+  hati   p, R,G,B;
+  int i, q, z;
+  int ew,eh,missing;
+
+  a=gdk_imlib_clone_scaled_image(src,src->rgb_width,src->rgb_height);
+  
+  // make a fake rgb alpha mask
+  s=alpha;
+  d=a->rgb_data;  
+
+  i=src->rgb_width * src->rgb_height;
+  for(;i;i--) {
+    p= *(s++) << 7;
+    *(d++) = p;
+    *(d++) = p;
+    *(d++) = p;
+  }
+
+  R=G=B=0x80;
+  bgcolor=pixel(R,G,B); 
+
+  // substitute bgcolor
+  i=src->rgb_width * src->rgb_height;
+  d=src->rgb_data;
+  z=pixel(*d,*(d+1),*(d+2));
+  for(;i;i--) {
+      q=pixel(*d,*(d+1),*(d+2));
+      if (q==z) *d=*(d+1)=*(d+2)=R; 
+      d+=3;
+  }
+
+  img=gimp_scale(src,nw,nh);
+  b=gimp_scale(a, nw, nh);
+  gdk_imlib_destroy_image(a);
+  // gdk_imlib_save_image_to_ppm(b, "debug2.ppm");
+
+  // fix the last column before the squares
+  d=img->rgb_data + 3* ((nw/7)*6 - 1);
+  for(i=0;i<nh;i++) {
+    *d     = R;
+    *(d+1) = G;
+    *(d+2) = B;
+    d+=3*nw;
+  }
+  
+  s=b->rgb_data;
+  d=img->rgb_data;
+  i=nw * nh * 3;
+
+  for(;i;i-=3) {
+    if ( (*s) > 0x60 ) {
+      *(d) = R;
+      *(d+1) = G;
+      *(d+2) = B;
+    }
+    s+=3;
+    d+=3;
+  }
+
+  // fix annoying border (d)effect on extruded sets
+  if (extruded) {
+    ew=nw/7;
+    eh=nh/2;
+
+    i=ew-1;
+    missing=0;
+
+    d=img->rgb_data+ 3* (nw*i + 6*ew + 2);
+    while(i>0) {
+      q=pixel(*d,*(d+1),*(d+2));
+
+      if (q!=bgcolor)
+	break;
+
+      ++missing;
+      --i;
+      d-=3*nw;
+    }
+
+    if (missing < ew/2) {
+      bitblt( img->rgb_data, img->rgb_data, // !!!
+	      6*ew, ew-(2*missing), nw,       // src
+	      6*ew, ew-missing    , nw,       // dest
+	      ew, missing );
+      
+      bitblt( img->rgb_data, img->rgb_data, // !!!
+	      6*ew, eh+ew-(2*missing), nw,     //src
+	      6*ew, eh+ew-missing, nw,       //dest
+	      ew, missing );
+    }
+
+  }
+
+  gdk_imlib_destroy_image(b);
+  gdk_imlib_changed_image(img);
+  //  gdk_imlib_save_image_to_ppm(img, "debug.ppm");
+
+  return(img);
+}
+
+// taken from gimp 1.0.4, scale_region(...) from app/paint_funcs.c
+GdkImlibImage * PieceSet::gimp_scale(GdkImlibImage *gsrc,int nw,int nh) {
+  
+
+  unsigned char * src_m1, * src, * src_p1, * src_p2;
+  unsigned char * s_m1, * s, * s_p1, * s_p2;
+  unsigned char * dest, * d;
+  double * row, * r;
+  int src_row, src_col;
+  int bytes, b;
+  int width, height;
+  int orig_width, orig_height;
+  double x_rat, y_rat;
+  double x_cum, y_cum;
+  double x_last, y_last;
+  double * x_frac, y_frac, tot_frac;
+  float dx, dy;
+  int i, j;
+  int frac;
+  int advance_dest_x, advance_dest_y;
+  int minus_x, plus_x, plus2_x;
+  int scale_type;
+
+  GdkImlibImage *img;
+  int stride;
+
+  orig_width = gsrc->rgb_width;
+  orig_height = gsrc->rgb_height;
+
+  width = nw;
+  height = nh;
+
+  img=gdk_imlib_clone_scaled_image(gsrc, nw, nh);  
+
+  /*  Some calculations...  */
+  bytes = 3;
+  stride= orig_width*bytes;
+
+  /*  the data pointers...  */
+  src_m1 = (unsigned char *) g_malloc (orig_width * bytes);
+  src    = (unsigned char *) g_malloc (orig_width * bytes);
+  src_p1 = (unsigned char *) g_malloc (orig_width * bytes);
+  src_p2 = (unsigned char *) g_malloc (orig_width * bytes);
+  dest   = (unsigned char *) g_malloc (width * bytes);
+
+  /*  find the ratios of old x to new x and old y to new y  */
+  x_rat = (double) orig_width / (double) width;
+  y_rat = (double) orig_height / (double) height;
+
+  /*  determine the scale type  */
+  if (x_rat < 1.0 && y_rat < 1.0)
+    scale_type = 0; // MagnifyX_MagnifyY
+  else if (x_rat < 1.0 && y_rat >= 1.0)
+    scale_type = 1; // MagnifyX_MinifyY
+  else if (x_rat >= 1.0 && y_rat < 1.0)
+    scale_type = 2; // MinifyX_MagnifyY
+  else
+    scale_type = 3; // MinifyX_MinifyY
+
+  /*  allocate an array to help with the calculations  */
+  row    = (double *) g_malloc (sizeof (double) * width * bytes);
+  x_frac = (double *) g_malloc (sizeof (double) * (width + orig_width));
+
+  /*  initialize the pre-calculated pixel fraction array  */
+  src_col = 0;
+  x_cum = (double) src_col;
+  x_last = x_cum;
+
+  for (i = 0; i < width + orig_width; i++)
+    {
+      if (x_cum + x_rat <= (src_col + 1 + EPSILON))
+	{
+	  x_cum += x_rat;
+	  x_frac[i] = x_cum - x_last;
+	}
+      else
+	{
+	  src_col ++;
+	  x_frac[i] = src_col - x_last;
+	}
+      x_last += x_frac[i];
+    }
+  
+  /*  clear the "row" array  */
+  memset (row, 0, sizeof (double) * width * bytes);
+  
+  /*  counters...  */
+  src_row = 0;
+  y_cum = (double) src_row;
+  y_last = y_cum;
+
+  /*  Get the first src row  */
+  memcpy(src, gsrc->rgb_data + src_row * stride, stride);
+
+  /*  Get the next two if possible  */
+  if (src_row < (orig_height - 1))
+    memcpy(src_p1, gsrc->rgb_data + (src_row + 1) * stride, stride);
+
+  if ((src_row + 1) < (orig_height - 1))
+    memcpy(src_p2, gsrc->rgb_data + (src_row + 2) * stride, stride);
+
+  /*  Scale the selected region  */
+  i = height;
+  while (i)
+    {
+      src_col = 0;
+      x_cum = (double) src_col;
+
+      /* determine the fraction of the src pixel we are using for y */
+      if (y_cum + y_rat <= (src_row + 1 + EPSILON))
+	{
+	  y_cum += y_rat;
+	  dy = y_cum - src_row;
+	  y_frac = y_cum - y_last;
+	  advance_dest_y = TRUE;
+	}
+      else
+	{
+	  y_frac = (src_row + 1) - y_last;
+	  dy = 1.0;
+	  advance_dest_y = FALSE;
+	}
+
+      y_last += y_frac;
+
+      s = src;
+      s_m1 = (src_row > 0) ? src_m1 : src;
+      s_p1 = (src_row < (orig_height - 1)) ? src_p1 : src;
+      s_p2 = ((src_row + 1) < (orig_height - 1)) ? src_p2 : s_p1;
+
+      r = row;
+
+      frac = 0;
+      j = width;
+
+      while (j)
+	{
+	  if (x_cum + x_rat <= (src_col + 1 + EPSILON))
+	    {
+	      x_cum += x_rat;
+	      dx = x_cum - src_col;
+	      advance_dest_x = TRUE;
+	    }
+	  else
+	    {
+	      dx = 1.0;
+	      advance_dest_x = FALSE;
+	    }
+	  
+	  tot_frac = x_frac[frac++] * y_frac;
+
+	  minus_x = (src_col > 0) ? -bytes : 0;
+	  plus_x = (src_col < (orig_width - 1)) ? bytes : 0;
+	  plus2_x = ((src_col + 1) < (orig_width - 1)) ? bytes * 2 : plus_x;
+
+	  switch (scale_type)
+	    {
+	    case 0: // MagnifyX_MagnifyY
+	      for (b = 0; b < bytes; b++)
+		r[b] += ((1 - dy) * ((1 - dx) * s[b] + dx * s[b+plus_x]) +
+			 dy  * ((1 - dx) * s_p1[b] + dx * s_p1[b+plus_x])) * tot_frac;
+	      break;
+	    case 1: // MagnifyX_MinifyY
+	      for (b = 0; b < bytes; b++)
+		r[b] += (s[b] * (1 - dx) + s[b+plus_x] * dx) * tot_frac;
+	      break;
+	    case 2: // MinifyX_MagnifyY
+	      for (b = 0; b < bytes; b++)
+		r[b] += (s[b] * (1 - dy) + s_p1[b] * dy) * tot_frac;
+	      break;
+	    case 3: //MinifyX_MinifyY
+	      for (b = 0; b < bytes; b++)
+		r[b] += s[b] * tot_frac;
+	      break;
+	    }
+
+	  if (advance_dest_x)
+	    {
+	      r += bytes;
+	      j--;
+	    }
+	  else
+	    {
+	      s_m1 += bytes;
+	      s    += bytes;
+	      s_p1 += bytes;
+	      s_p2 += bytes;
+	      src_col++;
+	    }
+	}
+
+      if (advance_dest_y)
+	{
+	  tot_frac = 1.0 / (x_rat * y_rat);
+
+	  /*  copy "row" to "dest"  */
+	  d = dest;
+	  r = row;
+
+	  j = width;
+	  while (j--)
+	    {
+	      b = bytes;
+	      while (b--)
+		*d++ = (unsigned char) (*r++ * tot_frac);
+	    }
+
+	  /*  set the pixel region span  */
+	  memcpy(img->rgb_data + (width * 3) * (height-i), dest, width * 3);
+
+	  /*  clear the "row" array  */
+	  memset (row, 0, sizeof (double) * width * bytes);
+
+	  i--;
+	}
+      else
+	{
+	  /*  Shuffle pointers  */
+	  s = src_m1;
+	  src_m1 = src;
+	  src = src_p1;
+	  src_p1 = src_p2;
+	  src_p2 = s;
+
+	  src_row++;
+	  if ((src_row + 1) < (orig_height - 1))
+	    memcpy(src_p2, gsrc->rgb_data + (src_row + 2) * stride, stride);
+	}
+    }
+
+  /*  free up temporary arrays  */
+  g_free (row);
+  g_free (x_frac);
+  g_free (src_m1);
+  g_free (src);
+  g_free (src_p1);
+  g_free (src_p2);
+  g_free (dest);
+
+  gdk_imlib_changed_image(img);
+  return img;
+}
+
+
 void PieceSet::calcAlphaMask() {
   int i,j,c;
   unsigned char r,g,b;
@@ -430,36 +771,35 @@ void PieceSet::drawPieceAndSquare(piece p,rgbptr dest,
 
 void PieceSet::bitblt(rgbptr src,rgbptr dest,int sx,int sy,int sw,
 	    int dx,int dy,int dw,int w,int h) {
-  register rgbptr s, d;
+  register rgbptr d, s;
   register int wcd;
-  static int hcd,slg,dlg;
-  s=src+3*(sw*sy+sx);
+  static int slg,dlg, wcd0;
   d=dest+3*(dw*dy+dx);
-  slg=3*(sw-w);
+  s=src+3*(sw*sy+sx);
   dlg=3*(dw-w);
-  hcd=h;
-  while(hcd) {
-    wcd=3*w;
+  slg=3*(sw-w);
+  wcd0=3*w;
+  while(h) {
+    wcd=wcd0;
     while(wcd--)
       *(d++)=*(s++);
     d+=dlg;
     s+=slg;
-    --hcd;
+    --h;
   }
 }
 
 void PieceSet::bitblt_a(rgbptr src,rgbptr dest,int sx,int sy,int sw,
 			int dx,int dy,int dw,int w,int h) {
-  register rgbptr s, d, a;
-  static int hcd,wcd,slg,dlg,alg;
+  register rgbptr d, s, a;
+  static int wcd,slg,dlg,alg;
   s=src+3*(sw*sy+sx);
   d=dest+3*(dw*dy+dx);
   a=alphamask+(sw*sy+sx);
   slg=3*(sw-w);
   dlg=3*(dw-w);
   alg=sw-w;
-  hcd=h;
-  while(hcd) {
+  while(h) {
     wcd=w;
     while(wcd--) {
       if (*a) {
@@ -475,7 +815,7 @@ void PieceSet::bitblt_a(rgbptr src,rgbptr dest,int sx,int sy,int sw,
     d+=dlg;
     s+=slg;
     a+=alg;
-    --hcd;
+    --h;
   }
 }
 
@@ -489,85 +829,6 @@ void PieceSet::rect_outline(rgbptr img,int x,int y,int w,int h,int iw) {
   }  
 }
 
-void PieceSet::prescale(int pixto,int pixfrom) {
-  int nw,nh,nside,ss,ds;
-  rgbptr nrgb;
-  rgbptr src,dest,sl,dl;
-  int i,j,oi,oj,w,tw,k,a,b,c,ex,ey;
-  int sx,sy,dx,dy;
-  GdkImlibImage *ni;
-
-  //  cerr << "prescaling " << pixfrom << "->" << pixto << endl;
-
-  nside=(side*pixto)/pixfrom;
-
-  //  cerr << "prescale side = " << nside << endl;
-
-  nw=nside*7;
-  nh=nside*2;
-
-  nrgb=(rgbptr) g_malloc(nw*nh*3);
-
-  src=isrc->rgb_data;
-  dest=nrgb;
-
-  sx=0; sy=0; dx=0; dy=0;
-  ss=isrc->rgb_width * 3;
-  ds=nw * 3;
-
-  k=pixfrom*pixfrom;
-
-  for(dy=0,sy=0;dy<nh;dy+=pixto,sy+=pixfrom) {
-    sl=src+sy*ss;
-    dl=dest+dy*ds;
-    for(dx=0,sx=0;dx<nw;dx+=pixto,sx+=pixfrom) {
-      
-      for(oi=0;oi<pixto;oi++)
-	for(oj=0;oj<pixto;oj++) {
-
-	  a=b=c=0;
-	  tw=0;
-
-	  for(i=0;i<pixfrom;i++) // X
-	    for(j=0;j<pixfrom;j++) { // Y
-
-	      ex=(6-abs(i-oi));
-	      ex*=ex;
-	      ey=(6-abs(j-oj));
-	      ey*=ey;
-
-	      w=cheap_sqrt(ex+ey);
-	      tw+=w;
-
-	      a+=w*(int)(sl[ss*j+(sx+i)*3]);
-	      b+=w*(int)(sl[ss*j+(sx+i)*3+1]);
-	      c+=w*(int)(sl[ss*j+(sx+i)*3+2]);
-	    }
-
-	  a/=tw;
-	  b/=tw;
-	  c/=tw;      
-
-	  dl[ds*oj+(dx+oi)*3]=(unsigned char)a;
-	  dl[ds*oj+(dx+oi)*3+1]=(unsigned char)b;
-	  dl[ds*oj+(dx+oi)*3+2]=(unsigned char)c;	  
-	  
-	} // oj    
-    } // dx
-  } // dy
-  
-  //  cerr << "sy=" << sy << "   dy=" << dy << "  nh=" << nh << endl;
-
-  // done
-
-  ni=gdk_imlib_create_image_from_data(dest,0,nw,nh);
-  gdk_imlib_changed_image(ni);
-  gdk_imlib_destroy_image(isrc);
-  isrc=ni;
-  side=nside;
-  g_free(nrgb);
-}
-
 int PieceSet::cheap_sqrt(int x) {
   return (cheap_table[x]);
 }
@@ -666,51 +927,6 @@ int PieceSet::alpha(int x,int y) {
   return( (int)(a[0]) );
 }
 
-void PieceSet::antialias_a() {
-  int i,j,k,m;
-  int w,h;
-  int d0,d1,d2,e;
-
-  static int gaussian[3][3]={ {1,2,1}, {2,4,2}, {1,2,1} };
-  int weight;
-  rgbptr dest;
-
-  w=isrc->rgb_width;
-  h=isrc->rgb_height;
-
-  dest=(rgbptr) malloc(3*w*h);
-
-  for(j=1;j<h-1;j++)
-    for(i=1;i<w-1;i++) {
-      if (alpha(i,j)) continue;
-
-      weight=0;
-      d0=0;
-      d1=0;
-      d2=0;
-      for(k=0;k<3;k++)
-	for(m=0;m<3;m++) {
-	  if (!alpha(i+m-1,j+k-1)) {
-	    weight+=gaussian[m][k];
-	    e = getPixel(i+m-1, j+k-1);
-	    d0 += gaussian[m][k] * pixr(e);
-	    d1 += gaussian[m][k] * pixg(e);
-	    d2 += gaussian[m][k] * pixb(e);
-	  }
-	}
-      
-      if (d0<0) d0=0;
-      if (d1<0) d1=0;
-      if (d2<0) d2=0;
-      if (weight==0) weight=1;
-      e= pixel ( d0/weight, d1/weight, d2/weight );
-      xSetPixel(dest, i, j, e);
-    }
-
-  memcpy(isrc->rgb_data, dest, 3*w*h);
-  free(dest);
-}
-
 void PieceSet::queueRequest(PReq *r) {
   list<PReq *>::iterator qi, qj;
 
@@ -760,6 +976,8 @@ VectorPieces::VectorPieces() {
   int i,j;
   orig_sqside=cur_sqside=108;
 
+  global.create("VectorPieces");
+
   np[0]=32; np[1]=67; np[2]=90; np[3]=76; np[4]=65; np[5]=46;
 
   for(i=0;i<6;i++) {
@@ -810,6 +1028,10 @@ VectorPieces::VectorPieces() {
   rescale(108);
 }
 
+VectorPieces::~VectorPieces() {
+  global.destroy("VectorPieces");
+}
+
 void VectorPieces::drawSquares(GdkPixmap *dest,GdkGC *gc,int sqside, int dx=0, int dy=0) {
   int i,j;
   for(i=0;i<8;i++)
