@@ -1,4 +1,4 @@
-/* $Id: position.cc,v 1.35 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: position.cc,v 1.39 2002/05/02 00:59:23 bergo Exp $ */
 
 /*
 
@@ -123,11 +123,13 @@ char Position::AnnotationBuffer[512];
 
 Position::Position() {
   setStartPos();
-  LastMove[0]=0;
+  LastMove.erase();
+  global.create("Position");
 }
 
 Position::~Position() {
   dropAnnotations();
+  global.destroy("Position");
 }
 
 void Position::dropAnnotations() {
@@ -191,7 +193,7 @@ Position Position::operator=(Position &p) {
     maycastle[i]=p.maycastle[i];
   ep[0]=p.ep[0];
   ep[1]=p.ep[1];
-  strcpy(LastMove,p.LastMove);
+  LastMove = p.LastMove;
 
   dropAnnotations();
   for(ai=p.Annotations.begin();ai!=p.Annotations.end();ai++) {
@@ -241,11 +243,14 @@ void Position::invalidate() {
 }
 
 void Position::setLastMove(char *s) {
-  LastMove[31]=0;
-  strncpy(LastMove,s,32);
+  LastMove=s;
 }
 
-char * Position::getLastMove() {
+void Position::setLastMove(string &s) {
+  LastMove=s;
+}
+
+string & Position::getLastMove() {
   return(LastMove);
 }
 
@@ -294,11 +299,11 @@ void Position::moveAnyNotation(char *m,piece color,variant Vr=REGULAR) {
       moveCartesian(m[0]-'a',m[1]-'1',x=(m[2]-'a'),y=(m[3]-'1'),Vr);
       if (m[4]!=0) { // promotion
 	switch(m[4]) {
-	case 'q': pr=QUEEN; break;
-	case 'r': pr=ROOK; break;
-	case 'n': pr=KNIGHT; break;
-	case 'b': pr=BISHOP; break;
-	case 'k': pr=KING; break;
+	case 'q': case 'Q': pr=QUEEN; break;
+	case 'r': case 'R': pr=ROOK; break;
+	case 'n': case 'N': pr=KNIGHT; break;
+	case 'b': case 'B': pr=BISHOP; break;
+	case 'k': case 'K': pr=KING; break;
 	default: return;
 	}
 	square[x][y]=pr|color|WASPAWN;
@@ -549,7 +554,10 @@ void Position::moveDrop(piece p,int x2,int y2) {
   }
 }
 
-void Position::moveCartesian(int x1,int y1,int x2,int y2,variant Vr=REGULAR) {
+void Position::moveCartesian(int x1,int y1,int x2,int y2,
+			     variant Vr=REGULAR, 
+			     bool resolvepromotion=false)
+{
   piece color;
 
   // castling
@@ -587,6 +595,21 @@ void Position::moveCartesian(int x1,int y1,int x2,int y2,variant Vr=REGULAR) {
   square[x2][y2]=square[x1][y1];
   square[x1][y1]=EMPTY;
 
+  // sometimes we don't want promotions solved by the GUI widget,
+  // like when loading moves from a PGN file. In these cases,
+  // lack of indication of the piece is a gross error in notation.
+  if (resolvepromotion) {
+
+    if (
+	( (y2==0) && (square[x2][y2] == (PAWN|BLACK) ) ) ||
+	( (y2==7) && (square[x2][y2] == (PAWN|WHITE) ) )
+       ) 
+      {
+	square[x2][y2]&=COLOR_MASK;
+	square[x2][y2]|=WASPAWN | global.promotion->getPiece();
+      }      
+  }
+
   if ((Vr!=WILD)&&(Vr!=SUICIDE))
     checkCastlingPossibility();
 }
@@ -938,13 +961,12 @@ void  Position::intersection(Position &b) {
     }
 }
 
-char * Position::getHouseString() {
+string & Position::getHouseString() {
   // [PPPNNB] - [NBQ]
   int i,j,e=1;
-  char *p;
   static char xlate[6]="PRNBQ";  
 
-  memset(HouseString,0,64);
+  HouseString.erase();
 
   for(i=0;i<5;i++)
     for(j=0;j<2;j++)
@@ -955,26 +977,26 @@ char * Position::getHouseString() {
 
   if (e) return(HouseString);
 
-  p=HouseString;
-  *(p++)='[';
+  HouseString+='[';
   
   for(i=0;i<5;i++)
     for(j=0;j<House[i][0];j++)
-      *(p++)=xlate[i];
+      HouseString+=xlate[i];
   
-  *(p++)=']'; *(p++)=' '; *(p++)='-'; *(p++)=' '; *(p++)='[';
+  HouseString+="] - [";
 
   for(i=0;i<5;i++)
     for(j=0;j<House[i][1];j++)
-      *(p++)=xlate[i];
+      HouseString+=xlate[i];
 
-  *(p++)=']';
+  HouseString+=']';
   return(HouseString);
 }
 
-char * Position::getMaterialString(variant Vr=REGULAR) {
+string & Position::getMaterialString(variant Vr=REGULAR) {
   int w=0,b=0,i,j,v;
-  
+  char tmp[64];
+
   for(i=0;i<8;i++)
     for(j=0;j<8;j++) {
 
@@ -1000,7 +1022,8 @@ char * Position::getMaterialString(variant Vr=REGULAR) {
 	b+=v;
     }
 
-  sprintf(MaterialString,"Material: %d - %d",w,b);
+  sprintf(tmp,"Material: %d - %d",w,b);
+  MaterialString=tmp;
   return(MaterialString);
 }
 
@@ -1224,7 +1247,7 @@ bool Position::isMate(piece c,variant Vr=REGULAR) {
 
 // FEN
 
-const char * Position::getFEN() {
+string & Position::getFEN() {
   static string fen;
   int i,j,ec;
   piece p,c;
@@ -1283,7 +1306,7 @@ const char * Position::getFEN() {
   fen+=" ";
   
   fen+="0 1";
-  return(fen.c_str());
+  return(fen);
 }
 
 void Position::setFEN(const char *fen) {
