@@ -1,4 +1,4 @@
-/* $Id: position.h,v 1.22 2002/02/10 13:43:40 bergo Exp $ */
+/* $Id: position.h,v 1.24 2002/05/02 00:41:04 bergo Exp $ */
 
 /*
 
@@ -86,15 +86,16 @@ class Position {
   void  setPiece(int col,int row,piece p);
   piece getPiece(int col,int row);  
   void  setLastMove(char *s);
-  char *getLastMove();
-  char *getMaterialString(variant Vr=REGULAR);
-  char *getHouseString();
+  void  setLastMove(string &s);
+  string &getLastMove();
+  string &getMaterialString(variant Vr=REGULAR);
+  string &getHouseString();
 
   void addAnnotation(int id);
   char *getAnnotation();
   void clearAnnotation();  
 
-  const char *getFEN();
+  string &getFEN();
   void setFEN(const char *fen);
 
   int  getStockCount(piece p);
@@ -108,7 +109,8 @@ class Position {
 
   void moveAnyNotation(char *m,piece color,variant Vr=REGULAR);
   void moveStdNotation(char *m,piece color,variant Vr=REGULAR);
-  void moveCartesian(int x1,int y1,int x2,int y2,variant Vr=REGULAR);
+  void moveCartesian(int x1,int y1,int x2,int y2,variant Vr=REGULAR,
+		     bool resolvepromotion=false);
   void moveDrop(piece p,int x2,int y2);
 
   void stdNotationForMove(int x1,int y1,int x2,int y2,piece prom,char *m,
@@ -124,14 +126,15 @@ class Position {
   bool isDropLegal(piece p,int x1,int y1,piece mycolor,variant vari);
 
   bool maycastle[4]; // 0=white short, 1=black short, 2=white long, 3=black long
-  int  ep[2]; // enpassant square (ep[0]=-1 for none, ep[0]=-2 for allow any)
+  char ep[2]; // enpassant square (ep[0]=-1 for none, ep[0]=-2 for allow any)
 
  private:
-  piece square[8][8];
-  char LastMove[32];
-  char MaterialString[32];
-  char HouseString[64];
-  int  House[5][2]; // {crazy,bug}house stock (P,R,N,B,Q)
+  piece  square[8][8];
+  string LastMove;
+  string MaterialString;
+  string HouseString;
+
+  short int House[5][2]; // {crazy,bug}house stock (P,R,N,B,Q)
   list<int> Annotations;
 
   static char AnnotationBuffer[512];
