@@ -1,4 +1,4 @@
-/* $Id: proto_crafty.cc,v 1.22 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: proto_crafty.cc,v 1.24 2002/04/24 05:25:09 bergo Exp $ */
 
 /*
 
@@ -45,16 +45,21 @@ char * CraftyProtocolParser::getDialogName() {
 }
 
 void CraftyProtocolParser::readDialog() {
-  char *p;
   XBoardProtocolParser::readDialog();
   resolvePaths();
   sprintf(EngineCommandLine,"crafty bookpath=%s logpath=%s tbpath=%s",
 	  BookPath,LogPath,LogPath);
-  p=getenv("HOME");
-  if (p)
-    sprintf(EngineRunDir,"%s/.eboard/craftylog",p);
+  if (!global.env.Home.empty())
+    sprintf(EngineRunDir,"%s/.eboard/craftylog",global.env.Home.c_str());
   else
     strcpy(EngineRunDir,"/tmp");
+
+  if (ebm) {
+    ebm->proto     = 1;
+    ebm->cmdline   = EngineCommandLine;
+    ebm->directory = EngineRunDir;
+    makeBookmarkCaption();
+  }
 }
 
 void CraftyProtocolParser::initEngine() {
@@ -68,8 +73,10 @@ void CraftyProtocolParser::resolvePaths() {
   char z[256],zz[256],*p;
 
   // book path
-  p=getenv("HOME");
-  if (p) sprintf(z,"%s/.eboard",p); else strcpy(z,"/tmp");
+  if (!global.env.Home.empty())
+    sprintf(z,"%s/.eboard",global.env.Home.c_str());
+  else
+    strcpy(z,"/tmp");
 
   ff.addMyDirectory(".eboard");
   ff.addMyDirectory(".");
@@ -87,8 +94,10 @@ void CraftyProtocolParser::resolvePaths() {
   strcpy(BookPath,zz);
 
   // log path
-  p=getenv("HOME");
-  if (p) sprintf(z,"%s/.eboard/craftylog",p); else strcpy(z,"/tmp");
+  if (!global.env.Home.empty())
+    sprintf(z,"%s/.eboard/craftylog",global.env.Home.c_str());
+  else
+    strcpy(z,"/tmp");
   strcpy(LogPath,z);
 }
 
