@@ -1,4 +1,4 @@
-/* $Id: proto_fics.cc,v 1.69 2002/03/29 14:27:25 bergo Exp $ */
+/* $Id: proto_fics.cc,v 1.77 2002/04/28 21:04:34 bergo Exp $ */
 
 /*
 
@@ -63,6 +63,8 @@ FicsProtocolParser::FicsProtocolParser() {
   lastGLC=0;
   lastALC=0;
 
+  PartnerGame = -1;
+
   xxnext=0;
   xxplayer[0][0]=0;
   xxplayer[1][0]=0;
@@ -154,6 +156,9 @@ FicsProtocolParser::FicsProtocolParser() {
   // bughouse stuff
   PTell.set("%a (your partner) tells you: *");
 
+  // sample output: Your partner is playing game 75 (aglup vs. cglup).
+  GotPartnerGame.set("Your partner is playing game %n (*");
+
   // challenge
   Challenge.set("Challenge: *");
 
@@ -188,7 +193,7 @@ FicsProtocolParser::FicsProtocolParser() {
   Binder.add(&Kibitz,&Whisper,&Say,&Shout,&cShout,&It,&ChannelTell,
 	     &Seek,&Notify,&SgClear,&SgAdd,&SgAdd2,&SgRemove,
 	     &PTell,&Say2,&Challenge,&LectureBotXTell,
-	     &LectureBotXTellContinued,0);
+	     &LectureBotXTellContinued,&GotPartnerGame,0);
 }
 
 FicsProtocolParser::~FicsProtocolParser() {
@@ -440,6 +445,10 @@ void FicsProtocolParser::receiveString(char *netstring) {
       goto just_output;
     }
 
+    // bughouse partner game
+    if (GotPartnerGame.match())
+      prepareBugPane();
+
     msgcolor=global.Colors.TextDefault;
     
     if (netstring[0]=='\\')
@@ -462,7 +471,7 @@ void FicsProtocolParser::receiveString(char *netstring) {
     if (PTell.match()) {
       msgcolor=global.Colors.PrivateTell; ispersonal=1;
       global.privatelyTold();
-      global.BoardList.front()->addBugText(PTell.getStarToken(0));
+      global.bugpane->addBugText(PTell.getStarToken(0));
       goto just_output;
     }
 
@@ -584,8 +593,8 @@ void FicsProtocolParser::createExaminedGame(int gameid,char *whitep,char *blackp
   b->reset();
   cg->setBoard(b);
   b->setGame(cg);
-  b->setCanMove(1);
-  b->setSensitive(1);
+  b->setCanMove(true);
+  b->setSensitive(true);
 
   sprintf(z,"Exam.Game #%d",cg->GameNumber);
   global.ebook->addPage(b->widget,z,cg->GameNumber);
@@ -627,8 +636,8 @@ void FicsProtocolParser::createScratchPos(int gameid,char *whitep,char *blackp)
   b->reset();
   cg->setBoard(b);
   b->setGame(cg);
-  b->setCanMove(0);
-  b->setSensitive(0);
+  b->setCanMove(false);
+  b->setSensitive(false);
 
   sprintf(z,"Pos: %s vs. %s",whitep,blackp);
   global.ebook->addPage(b->widget,z,cg->GameNumber);
@@ -694,15 +703,10 @@ void FicsProtocolParser::createGame(ExtPatternMatcher &pm) {
   global.prependGame(cg);
   global.BoardList.front()->reset();
 
-  if (cg->Variant==BUGHOUSE)
-    global.BoardList.front()->openBugPane();
-  else
-    global.BoardList.front()->closeBugPane();
-
   cg->setBoard(global.BoardList.front());
   global.BoardList.front()->setGame(cg);
   global.BoardList.front()->pop();
-  global.BoardList.front()->setCanMove(1);
+  global.BoardList.front()->setCanMove(true);
   global.BoardList.front()->repaint();
 
   cg->acknowledgeInfo();
@@ -712,6 +716,7 @@ void FicsProtocolParser::createGame(ExtPatternMatcher &pm) {
 void FicsProtocolParser::addGame(ExtPatternMatcher &pm) {
   ChessGame *cg;
   Board *b;
+  bool is_partner_game=false;
   char tab[96];
 
   global.debug("FicsProtocolParser","addGame");
@@ -732,17 +737,25 @@ void FicsProtocolParser::addGame(ExtPatternMatcher &pm) {
   b->reset();
   cg->setBoard(b);
   b->setGame(cg);
-  b->setCanMove(0);
-  b->setSensitive(0);
+  b->setCanMove(false);
+  b->setSensitive(false);
+
+  if (cg->GameNumber == PartnerGame) {
+    is_partner_game=true;
+    PartnerGame = -1;
+    cg->protodata[2] = 1;
+  }
 
   sprintf(tab,"Game #%d",cg->GameNumber);
   global.ebook->addPage(b->widget,tab,cg->GameNumber);
   b->setNotebook(global.ebook,cg->GameNumber);
 
-  if (global.PopupSecondaryGames) {
-    b->pop();
-    b->repaint();
-  }
+  if (!is_partner_game)
+    if (global.PopupSecondaryGames) {
+      b->pop();
+      b->repaint();
+    }
+
   cg->acknowledgeInfo();
 
   // to gather info about time, variant, player names...
@@ -807,6 +820,10 @@ void FicsProtocolParser::innerRemoveGame(int gameid) {
   if (!cg)
     return;
 
+  // bughouse partner game ?
+  if (cg->protodata[2])
+    global.bugpane->stopClock();
+
   b=cg->getBoard();
   if (b==0) return;
   global.ebook->removePage(gameid);
@@ -883,7 +900,7 @@ void FicsProtocolParser::attachHouseStock() {
   if (!cg)
     return;
 
-  pos=cg->getLastPosition();  
+  pos=&(cg->getLastPosition());
 
   pos->clearStock();
   p=Style12House.getStarToken(0);
@@ -905,9 +922,9 @@ void FicsProtocolParser::attachHouseStock() {
 
 void FicsProtocolParser::parseStyle12(char *data) {
   static char *sep=" \t\n";
-  TString t(data);
+  tstring t;
   int i,j,n;
-  char *tp;
+  string *sp;
   Position pos;
   int blacktomove;
   int game;
@@ -922,47 +939,47 @@ void FicsProtocolParser::parseStyle12(char *data) {
 
   global.debug("FicsProtocolParser","parseStyle12");
 
-  tp=t.token(sep); // <12>
+  t.set(data);
+  t.token(sep); // <12>
   
   // the position itself
   for(i=7;i>=0;i--) {
-    tp=t.token(sep);
+    sp=t.token(sep);
     for(j=0;j<8;j++)
-      pos.setPiece(j,i,style12table[tp[j]]);
+      pos.setPiece(j,i,style12table[sp->at(j)]);
   }
 
-  tp=t.token(sep); // B / W (color to move)
-  blacktomove=(tp[0]=='B');
+  sp=t.token(sep); // B / W (color to move)
+  blacktomove=(sp->at(0)=='B');
 
-  tp=t.token(sep); // double pawn push
-
-  pos.ep[0]=atoi(tp);
+  pos.ep[0]=t.tokenvalue(sep); // double pawn push
   if (blacktomove)
     pos.ep[1]=2;
   else
     pos.ep[1]=5;
 
-  tp=t.token(sep); // white castle short
-  pos.maycastle[0]=atoi(tp)?true:false;
+  // white castle short
+  pos.maycastle[0]=t.tokenvalue(sep)?true:false;
   
-  tp=t.token(sep); // white castle long
-  pos.maycastle[2]=atoi(tp)?true:false;
+  // white castle long
+  pos.maycastle[2]=t.tokenvalue(sep)?true:false;
 
-  tp=t.token(sep); // black castle short
-  pos.maycastle[1]=atoi(tp)?true:false;
+  // black castle short
+  pos.maycastle[1]=t.tokenvalue(sep)?true:false;
 
-  tp=t.token(sep); // black castle long
-  pos.maycastle[3]=atoi(tp)?true:false;
+  // black castle long
+  pos.maycastle[3]=t.tokenvalue(sep)?true:false;
 
   t.token(sep); // mv count for draw
 
-  game=atoi(t.token(sep)); // game number
-  
-  strncpy(plyr[0],t.token(sep),64); // white's name
-  strncpy(plyr[1],t.token(sep),64); // black's name
-  plyr[0][63]=plyr[1][63]=0;
+  game=t.tokenvalue(sep); // game number
 
-  rel=atoi(t.token(sep)); // relationship
+  memset(plyr[0],0,64);
+  memset(plyr[1],0,64);
+  t.token(sep)->copy(plyr[0],63); // white's name
+  t.token(sep)->copy(plyr[1],63); // black's name
+
+  rel=t.tokenvalue(sep); // relationship
 
   switch(rel) {
   case 2: // I am the examiner of this game
@@ -990,23 +1007,25 @@ void FicsProtocolParser::parseStyle12(char *data) {
     break;
   }
   
-  itime=atoi(t.token(sep)); // initial time (minutes)
-  inc=atoi(t.token(sep));   // increment (secs)
+  itime=t.tokenvalue(sep); // initial time (minutes)
+  inc=t.tokenvalue(sep);   // increment (secs)
   
-  str[0]=atoi(t.token(sep));  // white strength
-  str[1]=atoi(t.token(sep));  // black strength
+  str[0]=t.tokenvalue(sep);  // white strength
+  str[1]=t.tokenvalue(sep);  // black strength
 
-  clk[0]=atoi(t.token(sep));  // white rem time
-  clk[1]=atoi(t.token(sep));  // black rem time
+  clk[0]=t.tokenvalue(sep);  // white rem time
+  clk[1]=t.tokenvalue(sep);  // black rem time
 
-  movenum=atoi(t.token(sep)); // move # to be made
+  movenum=t.tokenvalue(sep); // move # to be made
 
   t.token(sep);               // verbose for previous move
 
-  strcpy(stra,t.token(sep));  // time taken for previous move
-  strcpy(strb,t.token(sep));  // pretty print for previous move
+  memset(stra,0,64);
+  memset(strb,0,64);
+  t.token(sep)->copy(stra,63); // time taken for previous move
+  t.token(sep)->copy(strb,63); // pretty print for previous move
 
-  flip=atoi(t.token(sep));    // flip board ?
+  flip=t.tokenvalue(sep);    // flip board ?
  
   refgame=global.getGame(game);
   if (!refgame) {
@@ -1090,8 +1109,8 @@ void FicsProtocolParser::parseStyle12(char *data) {
     global.opponentMoved();
 
   if (rel!=0) refgame->flipHint(flip);
-  if (rel==1) refgame->enableMoving(1);
-  if (rel==-1) refgame->enableMoving(0);
+  if (rel==1) refgame->enableMoving(true);
+  if (rel==-1) refgame->enableMoving(false);
 }
 
 void FicsProtocolParser::updateGame(ExtPatternMatcher &pm) {
@@ -1158,6 +1177,12 @@ void FicsProtocolParser::updateGame(ExtPatternMatcher &pm) {
     global.network->writeLine(z);
     RetrievingMoves=1;
   }
+
+  // bughouse partner game, set up names in the bugpane
+  if (refgame->protodata[2]) {
+    global.bugpane->setPlayerNames(refgame->PlayerName[0],
+				   refgame->PlayerName[1]);
+  }
 }
 
 void FicsProtocolParser::gameOver(ExtPatternMatcher &pm, GameResult result) {
@@ -1241,27 +1266,21 @@ void FicsProtocolParser::clearMoveBuffer() {
   list<Position *>::iterator li;
 
   global.debug("FicsProtocolParser","clearMoveBuffer");
-
-  for(li=MoveBuf.begin();li!=MoveBuf.end();li++)
-    delete(*li);
   MoveBuf.clear();
 }
 
 void FicsProtocolParser::retrieve1(ExtPatternMatcher &pm) {
-  Position *p,*q;
   char z[32],mv[16];
   char tmp[2][32];
   int itmp;
+  Position p,q;
 
   global.debug("FicsProtocolParser","retrieve1");
 
-  p=new Position();
   if (!MoveBuf.empty())
-    (*p)=*(MoveBuf.back());
-  else {
-    MoveBuf.push_back(p);
-    p=new Position();
-  }
+    p=MoveBuf.back();
+  else
+    MoveBuf.push_back(Position());
 
   itmp=atoi(pm.getNToken(0));
   strncpy(tmp[0],pm.getRToken(0),32);
@@ -1269,14 +1288,12 @@ void FicsProtocolParser::retrieve1(ExtPatternMatcher &pm) {
   sprintf(z,"%.2d. %s %s",itmp,tmp[0],tmp[1]);
   strcpy(mv,tmp[0]);
 
-  p->moveStdNotation(mv,WHITE);
-  p->setLastMove(z);
+  p.moveStdNotation(mv,WHITE);
+  p.setLastMove(z);
 
   MoveBuf.push_back(p);
 
   q=p;
-  p=new Position();
-  (*p)=(*q);
 
   itmp=atoi(pm.getNToken(0));
   strncpy(tmp[0],pm.getRToken(2),32);
@@ -1284,23 +1301,22 @@ void FicsProtocolParser::retrieve1(ExtPatternMatcher &pm) {
   sprintf(z,"%.2d. ... %s %s",itmp,tmp[0],tmp[1]);
   strcpy(mv,tmp[0]);
 
-  p->moveStdNotation(mv,BLACK);
-  p->setLastMove(z);
+  q.moveStdNotation(mv,BLACK);
+  q.setLastMove(z);
 
-  MoveBuf.push_back(p);
+  MoveBuf.push_back(q);
 }
 
 void FicsProtocolParser::retrieve2(ExtPatternMatcher &pm) {
-  Position *p;
+  Position p;
   char z[32],mv[16];
   char tmp[2][32];
   int itmp;
 
   global.debug("FicsProtocolParser","retrieve2");
 
-  p=new Position();
   if (!MoveBuf.empty())
-    (*p)=*(MoveBuf.back());
+    p=MoveBuf.back();
 
   itmp=atoi(pm.getNToken(0));
   strncpy(tmp[0],pm.getRToken(0),32);
@@ -1308,8 +1324,8 @@ void FicsProtocolParser::retrieve2(ExtPatternMatcher &pm) {
   sprintf(z,"%.2d. %s %s",itmp,tmp[0],tmp[1]);
   strcpy(mv,tmp[0]);
 
-  p->moveStdNotation(mv,WHITE);
-  p->setLastMove(z);
+  p.moveStdNotation(mv,WHITE);
+  p.setLastMove(z);
 
   MoveBuf.push_back(p);
 }
@@ -1460,7 +1476,7 @@ void FicsProtocolParser::ensureSeekGraph() {
   if (!global.skgraph) {
     global.skgraph=new SeekGraph();
     global.skgraph->show();
-    global.ebook->addPage(global.skgraph->widget,"Seek Graph",-3);
+    global.ebook->addPage(global.skgraph->widget,"Seek Table",-3);
     global.skgraph->setNotebook(global.ebook,-3);
   }
 }
@@ -1525,12 +1541,13 @@ void FicsProtocolParser::seekAdd(ExtPatternMatcher &pm) {
 }
 
 void FicsProtocolParser::seekRemove(ExtPatternMatcher &pm) {
-  TString t(256);
-  char *p;
+  tstring t;
+  int i;
+  t.setFail(-1);
   t.set(pm.getStarToken(0));
   ensureSeekGraph();
-  while(p=t.token(" \t\n"))
-    global.skgraph->remove(atoi(p));
+  while( (i=t.tokenvalue(" \t\n")) >= 0 )
+    global.skgraph->remove(i);
 }
 
 vector<string *> * FicsProtocolParser::getPlayerActions() {
@@ -1588,3 +1605,9 @@ void FicsProtocolParser::doGameAction(int gameid, int id) {
   }
 }
 
+void FicsProtocolParser::prepareBugPane() {
+  char z[64];
+  PartnerGame = atoi(GotPartnerGame.getNToken(0));
+  sprintf(z,"observe %d",PartnerGame);
+  global.network->writeLine(z);
+}
