@@ -1,4 +1,4 @@
-/* $Id: proto_gnuchess4.cc,v 1.6 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: proto_gnuchess4.cc,v 1.7 2002/04/24 05:25:09 bergo Exp $ */
 
 /*
 
@@ -69,6 +69,13 @@ void GnuChess4ProtocolParser::readDialog() {
   XBoardProtocolParser::readDialog();
   strcpy(EngineCommandLine,"gnuchessx");
   EngineRunDir[0]=0;
+
+  if (ebm) {
+    ebm->proto=3;
+    ebm->cmdline=EngineCommandLine;
+    ebm->directory=EngineRunDir;
+    makeBookmarkCaption();
+  }
 }
 
 char * GnuChess4ProtocolParser::getDialogName() {
