@@ -1,4 +1,4 @@
-/* $Id: proto_sjeng.cc,v 1.5 2002/03/11 14:00:35 bergo Exp $ */
+/* $Id: proto_sjeng.cc,v 1.7 2002/04/24 05:25:09 bergo Exp $ */
 
 /*
 
@@ -117,6 +117,11 @@ void SjengProtocolParser::readDialog() {
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(varbutton[2]))) Variant=SUICIDE;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(varbutton[3]))) Variant=LOSERS;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(varbutton[4]))) Variant=GIVEAWAY;
+
+  if (ebm) {
+    ebm->proto=2;
+    makeBookmarkCaption();
+  }
 }
 
 char * SjengProtocolParser::getDialogName() {
@@ -157,6 +162,6 @@ void SjengProtocolParser::sendDrop(piece p,int x,int y) {
 
   global.network->writeLine(mymove);
   advanceMove();
-  global.BoardList.front()->setCanMove(0);  
+  global.BoardList.front()->setCanMove(false);
 }
 
