@@ -1,4 +1,4 @@
-/* $Id: proto_xboard.cc,v 1.19 2002/03/29 05:23:37 bergo Exp $ */
+/* $Id: proto_xboard.cc,v 1.22 2002/05/02 00:41:04 bergo Exp $ */
 
 /*
 
@@ -37,6 +37,8 @@
 #include "chess.h"
 #include "status.h"
 #include "util.h"
+#include "stl.h"
+#include "tstring.h"
 
 XBoardProtocolParser::XBoardProtocolParser() {
   EngineWhite=0;
@@ -74,6 +76,8 @@ XBoardProtocolParser::XBoardProtocolParser() {
 
   CurrentPosition.setStartPos();
   LegalityBackup.setStartPos();
+
+  ebm=0;
 }
 
 // the "dialect" for game ends
@@ -119,7 +123,7 @@ void XBoardProtocolParser::receiveString(char *netstring) {
 			    EngineWhite?CLOCK_UNCHANGED:0,
 			    "illegal move!");
     global.status->setText(netstring);
-    global.BoardList.front()->setCanMove(1);
+    global.BoardList.front()->setCanMove(true);
     return;
   }
 
@@ -144,7 +148,7 @@ void XBoardProtocolParser::receiveString(char *netstring) {
 			    EngineWhite?0:CLOCK_UNCHANGED,
 			    b, true);
     advanceMove();
-    global.BoardList.front()->setCanMove(1);
+    global.BoardList.front()->setCanMove(true);
     global.opponentMoved();
     return;
   }
@@ -193,7 +197,7 @@ void XBoardProtocolParser::sendMove(int x1,int y1,int x2,int y2,int prom) {
   pp = prom ? global.promotion->getPiece() : EMPTY;
 
   CurrentPosition.stdNotationForMove(x1,y1,x2,y2,pp,mymove,refgame->Variant);
-  CurrentPosition.moveCartesian(x1,y1,x2,y2,Variant);
+  CurrentPosition.moveCartesian(x1,y1,x2,y2,Variant,true);
 
   if (EngineWhite)
     sprintf(ppfmm,"%d. ... %s",MoveNumber,mymove);
@@ -225,7 +229,7 @@ void XBoardProtocolParser::sendMove(int x1,int y1,int x2,int y2,int prom) {
 
   global.network->writeLine(move);
   advanceMove();
-  global.BoardList.front()->setCanMove(0);
+  global.BoardList.front()->setCanMove(false);
 }
 
 void XBoardProtocolParser::finalize() {
@@ -300,11 +304,25 @@ int XBoardProtocolParser::run() {
     destroyDialog();
     return 0;
   }
+
+  ebm=0;
+
   readDialog();
+
+  if (ebm!=0)
+    global.addEngineBookmark(ebm);
+
+  ebm=0;
+
   destroyDialog();
   return(loadEngine());
 }
 
+int XBoardProtocolParser::run(EngineBookmark *bm) {  
+  loadBookmark(bm);
+  return(loadEngine());
+}
+
 int XBoardProtocolParser::loadEngine() {
   PipeConnection *link;
   global.debug("XBoardProtocolParser","loadEngine",EngineCommandLine);
@@ -365,21 +383,17 @@ void XBoardProtocolParser::initEngine() {
 
 void XBoardProtocolParser::createGame() {
   ChessGame *cg;
-  char *p;
   Position *pos;
   global.debug("XBoardProtocolParser","createGame");
 
   MoveNumber=1;
   MoveBlack=0;
 
-  p=getenv("USER");
-  if (!p) p="Human";
-
   cg=new ChessGame();
   cg->GameNumber=XBOARD_GAME;
   cg->source = GS_Engine;
   strcpy(cg->PlayerName[EngineWhite?0:1],getComputerName());
-  strcpy(cg->PlayerName[EngineWhite?1:0],p);
+  strcpy(cg->PlayerName[EngineWhite?1:0],global.env.User.c_str());
 
   cg->MyColor=EngineWhite?BLACK:WHITE;
 
@@ -393,10 +407,10 @@ void XBoardProtocolParser::createGame() {
   cg->setBoard(global.BoardList.front());
   global.BoardList.front()->setGame(cg);
   global.BoardList.front()->pop();
-  global.BoardList.front()->setCanMove(1);
+  global.BoardList.front()->setCanMove(true);
   global.BoardList.front()->repaint();
   if (EngineWhite)
-    global.BoardList.front()->setFlipped(1);
+    global.BoardList.front()->setFlipped(true);
   cg->acknowledgeInfo();
   cg->fireWhiteClock(0,0);
 
@@ -523,6 +537,11 @@ void XBoardProtocolParser::createDialog() {
   gtk_table_attach_defaults(GTK_TABLE(tabl),think,0,2,2,3);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(think), 1);
 
+  bookmark=gtk_check_button_new_with_label("Add to Peer/Engine Bookmarks menu");
+  gtk_box_pack_start(GTK_BOX(v),bookmark,FALSE,FALSE,2);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bookmark), 1);
+
+  gtk_widget_show(bookmark);
   gtk_widget_show(think);
   gtk_widget_show(te1);
   gtk_widget_show(tl2);
@@ -584,6 +603,7 @@ void XBoardProtocolParser::createDialog() {
   // bottom box
   hs=gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(v),hs,FALSE,TRUE,4);
+
   bhb=gtk_hbutton_box_new();
   gtk_button_box_set_layout(GTK_BUTTON_BOX(bhb), GTK_BUTTONBOX_END);
   gtk_button_box_set_spacing(GTK_BUTTON_BOX(bhb), 5);
@@ -619,6 +639,7 @@ void XBoardProtocolParser::createDialog() {
 }
 
 void XBoardProtocolParser::readDialog() {
+
   EngineWhite=0;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(engctl_ewhite)))
     EngineWhite=1;
@@ -627,6 +648,60 @@ void XBoardProtocolParser::readDialog() {
   strncpy(EngineCommandLine,gtk_entry_get_text(GTK_ENTRY(engctl_engcmd)),512);
   strncpy(EngineRunDir,gtk_entry_get_text(GTK_ENTRY(engctl_engdir)),256);
   ThinkAlways=(bool)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(think));
+
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(bookmark))) {
+    ebm=new EngineBookmark();
+    ebm->secspermove=SecsPerMove;
+    ebm->maxply     =MaxDepth;
+    ebm->humanwhite =1-EngineWhite;
+    ebm->think      =ThinkAlways?1:0;
+    ebm->directory  =EngineRunDir;
+    ebm->cmdline    =EngineCommandLine;
+    ebm->mode       =REGULAR;
+    ebm->proto      =0; // base xboard protocol
+
+    makeBookmarkCaption();
+
+    // addition is done in run() to allow all virtual versions of this
+    // to run before doing it
+
+  } else {
+    ebm=0; // derivated classes should check ebm!=0 before doing oddities
+  }
+}
+
+void XBoardProtocolParser::makeBookmarkCaption() {
+  tstring t,u;
+  string x, *p;
+  char caption[1024];
+
+  if (!ebm) return;
+
+  t.set(EngineCommandLine);
+  x=EngineCommandLine;
+  if (x.empty()) x="<blank?>";
+  while ( ( p=t.token("/") ) != 0 ) {
+    u.set(*p);
+    x=*(u.token(" "));
+  }
+  sprintf(caption,"Play %s as %s vs. %s (%d secs/move, maxdepth %d, think always: %s)",
+	  ChessGame::variantName(ebm->mode),
+	  ebm->humanwhite?"White":"Black",
+	  x.c_str(),
+	  ebm->secspermove,
+	  ebm->maxply,
+	  ebm->think?"yes":"no");
+  ebm->caption = caption;
+}
+
+void XBoardProtocolParser::loadBookmark(EngineBookmark *bm) {
+  SecsPerMove = bm->secspermove;
+  MaxDepth    = bm->maxply;
+  ThinkAlways = (bm->think!=0);
+  EngineWhite = 1-bm->humanwhite;
+  Variant     = bm->mode;
+  strcpy(EngineRunDir, bm->directory.c_str() );
+  strcpy(EngineCommandLine, bm->cmdline.c_str() );
 }
 
 int XBoardProtocolParser::runDialog() {
