@@ -1,4 +1,4 @@
-/* $Id: quickbar.cc,v 1.3 2002/04/07 04:49:21 bergo Exp $ */
+/* $Id: quickbar.cc,v 1.5 2002/04/23 07:55:09 bergo Exp $ */
 
 /*
 
@@ -27,6 +27,7 @@
 #include <string.h>
 #include "quickbar.h"
 #include "global.h"
+#include "stl.h"
 
 #include "qb1.xpm"
 #include "qb2.xpm"
@@ -266,6 +267,9 @@ QuickBarSetupDialog::QuickBarSetupDialog() : ModalDialog("Shortcut Button Setup"
 
   setDismiss(GTK_OBJECT(cancel),"clicked");
 
+  for(i=0;i<10;i++)
+    iconValue[i]=0;
+
   for(i=0;i<global.QuickbarButtons.size();i++) {
     gtk_entry_set_text(GTK_ENTRY(bcap[i]), global.QuickbarButtons[i]->caption.c_str());
     gtk_entry_set_text(GTK_ENTRY(bcom[i]), global.QuickbarButtons[i]->command.c_str());
@@ -299,6 +303,11 @@ QButton::QButton(char *a, char *b, int c) {
   icon=c;
 }
 
+ostream &operator<<(ostream &s, QButton q) {
+  s << q.icon << ':' << q.caption << ':' << q.command;
+  return(s);
+}
+
 // ------------------------------
 
 GdkPixmap * QuickBarIcons::arrow[6]={0,0,0,0,0,0};
