@@ -1,4 +1,4 @@
-/* $Id: script.cc,v 1.5 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: script.cc,v 1.8 2002/04/24 01:16:33 bergo Exp $ */
 
 /*
 
@@ -33,6 +33,12 @@
 #include <sys/stat.h>
 #include "script.h"
 #include "global.h"
+#include "stl.h"
+
+#include "spiral0.xpm"
+#include "spiral1.xpm"
+#include "spiral2.xpm"
+#include "spiral3.xpm"
 
 ScriptList::ScriptList() : ModalDialog("Script List") {
   GtkWidget *v,*sw,*bh;
@@ -102,18 +108,16 @@ ScriptList::ScriptList() : ModalDialog("Script List") {
 
 void ScriptList::refresh() {
   DIR *sd;
-  char *p, z[256],w[256],desc[256],tmp[512],*dp[2];
+  char z[256],w[256],desc[256],tmp[512],*dp[2];
   struct dirent *de;
   struct stat statd;
   mode_t mode;
-  FILE *f;
 
   gtk_clist_freeze(GTK_CLIST(clist));
   gtk_clist_clear(GTK_CLIST(clist));
 
-  p=getenv("HOME");
-  if (p) {
-    sprintf(z,"%s/.eboard/scripts",p);
+  if (!global.env.Home.empty()) {
+    sprintf(z,"%s/.eboard/scripts",global.env.Home.c_str());
     sd=opendir(z);
     if (sd) {
       while(de=readdir(sd)) {
@@ -124,22 +128,22 @@ void ScriptList::refresh() {
 	    if (geteuid()==statd.st_uid) {
 	      strncpy(w,de->d_name,255);
 	      // grab description
-	      f=fopen(tmp,"r");
+	      ifstream f(tmp);
 	      if (!f) continue;
-	      if (fgetc(f)!='#') {
-		fclose(f);
+	      if (f.get()!='#') {
+		f.close();
 		desc[0]=0;
 	      } else {
-		fgets(desc,255,f);
+		f.getline(desc,255,'\n');
 		do {
-		  if (!fgets(desc,255,f))
+		  memset(desc,0,256);
+		  if (!f.getline(desc,255,'\n'))
 		    break;
 		} while(desc[0]!='#');
-		fclose(f);
+		f.close();
 		if (desc[0]=='#') {
 		  strcpy(tmp,desc);
 		  strcpy(desc,tmp+1);
-		  desc[strlen(desc)-1]=0;
 		} else {
 		  desc[0]=0;
 		}
@@ -190,24 +194,48 @@ void script_unselect(GtkCList *cl, gint row, gint column, GdkEventButton *eb,
 
 // -------------------------
 
-ScriptInstance::ScriptInstance(char *name) : WidgetProxy() {
-  GtkWidget *v,*l,*h,*k;
-  char z[256],fp[512],*p;
-  FILE *f;
+bool ScriptInstance::pixmapsok=false;
+GdkPixmap * ScriptInstance::spir[4];
+GdkBitmap * ScriptInstance::smask[4];
 
-  p=getenv("HOME");
-  if (!p) {
+void ScriptInstance::initPixmaps() {
+  GtkStyle *style;
+  if (pixmapsok) return;
+
+  style=gtk_widget_get_style(global.killbox);
+
+  spir[0] = gdk_pixmap_create_from_xpm_d (global.killbox->window, &smask[0],
+					  &style->bg[GTK_STATE_NORMAL],
+					  (gchar **) spiral0_xpm);
+  spir[1] = gdk_pixmap_create_from_xpm_d (global.killbox->window, &smask[1],
+					  &style->bg[GTK_STATE_NORMAL],
+					  (gchar **) spiral1_xpm);
+  spir[2] = gdk_pixmap_create_from_xpm_d (global.killbox->window, &smask[2],
+					  &style->bg[GTK_STATE_NORMAL],
+					  (gchar **) spiral2_xpm);
+  spir[3] = gdk_pixmap_create_from_xpm_d (global.killbox->window, &smask[3],
+					  &style->bg[GTK_STATE_NORMAL],
+					  (gchar **) spiral3_xpm);
+  pixmapsok=true;  
+}
+
+ScriptInstance::ScriptInstance(char *name) : WidgetProxy() {
+  GtkWidget *h, *l, *b;
+  char z[256],fp[512],*p;
+
+  if (global.env.Home.empty()) {
     delete this;
     return;
   }
-  sprintf(fp,"%s/.eboard/scripts/%s",p,name);
 
-  f=fopen(fp,"r");
+  sprintf(fp,"%s/.eboard/scripts/%s",global.env.Home.c_str(),name);
+
+  ifstream f(fp);
   if (!f) {
     delete this;
     return;
   }
-  fclose(f);
+  f.close();
 
   child=new PipeConnection("/bin/sh","-c",fp,0,0);
   child->Quiet=1;
@@ -218,49 +246,57 @@ ScriptInstance::ScriptInstance(char *name) : WidgetProxy() {
   }
   global.addAgent(child);
 
-  widget=gtk_window_new(GTK_WINDOW_DIALOG);
-  gtk_window_set_title(GTK_WINDOW(widget),"Script");
-  gtk_window_set_position(GTK_WINDOW(widget),GTK_WIN_POS_CENTER);
-  gtk_container_set_border_width(GTK_CONTAINER(widget),6);
-  gtk_widget_realize(widget);
+  if (!pixmapsok) initPixmaps();
 
-  sprintf(z,"Script %s running",name);
+  widget=gtk_frame_new(0);
+  gtk_container_set_border_width(GTK_CONTAINER(widget),2);
+  gtk_frame_set_shadow_type(GTK_FRAME(widget), GTK_SHADOW_ETCHED_IN);
+  h=gtk_hbox_new(FALSE,0);
+  gtk_container_set_border_width(GTK_CONTAINER(h),2);
 
-  v=gtk_vbox_new(FALSE,4);
+  runner=gtk_pixmap_new(spir[0],smask[0]);
+  sprintf(z,"running %s",name);
   l=gtk_label_new(z);
-  k=gtk_button_new_with_label("Kill");
-  h=gtk_hbox_new(FALSE,2);
-  
-  gtk_container_add(GTK_CONTAINER(widget),v);
-  gtk_box_pack_start(GTK_BOX(v),l,FALSE,FALSE,2);
-  gtk_box_pack_start(GTK_BOX(v),h,FALSE,FALSE,2);
-  gtk_box_pack_end(GTK_BOX(h),k,FALSE,FALSE,2);
+  b=gtk_button_new_with_label("Kill");
 
-  gtk_signal_connect(GTK_OBJECT(widget),"delete_event",
-		     GTK_SIGNAL_FUNC(scripti_delete),(gpointer)this);
-  gtk_signal_connect(GTK_OBJECT(k),"clicked",
+  gtk_container_add(GTK_CONTAINER(widget), h);
+  gtk_box_pack_start(GTK_BOX(h), runner, FALSE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(h), l, FALSE, TRUE, 2);
+  gtk_box_pack_start(GTK_BOX(h), b, FALSE, TRUE, 0);
+
+  gtk_widget_show(b);
+  gtk_widget_show(l);
+  gtk_widget_show(runner);
+  gtk_widget_show(h);
+  gtk_box_pack_start(GTK_BOX(global.killbox), widget, FALSE, TRUE, 0);
+
+  gtk_signal_connect(GTK_OBJECT(b),"clicked",
 		     GTK_SIGNAL_FUNC(scripti_kill),(gpointer)this);
 
-  gtk_widget_show(k);
-  gtk_widget_show(h);
-  gtk_widget_show(l);
-  gtk_widget_show(v);
-
+  frame=0;
   toid=gtk_timeout_add(500,scripti_check,(gpointer)this);
+  anid=gtk_timeout_add(150,scripti_anim,(gpointer)this);
   show();
 }
 
-gint scripti_delete (GtkWidget * widget, GdkEvent * event, gpointer data) {
+gboolean scripti_anim(gpointer data) {
+  ScriptInstance *me;
+  me=(ScriptInstance *)data;
+  me->frame++;
+  me->frame%=4;
+  gtk_pixmap_set(GTK_PIXMAP(me->runner), me->spir[me->frame], me->smask[me->frame]);
   return TRUE;
 }
 
 void scripti_kill(GtkWidget *w,gpointer data) {
   ScriptInstance *me;
   me=(ScriptInstance *)data;
+  gtk_timeout_remove(me->anid);
   gtk_timeout_remove(me->toid);
   global.removeAgent(me->child);
   me->child->close();
-  gtk_widget_destroy(me->widget);
+  gtk_container_remove(GTK_CONTAINER(global.killbox), me->widget);
+  //gtk_widget_destroy(me->widget);
   delete me->child;
   delete me;
 }
