@@ -11,12 +11,18 @@
 #include <string>
 #include <algorithm>
 
+#include <iostream>
+#include <fstream>
+
 #ifdef NEED_STL_HACK
 using std::list;
 using std::vector;
 using std::deque;
 using std::stack;
 using std::string;
+
+using std::ofstream;
+using std::ifstream;
 #endif
 
 #endif
