@@ -1,10 +1,10 @@
-/* $Id: text.h,v 1.19 2001/10/01 01:56:04 bergo Exp $ */
+/* $Id: text.h,v 1.20 2002/04/23 07:55:09 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -48,9 +48,8 @@ class OutputPane {
 
 class Searchable {
  public:
-  Searchable();
   virtual void execSearch()=0;
-  char SearchString[128];
+  string SearchString;
 };
 
 class TextFilter {
