@@ -1,4 +1,4 @@
-/* $Id: util.cc,v 1.16 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: util.cc,v 1.18 2002/04/24 01:16:33 bergo Exp $ */
 
 /*
 
@@ -29,7 +29,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include "global.h"
 #include "prefix.h"
+#include "stl.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -40,9 +42,6 @@ FileFinder::FileFinder() {
 }
 
 FileFinder::~FileFinder() {
-  vector<char *>::iterator i;
-  for(i=path.begin();i!=path.end();i++)
-    free(*i);
   path.clear();
 }
 
@@ -50,69 +49,77 @@ int FileFinder::getPathCount() {
   return(path.size());
 }
 
-char * FileFinder::getPath(int i) {
+string &FileFinder::getPath(int i) {
+  static string dot(".");
   if ((i<0)||(i>=path.size()))
-    return(0);
+    return(dot);
   return(path[i]);
 }
 
-void FileFinder::addDirectory(char *dir) {
-  char *s;
-  s=(char *)malloc(strlen(dir)+1);
-  strcpy(s,dir);
-  path.push_back(s);
-}
+void FileFinder::addDirectory(char *dir)   { path.push_back(string(dir)); }
+void FileFinder::addDirectory(string &dir) { path.push_back(string(dir)); }
 
 void FileFinder::addMyDirectory(char *dir) {
-  char *s,*h;
-  h=getenv("HOME");
-  s=(char *)malloc(strlen(h)+strlen(dir)+2);
-  strcpy(s,h);
-  strcat(s,"/");
-  strcat(s,dir);
+  string s;
+  s=global.env.Home; s+='/'; s+=dir;  
   path.push_back(s);
 }
 
-int FileFinder::find(char *name,char *fullpath) {
-  FILE *f;
-  vector<char *>::iterator i;
-  char fullname[512];
-  f=::fopen(name,"r");
+void FileFinder::addMyDirectory(string &dir) {
+  string s;
+  s=global.env.Home; s+='/'; s+=dir;  
+  path.push_back(s);
+}
+
+int FileFinder::find(string &name, string &out) {
+  int i,j;
+  string fullname;
+
+  ifstream f(name.c_str());
   if (!f) {
-    for(i=path.begin();i!=path.end();i++) {
-      strcpy(fullname,*i);
-      strcat(fullname,"/");
-      strcat(fullname,name);
-      f=::fopen(fullname,"r");
-      if (f!=NULL) {
-	strcpy(fullpath,fullname);
-	fclose(f);
+    j=path.size();
+    for(i=0;i<j;i++) {
+      fullname = path[i];
+      fullname += '/';
+      fullname += name;
+
+      ifstream g(fullname.c_str());
+      if (g) {
+	out=fullname;
+	g.close();
 	return 1;
       }
     }
     return 0;
   }
-  fclose(f);
-  strcpy(fullpath,name);
-  return 1;
+  f.close();
+  out=name;
+  return 1;  
 }
 
-FILE * FileFinder::fopen(char *name,char *mode) {
-  FILE *f;
-  vector<char *>::iterator i;
+int FileFinder::find(char *name,char *fullpath) {
+  int i,j;
   char fullname[512];
-  f=::fopen(name,mode);
+
+  ifstream f(name);
   if (!f) {
-    for(i=path.begin();i!=path.end();i++) {
-      strcpy(fullname,*i);
+    j=path.size();
+    for(i=0;i<j;i++) {
+      strcpy(fullname,path[i].c_str());
       strcat(fullname,"/");
       strcat(fullname,name);
-      f=::fopen(fullname,mode);
-      if (f!=NULL) return f;
+      ifstream g(fullname);
+      if (g) {
+	strcpy(fullpath,fullname);
+	g.close();
+	return 1;
+      }
     }
-    return NULL;
+    return 0;
   }
-  return f;
+  f.close();
+  strcpy(fullpath,name);
+  return 1;
 }
 
 EboardFileFinder::EboardFileFinder() {
